#include "DLImage.h"
#include "DLDocument.h"
#include "DLPage.h"
#include "DLZone.h"
#include "DLTagList.h"
#include "JScript.h"
#include "DLScriptIDBatchTest.h"
#include "DLTestManager.h"
#include "DLConst.h"
#include "DLxml.h"

#include <stdlib.h>
#include <stdio.h>
#include <string>
#include <vector>
#include <list>
#include <algorithm>
#include <iostream>
#include <fstream>
#include <math.h>

using namespace std;

#define MAX_PATH 260

DLScriptIDBatchTest::DLScriptIDBatchTest(char* name, int argc, char** argv) : DLTest(name, argc, argv)
{
	testName = name;
	dlArgc = argc;
	dlArgv = argv;
}

DLScriptIDBatchTest::~DLScriptIDBatchTest(void)
{
}

void DLScriptIDBatchTest::showHelp()
{
	/* Show command line help information */
	printf( "ScriptID [-x] [-l] filename\n" );
	printf( "   -x  ---  Write classification results into an xml file for each input image.\n" );
	printf( "            It creates a new xml file if no associated xml file exists.\n" );
	printf( "   -l  ---  File containing the list of input images to execute\n" );
	printf( "   -h  ---  Show help at command line\n" );
}

int DLScriptIDBatchTest::go()
{
	char inputFile[MAX_PATH];
	char logFile[MAX_PATH];
	bool bFileList = false;
	bool bOutputXML = false;

	/* Parse command line arguments */
	if(dlArgc == 1)
	{
		showHelp();
		return 0;
	}
    
	for(int i=1; i<dlArgc; i++)
	{
		if(dlArgv[i][0] != '-') break;
		++i;
		switch(tolower(dlArgv[i-1][1]))
		{
			case 'x':
				bOutputXML = true;
				i--;
				break;
			case 'l':
				bFileList = true;
				i--;
				break;
			case 'o':
				strcpy( logFile, dlArgv[i] );
				break;
			case '?':
			case 'h':
				showHelp();
				return 0;
			default:
				printf( "Unable to interprete your command!\n" );
				showHelp();
				return -1;
		}
	}

	/* Show commmand line help panel if incorrect arguments are supplied */
	if( i != dlArgc-1 )
	{
		showHelp();
		return -1;
	}

	/* Grab the input file name specified at the command line */
	strcpy( inputFile, dlArgv[i] );

	vector <string> imageFileList;
	string imageFileName;

	/* Read in the list of images for processing if the list argument in the command line is supplied */
	if(bFileList)
	{
		ifstream infile(inputFile);
		cout << "Batch file - " << string(inputFile) << endl << endl;
		appendToTestLog("Batch file - " + string(inputFile) + "\n");
		if(infile)
		{
			while (!infile.eof())
			{
				imageFileName = "";
				infile >> imageFileName;
				if(imageFileName != "\n" && imageFileName != "")
					imageFileList.push_back(imageFileName);
			}
		}
		infile.close();
	}
	/* Process the single file if the list argument in the command line is empty */
	else
	{
		imageFileList.push_back(static_cast<string>(inputFile));
		cout << "Image file - " << string(inputFile) << endl;
		appendToTestLog("Image file - " + string(inputFile) + "\n");
	}

	DLDocument::DLPagePtrIterator page_Iter;
	string srcImageFullPathName;
	char docCounter[5], pageCounter[5], zoneCounter[5];
	int imageIndex;

	JScript jargonJS(".\\trainedScripts\\language_list.txt");
	jargonJS.setSymbol_count(500);
	jargonJS.setOrientThreshold(100);

	/* Detect logos from input document images and store them */
	for(imageIndex = 0; imageIndex < int(imageFileList.size()); imageIndex++)
	{
		srcImageFullPathName = (string)imageFileList[imageIndex];
		DLImage docImage(srcImageFullPathName.c_str());
		DLDocument currentDoc;

		sprintf(docCounter, "%d", imageIndex+1);
		sprintf(pageCounter, "%d", 1);
		string docID = "Document" + static_cast<string>(docCounter);
		string pageID = "Page" + static_cast<string>(pageCounter);

		bool xmlFileFound = false;
		int fileExtPosition = int(srcImageFullPathName.find("."));
		int dirSepPosition = srcImageFullPathName.find_last_of(static_cast<string>(DL_DIR_SEPARATOR), fileExtPosition);
		string srcImageFileName, xmlFileName, xmlFilePath, xmlFullPathName;

		if(dirSepPosition != -1)	
		{
			srcImageFileName = srcImageFullPathName.substr(dirSepPosition);
			xmlFileName = srcImageFullPathName.substr(dirSepPosition, fileExtPosition);
			xmlFilePath = srcImageFullPathName.substr(0, dirSepPosition);
		}
		else 
		{
			srcImageFileName = srcImageFullPathName;
			xmlFileName = srcImageFullPathName.substr(0, fileExtPosition);
			xmlFilePath = "";
		}
		xmlFileName = xmlFileName + ".xml";
		xmlFullPathName = xmlFilePath + xmlFileName;

		ifstream xmlfile(xmlFileName.c_str());
		if(xmlfile)
		{
			/* Read in the xml file associated with the input document image if the xml file exists */
			xmlFileFound = true;
			cout << "Reading xml file - " << string(xmlFileName) << endl << endl;	
			DLxml xmlInput;
			currentDoc = xmlInput.dlReadFromXmlFile( xmlFileName, xmlFilePath );
			page_Iter = currentDoc.begin();
		}
		else
		{
			/* Create a single page document for storing detection information if there is no associated xml file */
			DLDocument thisDoc(docImage, pageID, docID);
			currentDoc = thisDoc;
			page_Iter = currentDoc.begin();
		}
		xmlfile.close();

		cout << "Identifying script on " << srcImageFileName.c_str() << " ..." << endl;

		string script = static_cast<string>(jargonJS.getScript(docImage));
		char firstLetter = NULL;
		string otherLetters;
		if(!script.empty()) {
			firstLetter = script.at(0);
			if((firstLetter >= 97) && (firstLetter <= 122))
			{
				firstLetter = firstLetter - 32;
				script.at(0) = firstLetter;
			}
		}
		cout << script << endl;

        jargonJS.enableAllScripts();
		(*page_Iter)->dlSetTag("PrimaryLanguage", script, true);

		/* Update existing xml file or create a new xml file containing the detection results */
		if(bOutputXML)
		{
			DLxml xmlOutput(currentDoc, srcImageFileName);	
			xmlOutput.dlSaveToXmlFile(xmlFullPathName);
			if(xmlFileFound) {
				cout << string(xmlFullPathName) << " updated" << endl << endl;
			}
			else {
				cout << string(xmlFullPathName) << " created" << endl << endl;
			}
		}
	}

	cout << imageIndex << " images classfied" << endl;
	
	appendToTestLog("\n\n\n******************************************************\n");
	appendToTestLog("**Test Summary\n");
	appendToTestLog("******************************************************\n\n");

	return 0;

}



