#ifndef CLUSTERUTILS_H
#define CLUSTERUTILS_H


#include <stdio.h>
#include "JScript.h"

/***
 * The ClusterUtils class contains tools to manipulate clusters (add to,
 * filter, merge, etc.) perform cluster operations
 */

/// Class providing tools to manipulate clusters
class ClusterUtils
{
   public:
   /// constructor
   ClusterUtils(void);
   /// destructor
   ~ClusterUtils(void);

   /**
    * create_cluster creates a list of clusters from an input image
    * with the hight and width
    *
    * \param image char* pointing to memory buffer
    * \param prev pointing to the previous cluster
    * \param count the number of clusters
    * \param height height of the clusters
    * \param width width of the clusters
    * \return list of clusters
    */
   static jcluster* create_cluster( char *image, jcluster *prev,
                                    int *count,int height, int width);
   /** 
    * Add_to_cluster add a giving image to the cluster list
    * \param cluster list of cluster
    * \param image char* to image buffer
    * \param height height of the clusters
    * \param width width of the clusters
    */
   static int add_to_cluster(jcluster *cluster, char *image,
                             int height, int width);

   /**
    * filter_small_clusters is a filter function to filter out the 
    * small clusters form the cluster list.  
    * 
    * \param first_cluster the begining of the first cluster
    * \param threshold threshold for filter the cluster list
    * \param count the number of cluster after the filter
    * \return pointers to the jcluster list
    */
   static jcluster *filter_small_clusters(jcluster *first_cluster,
                                          int threshold,int *count);

   /**
    * match_to_clusters matches a giving image to a list of clusters
    * 
    * \param image char* to image buffer
    * \param first_cluster the begining of the first cluster
    * \param hamming_number harmming number
    * \param height height of the clusters
    * \param width width of the clusters
    * \return int
    */
   static int match_to_clusters(char *image,jcluster *first_cluster,
                                int hamming_number,int height,int width);

};



#endif
