#ifndef _JMORPOHOLOGY_H_
#define _JMORPOHOLOGY_H_

#include <string.h>


#define FLIP_IMAGE_THRESHOLD .03


typedef struct _cluster *jcluster_ptr;

/** _cluster contains information relating to a particular cluster
 **/

/// cluster related information 
typedef struct _cluster {
   /** array containing the first component used to form the cluster*/
   char *first_image;
   /** array representing the centroid of a cluster */
   int *centroid;
   /* number of components used to form the cluster*/
   int count;
   /* number of components that match this component and were of
    * this components script (used in computing reliabililty) 
    */
   int correct_matches;    
   /* number of components that match this component and were not of
    * this components script (used in computing reliabililty) 
    */
   int incorrect_matches; 
   /* Number, between 0 and 1, representing how unique the symbol is 
    * to the script 
    */
   float reliability;
   jcluster_ptr prev;
   jcluster_ptr next;
   unsigned long similar_symbol_count;
   int matches_per_image;
   int on_pixels;
   /** densities stores the number of black pixels in each of the 4 quadrants.
    * It is used to speed up processing when finding the best match for a symbol 
    */
   int densities[4];
   int cluster_number;
   char spare[18];
} jcluster;

/// structure containing information on a script 

typedef struct i_language_profile {
   /** string indicating the name of a script 
    */
   char identification[21];
   /** number of clusters formed for a script 
    */
   int cluster_count;
   /* number of clusters remaining after they are
    * filtered 
    */

   int filtered_cluster_count;
   /* number of training images used to form clusters for 
    * the script 
    */
   int file_count;
   /** hamming distance threshold being used 
    */
   int hamming;
   /** total number of components processed during training
    */
   int symbol_count;
   /** Threshold used in filtering clusters.  Indicates the minimum
    * number of symbols used to form a cluster 
    */
   int small_cluster_threshold;
   /** Minimum size of a connected component to be used in training 
    */
   int min_size;
   /* Maximum height of a connected component to be used in training
    */
   int max_height;
   /* Maximum width of a connected component to be used in training
    */
   int max_width;
   int total_symbol_count;
   /* Number of symbols correctly match to this script during the
    * reliability calculation 
    */
   int correctly_matched;
   /* Number of symbols incorrectly match to this script during the
    * reliability calculation 
    */
   int incorrectly_matched;
   unsigned int hamming_sum;
   /** hamming distance for the best matching symbol 
    */
   unsigned int best_hamming;
   float matched_sum;
   unsigned int *cluster32;
   char spare[18];
   char *cluster_data;
   jcluster *first_cluster;
   char *prev;
   char *next;
} language_profile;

#include "DLImage.h"
#include "DLConnectedComponents.h"
#include "JargonTBRoutines.h"

///Class containing routines call by JScript in performing script identification
/** Class containing routines call by JScript in performing script identification
 * It should eventually be combined with another class
 */
class JMorphology {
//	friend class JScript;
   public:
//private:
   /** Converts a 1 byte/pixel to 1 bit/pixel representation 
    */
   static void compress_to_bit(unsigned char *byte_array, unsigned char *destination,
                               int bits_needed, int bytes_needed);

   static int ConvertData32Bit(language_profile* first_lang,
                               int iPixelRow, int iPixelCol,
                               unsigned int** uippNewData);


	
   static int  get_mean_and_stddev( int     local_directions[4], 
                                    float * scores[4],
                                    int *   prob,
                                    int     language_count,
                                    int     u_symbol_count,
                                    float   m_array[4],
                                    float   s_array[4],
                                    float * high_stddev,
                                    float * second_stddev,
                                    int *   second);

   /** Main routine called to calculate script and orientation for an image
    */
   static int identify_and_orient(
      int hamming_number,
      int min_component_size, int max_height, int
      max_width, int min_height, int min_width,
      int min_symbol_count,
      float *blob_percent, 
      int symbolcount,
      int rescale_height, int rescale_width,
      int upper_offset,
      int lower_offset, int left_offset, int right_offset,
      float *reliability_threshold, 
      DLImage & strDocImage,   //	TBdocument * strDocument,
      int language_count, language_profile * first_lang,
      char *script, int *used_symbol_count,
      int *file_symbol_count,
      int *filtered_symbol_count,
      float *randoms, int max_random_index,
      float *separation_threshold, char *best_fit_script,
      char *second_best_script,
      float *stddev, float *separation, int collect_data_flag,
      float *symbol_distribution,
      unsigned int *hamming_lookup, int *prob,
      float *mlp_thresh,
      char *orientation, int *directions,
      int orient_det_threshold);

   static void ConvertUSCInt(unsigned int* ui, unsigned char* image);

   static void freeLanguageProfileStruct(language_profile * langProfile, int langCount);


}; //end class
#endif //_JMORPOHOLOGY_H_
