#include "JScript.h"
#include "DLLogger.h"
#include "DLBitsPerPixelConverter.h"
#include "DLVersions.h"

void JScript::ReadScriptData(char *language_list_path, language_profile **language_array, int *language_count, int scaled_height, int scaled_width, float *threshold, int **gray_values)
{
   int max_gray = 0;
   int char_count;
   FILE *language_fp;
   FILE *script_data_fp;
   int result;
   char language_string[MAX_FILENAME_LEN+1];
   char script_data_string[MAX_FILENAME_LEN+1];
   language_profile *language_ptr;
   int bits_needed;
   int bytes_needed;
   unsigned char *byte_array;
   int *int_array;
   int i;
   int j;
   int k;
   jcluster *cluster_ptr;
   unsigned char *data_ptr;
   int *test_array;
   int h_half;
   int w_half;
   int gray_index = 0;
   int *gray_ptr = 0;

   /* float values[13] = {.9,.45,.7,.35,.425,.3,.7,.15,.7,.125,.2,.67,.74}; */
   float values[13] = {.65, .65, .65, .65, .4, .65, .65, .65, .55, .15, .65, .65, .65}; 
   int iCounter;

   /* open file and compute number of languages */

   language_fp = fopen(language_list_path,"r");
   if (language_fp == NULL)
   {
      dlLogger.dlLog(DL_CRITICAL, "Could not open list of scripts: %s \n", language_list_path);
      DLException(DL_IO_EXCEPTION, "<JScript::ReadScriptData()> Could not open script list file.");

   }

   result = fscanf(language_fp,"%s %s",language_string, script_data_string);
   *language_count = 0;
   while (result > 0)
   {
      *language_count = *language_count+1;
      result = fscanf(language_fp,"%s %s",language_string, script_data_string);

   }


   /* allocate room for language array */

   //*language_array = (language_profile *) calloc (*language_count,
   //	sizeof(language_profile));
   *language_array = new language_profile[*language_count];

   memset(*language_array, 0, *language_count * sizeof(language_profile));

   language_ptr = *language_array;

   /* if needed allocate room to store the gray scale values.  Need
      to monitor to make sure we don't run out of room */


   bits_needed = scaled_height*scaled_width;
   if (gray_values != NULL)
   {
      //*gray_values = (int *) calloc(*language_count*
      //	2000*bits_needed,
      //	sizeof(int));
      *gray_values = new int[*language_count*2000*bits_needed];

      max_gray = *language_count*2000*bits_needed;
      gray_ptr = *gray_values;
   }

   fseek (language_fp, 0, 0);


   /* allocate slightly more space then needed for the byte
      array to uninitialized memory read errors within the compress
      to bit routine */

   bytes_needed = (int) (ceil((double)bits_needed/8.0));
   //	int_array = (int *) calloc(1,bits_needed*sizeof(int));
   int_array = new int[bits_needed];
   memset(int_array, 0, sizeof(int)* bits_needed);

   test_array = (int *) int_array;
   result = fscanf(language_fp,"%s %s",language_string, script_data_string);
   iCounter = 0;
   while (result > 0)
   {
      script_data_fp = fopen(script_data_string,"r");
      if (script_data_fp == NULL)
      {
         dlLogger.dlLog(DL_CRITICAL, "could not open script data for script: %s",
                        language_string);
         exit(-1);
      }

      /* read language data */

      fseek(script_data_fp,0,SEEK_SET);
      result = fscanf(script_data_fp,"%s %n",
                      language_ptr-> identification,&char_count);

      result = fscanf(script_data_fp,"%d  %n",
                      &language_ptr-> cluster_count,&char_count);

      result = fscanf(script_data_fp,"%d %d %d %d %d ",
                      &language_ptr-> filtered_cluster_count,
                      &language_ptr-> file_count,
                      &language_ptr-> hamming,
                      &language_ptr-> symbol_count,
                      &language_ptr-> small_cluster_threshold);
      result = fscanf(script_data_fp, "%d %d %d %d %d %d ",
                      &language_ptr-> min_size,
                      &language_ptr-> max_height,
                      &language_ptr-> max_width,
                      &language_ptr-> total_symbol_count,
                      &language_ptr-> correctly_matched,
                      &language_ptr-> incorrectly_matched);
      fscanf(script_data_fp,"%u %u %f ",
             &language_ptr->  hamming_sum,
             &language_ptr->  best_hamming,
             &language_ptr-> matched_sum);



      /* allocate room for cluster data structure and a
         array for the centroid */

      //language_ptr ->first_cluster = (jcluster *) 
      //	calloc(language_ptr->cluster_count,
      //	sizeof(jcluster));

      language_ptr->first_cluster = new jcluster [language_ptr->cluster_count];
      memset(language_ptr->first_cluster, 0, language_ptr->cluster_count * sizeof(jcluster));

      //language_ptr ->cluster_data= (char *) 
      //	calloc(language_ptr->cluster_count*bytes_needed,
      //	sizeof(char));

      language_ptr ->cluster_data= new char[language_ptr->cluster_count*bytes_needed];
      memset(language_ptr ->cluster_data, 0, language_ptr->cluster_count*bytes_needed);

      data_ptr = (unsigned char *) language_ptr->cluster_data;
      cluster_ptr = (jcluster *) language_ptr->first_cluster;

      cluster_ptr->densities[0] = 0;
      cluster_ptr->densities[1] = 0;
      cluster_ptr->densities[2] = 0;
      cluster_ptr->densities[3] = 0;
      for (i=0; i< language_ptr->cluster_count; i++)
      {

         /* read cluster info */

         fscanf(script_data_fp,"%d %d %d %f %u %d %d ",
                &cluster_ptr-> count,
                &cluster_ptr-> correct_matches,
                &cluster_ptr-> incorrect_matches,
                &cluster_ptr-> reliability,
                &cluster_ptr-> similar_symbol_count,
                &cluster_ptr-> matches_per_image,
                &cluster_ptr-> on_pixels);

         cluster_ptr->on_pixels = 0;

         for (j=0;j< bits_needed;j++)
         {
            fscanf(script_data_fp,"%d ",&int_array[j]);
            if (gray_values != NULL)
            {
               if (gray_index >= max_gray)
               {//summit type casted to int * here
                  *gray_values = (int *)realloc(*gray_values,
                                                max_gray*2);
                  max_gray = 2*max_gray;
                  gray_ptr = *gray_values;
               }
               gray_ptr[gray_index] = int_array[j];
               gray_index++;
            }
         }

         /*****

         // read and discard the first image 

         //fread(byte_array, bits_needed, 1, script_data_fp);

         // read centroid 

         //	fread(int_array, sizeof(int)*bits_needed, 1, script_data_fp);

         */

         /* threshold at 125 and store in byte array */

         //byte_array = (unsigned char *) calloc(1,bits_needed);
         byte_array = new unsigned char[bits_needed];
         memset(byte_array, 0, bits_needed);

         h_half = scaled_height/2;
         w_half = scaled_width/2;
         for (j=0;j< scaled_height;j++)
         {
            for (k = 0;k< scaled_width;k++)
            {			
               /* 			  if (test_array[j*scaled_height + k] > (int) ((float) cluster_ptr->count * (*threshold))) 			    { */
               if (test_array[j*scaled_height + k] > (int) ((float) cluster_ptr->count * values[iCounter])) {

                  byte_array[j*scaled_height + k] = 0xff;
                  cluster_ptr->on_pixels =  cluster_ptr->on_pixels+1;
                  /* compute density for quadrants */

                  if ((j < h_half) && (k < w_half))
                     cluster_ptr->densities[0] = 
                        cluster_ptr->densities[0] +1;
                  else
                     if ((j < h_half) && (k >= w_half))
                        cluster_ptr->densities[1] = 
                           cluster_ptr->densities[1] +1;
                     else
                        if ((j >= h_half) && (k < w_half))
                           cluster_ptr->densities[2] = 
                              cluster_ptr->densities[2] +1;
                        else
                           cluster_ptr->densities[3] = 
                              cluster_ptr->densities[3] +1;

               }
            }
         }

         /* store the bit data immediately after cluster
            structure */
         //summit calling compress_to_bit based on static Object
         JMorphology::compress_to_bit(byte_array, 
                                      data_ptr,
                                      bits_needed, bytes_needed);

         //free(byte_array);
         delete [] byte_array;
         /* move pointer to next cluster */

         cluster_ptr++;
         data_ptr = data_ptr + bytes_needed;
      }
      language_ptr++;
      iCounter++;
      fclose (script_data_fp);
      result = fscanf(language_fp,"%s %s",language_string, script_data_string);
   }
   /* already freed */
   /* free(byte_array); */
   //free(int_array);
   delete [] int_array;
   fclose (language_fp);
}

void JScript::read_script_data(char *language_list_path, language_profile **language_array, int *language_count, int scaled_height, int scaled_width)
{

   FILE *language_fp;
   FILE *script_data_fp;
   int result;
   char language_string[MAX_FILENAME_LEN+1];
   char script_data_string[MAX_FILENAME_LEN+1];
   language_profile *language_ptr;
   int bits_needed;
   int bytes_needed;
   char *byte_array;
   char *int_array;
   int i;
   int j;
   int k;
   jcluster *cluster_ptr;
   char *data_ptr;
   int *test_array;
   int h_half;
   int w_half;

   /* open file and compute number of languages */

   language_fp = fopen(language_list_path,"r");
   if (language_fp == NULL)
   {
      dlLogger.dlLog(DL_CRITICAL, "Could not open list of scripts: %s \n", language_list_path);
      DLException(DL_IO_EXCEPTION, "<JScript::ReadScriptData()> Could not open script list file.");

   }


   result = fscanf(language_fp,"%s %s",language_string, script_data_string);
   *language_count = 0;
   while (result > 0)
   {
      *language_count = *language_count+1;
      result = fscanf(language_fp,"%s %s",language_string, script_data_string);

   }

   /* allocate room for language array */

   //*language_array = (language_profile *) calloc (*language_count,
   //	sizeof(language_profile));

   //*language_array = (language_profile *) calloc (*language_count,
   //	sizeof(language_profile));
   *language_array = new language_profile [*language_count];


   language_ptr = *language_array;

   fseek (language_fp, 0, 0);

   bits_needed = scaled_height*scaled_width;

   /* allocate slightly more space then needed for the byte
      array to uninitialized memory read errors within the compress
      to bit routine */

   bytes_needed =(int)( ceil((double)bits_needed/8.0));
   //byte_array = (char *) calloc(1,bits_needed);
   //int_array = (char *) calloc(1,bits_needed*sizeof(int));
   byte_array = new char[bits_needed];
   int_array = new char[bits_needed];

   test_array = (int *) int_array;
   result = fscanf(language_fp,"%s %s",language_string, script_data_string);
   while (result > 0)
   {
      script_data_fp = fopen(script_data_string,"r");
      if (script_data_fp == NULL)
      {
         dlLogger.dlLog(DL_CRITICAL, "could not open script data for script: %s",
                        language_string);
         exit(-1);
      }

      /* read language data */

      fread(language_ptr,sizeof(language_profile),1,script_data_fp);


      /* allocate room for cluster data structure and a
         array for the centroid */

      //language_ptr ->first_cluster = (jcluster *) 
      //	calloc(language_ptr->cluster_count,
      //	sizeof(jcluster));
      //language_ptr ->cluster_data= (char *) 
      //	calloc(language_ptr->cluster_count*bytes_needed,
      //	sizeof(char));

      language_ptr ->first_cluster = new jcluster[language_ptr->cluster_count];
      memset(language_ptr->first_cluster, 0, language_ptr->cluster_count * sizeof(jcluster));

      language_ptr ->cluster_data = new char[language_ptr->cluster_count*bytes_needed];
      memset(language_ptr ->cluster_data, 0, language_ptr->cluster_count*bytes_needed);

      data_ptr = (char *) language_ptr->cluster_data;
      cluster_ptr = (jcluster *) language_ptr->first_cluster;

      for (i=0; i< language_ptr->cluster_count; i++)
      {

         /* read cluster info */

         fread(cluster_ptr, sizeof(jcluster), 1, script_data_fp);
         cluster_ptr->on_pixels = 0;

         /* read and discard the first image */

         fread(byte_array,  1,(sizeof(int) + 1)*bits_needed, script_data_fp);

         /* read centroid */

         /* fread(int_array, sizeof(int)*bits_needed, 1, script_data_fp); */

         /* threshold at 125 and store in byte array */

         h_half = scaled_height/2;
         w_half = scaled_width/2;
         for (j=0;j< scaled_height;j++)
         {
            for (k = 0;k< scaled_width;k++)
            {
               if (test_array[j*scaled_height + k] > (int) (cluster_ptr->count/2))
               {
                  byte_array[j*scaled_height + k] = 0xff;
                  cluster_ptr->on_pixels =  cluster_ptr->on_pixels+1;
                  /* compute density for quadrants */

                  if ((j < h_half) && (k < w_half))
                     cluster_ptr->densities[0] = 
                        cluster_ptr->densities[0] +1;
                  else
                     if ((j < h_half) && (k >= w_half))
                        cluster_ptr->densities[1] = 
                           cluster_ptr->densities[1] +1;
                     else
                        if ((j >= h_half) && (k < w_half))
                           cluster_ptr->densities[2] = 
                              cluster_ptr->densities[2] +1;
                        else
                           cluster_ptr->densities[3] = 
                              cluster_ptr->densities[3] +1;

               }
               /*
                 else
                 byte_array[j*scaled_height + k] = 0x00;
               */
            }
         }

         /* store the bit data immediately after cluster
            structure */

         //summit made call to static object 
         JMorphology::compress_to_bit((unsigned char*) byte_array, 
                                      (unsigned char*)data_ptr,
                                      bits_needed, bytes_needed);

         /* move pointer to next cluster */

         cluster_ptr++;
         data_ptr = data_ptr + bytes_needed;
      }
      language_ptr++;
      fclose (script_data_fp);
      result = fscanf(language_fp,"%s %s",language_string, script_data_string);
   }
   //free(byte_array);
   //free(int_array);
   delete[] byte_array;
   delete[] int_array;
   fclose (language_fp);
}

JScript::JScript(char *language_list_file)  // class instantitation
{
   language_profile *lang_ptr;
   int               j;
   int               k;


   // symbol_count = 200; Old Default Value
   setSymbol_count(500);      // Establish default value  07/14/04

   setOrientThreshold(100);   // Establish default value 07/14/04 (old value was effectively 50, used <= 100)

   symbols_used_for_id_count = 0;

   rescale_height = 20;
   rescale_width = 20;
   min_comp_size = 20;
   max_height = 500;
   max_width = 500;
   min_height = 6;
   min_width = 6;
   min_symbol_count = 20;
   blob_percent = 0.75;     // 75 %
   upper_offset = 100;
   lower_offset = 10;
   left_offset = 10;
   right_offset = 10;
   hamming = 111;

   reliability_threshold = static_cast <float> (.8);
   separation_threshold  = 0.75;
   template_threshold    = static_cast <float> (.67);
   mlp_thresh            = static_cast <float> (.6);

   for (k=0; k<4; k++)
      directions[k]= 1;

   // read jargon template data, determine language_count,
   //   and allocate first_lang
   ReadScriptData(language_list_file, &first_lang, &language_count,
                  rescale_height, rescale_width, &template_threshold,NULL);

   j= language_count;
   if (j<13)
      j= 13;   // allocation size base, minimum of 13

   symbol_distribution = new float [j];   // allocate, minimum of 13
   for (k=0; k<j; k++)
      symbol_distribution[k] = 0.0;

   probability = new int [j];             // allocate, minimum of 13
   enableAllScripts();

   lang_ptr = first_lang;
   for (k = 0; k < language_count; k++)
   {
      /* need to call this x times for each lang */
      JMorphology::ConvertData32Bit(lang_ptr, rescale_height, rescale_width, &lang_ptr->cluster32);

      lang_ptr++;   // advance to the next language 
   }

   randoms = new float [5000];
   memset(randoms, 0, 5000*sizeof(float));

   JStatistics::generate_random_number_array(randoms, 5000);

   hamming_lookup = new int [65536];
   memset(hamming_lookup, 0, sizeof(int)*65536);
   JHamming::CreateHammingLUT32(hamming_lookup,65536);
}

// ************************************************************************** //

JScript::~JScript()              // destructor
{
   //int k;
   //language_profile *lang_ptr;


   delete []randoms;
   delete []hamming_lookup;

   delete [] symbol_distribution;
   delete []  probability;
   JMorphology::freeLanguageProfileStruct(first_lang, language_count);
   delete [] first_lang;

   //	lang_ptr= first_lang;
   //	for (k=0; k<language_count; k++)
   //	{
   //		// for each language
   //		free(lang_ptr->cluster32);
   //free(lang_ptr->cluster_data);
   //free(lang_ptr->first_cluster);

   //       delete[] lang_ptr->cluster32;
   //		delete[] lang_ptr->cluster_data;
   //		delete[] lang_ptr->first_cluster;

   //		lang_ptr++; 
   //	}
   //
   ////	//free(first_lang);
   //  delete[] first_lang;

}

// ************************************************************************** //

void JScript::changeBorders( int top, int left, int bottom, int right)
{
   upper_offset= top;
   lower_offset= bottom;
   right_offset= right;
   left_offset= left;
}

// ************************************************************************** //

void JScript::clearDistribution()
{
   int k;

   for (k=0; k<language_count; k++)
      symbol_distribution[k]= 0.0; 
}

// ************************************************************************** //

void JScript::disableScript(int script_no)
{
   if (script_no < language_count)
      probability[script_no]= 0; 
}

// ************************************************************************** //

void JScript::enableScript(int script_no)
{
   if (script_no < language_count)
      probability[script_no]= 1; 
}

// ************************************************************************** //

void JScript::enableAllScripts()
{
   int k;

   for (k=0; k<language_count; k++)
      probability[k]= 1; 
}

// ************************************************************************** //

void JScript::setDirections(int newdirections[4])
{
   int i;

   for (i=0; i<4; i++)
      directions[i]= newdirections[i];
}
// ************************************************************************** //

void JScript::setSymbol_count(int symbol_limit)
{
   symbol_count= symbol_limit;
}
// ************************************************************************** //

void JScript::setOrientThreshold(int thresh)
{
   orient_threshold= thresh;
}

// ************************************************************************** //

char *JScript::getScript(DLImage& testImageP)
{
   float           float_separation;    

   float           stddev;
   int             symbols_used_for_id;
   int             symbols_scanned;
   int             symbols_filtered;


   if(testImageP.dlGetDepth() != DL_BITDEPTH_1)
   {
	   testImageP = DLBitsPerPixelConverter::dlConvertImage(testImageP, DLBitsPerPixelConverter::DL_2BINARY);
   }

   clearDistribution();   // initialize symbol_distribution to zeroes 

   JMorphology::identify_and_orient(hamming, min_comp_size, max_height,
                                    max_width, min_height, min_width, min_symbol_count,
                                    &blob_percent, symbol_count, rescale_height,
                                    rescale_width, upper_offset, lower_offset,
                                    left_offset, right_offset, &reliability_threshold,
                                    testImageP, 
                                    language_count, first_lang,
                                    script_id,
                                    &symbols_used_for_id,
                                    &symbols_scanned, &symbols_filtered,
                                    randoms, 5000, &separation_threshold, best_fit_script,
                                    second_best_script, &stddev, 
                                    &float_separation,
                                    1,
                                    symbol_distribution,
                                    (unsigned int *) hamming_lookup, 
                                    probability,
                                    &mlp_thresh, orient_char,
                                    directions,
                                    getOrientThreshold());

   _orientation= atoi(orient_char);

   this->symbols_used_for_id_count = symbols_used_for_id;

   return (&script_id[0]); 
}

int jarGetScriptNo(char *script)
{
   int result;

   if (strcmp(script,"amharic") == 0)
      result = 0;
   else if  (strcmp(script,"arabic") == 0)
      result = 1;
   else if  (strcmp(script,"armenian") == 0)
      result = 2;
   else if  (strcmp(script,"burmese") == 0)
      result = 3;
   else if  (strcmp(script,"chinese") == 0)
      result = 4;
   else if  (strcmp(script,"cyrillic") == 0)
      result = 5;
   else if  (strcmp(script,"devanagari") == 0)
      result = 6;
   else if  (strcmp(script,"greek") == 0)
      result = 7;
   else if  (strcmp(script,"hebrew") == 0)
      result = 8;
   else if  (strcmp(script,"japanese") == 0)
      result = 9;
   else if  (strcmp(script,"korean") == 0)
      result = 10;
   else if  (strcmp(script,"latin") == 0)
      result = 11;
   else if  (strcmp(script,"thai") == 0)
      result = 12;
   else if  (strcmp(script,"nodata") == 0)
      result = 13;
   else  
      result = 14;
   return(result);
}

string JScript::dlGetVersionInfo()
{
   string a = DLVersions::dlGetAllVersionInfo() + "\nJARGON: " + DL_JARGON_VERSION + "\n";

   return a;
}
