#include "JargonReliability.h"
#include <stdlib.h>
#include "DLLogger.h"

#ifndef WIN32
#include <sys/types.h>
#include <sys/stat.h>
#else
#include <direct.h>
#endif

JargonReliability::JargonReliability(void)
{
}

JargonReliability::~JargonReliability(void)
{
}



/**
 * Tokenize takes a string and split the string by a token
 * \param str string to tokenize
 * \param tokens stores the result of tokens
 * \param delimiters delimiter use to divide the string
 **/
void JargonReliability::Tokenize(const string& str,
                                 vector<string>& tokens,
                                 const string& delimiters)
{
   // Skip delimiters at beginning.
   string::size_type lastPos = str.find_first_not_of(delimiters, 0);
   // Find first "non-delimiter".
   string::size_type pos     = str.find_first_of(delimiters, lastPos);

   while (string::npos != pos || string::npos != lastPos)
   {
      // Found a token, add it to the vector.
      tokens.push_back(str.substr(lastPos, pos - lastPos));
      // Skip delimiters.  Note the "not_of"
      lastPos = str.find_first_not_of(delimiters, pos);
      // Find next "non-delimiter"
      pos = str.find_first_of(delimiters, lastPos);
   }
}



bool JargonReliability::copyFile(string org, string dest)
{
   bool hasError = false;
   ifstream infile(org.c_str(), ios::in | ios::binary);
   ofstream outfile(dest.c_str(), ios::out | ios::binary);


   if(!infile)
   {
      dlLogger.dlLog(DL_CRITICAL, "Can not open %s\n",org);
      hasError = true;
   }
   if(!outfile)
   {
      dlLogger.dlLog(DL_CRITICAL, "Can not open %s\n", dest);
      hasError = true;
   }
   char c1;
   //bool charEqual = true;

   if(!hasError)
   {
      while(!infile.eof())
      {
         infile.get(c1);
         outfile.put(c1);
      }
      infile.close();
      outfile.close();
   }

   return true;
}

bool JargonReliability::makeDirectory(string dir)
{
   bool result = false;

   vector<string> tokens;
   string dirSept = "/";
   Tokenize(dir, tokens, "/");

   if(tokens.size() <= 1)
   {   
      tokens.clear();
      dirSept = "\\";
      Tokenize(dir, tokens, "\\");
   }

   string dirs = "";// = (string)tokens[0];
   for(int idx = 0; idx < (int)tokens.size(); idx ++)
   {
      dirs = dirs + (string)tokens[idx];
#ifndef WIN32
      //result =  mkdir(dirs.c_str(),774);
      string cmd = "mkdir "  + dirs;
      system(cmd.c_str());

#else
      int val = mkdir(dirs.c_str());
      if(val == 0)
         result = 0;
      else 
         val = 1;

#endif
      dirs = dirs+dirSept;
   }

   return result;
}
void 
JargonReliability::compute_reliability(char *file_list_path, char *language_list_path,
                                       int max_height, int max_width, int min_component_size,
                                       int scaled_height,
                                       int scaled_width,
                                       char *reliability_directory_path, char *identifier,
                                       int *hamming_lookup)
{
   FILE           *language_list_fp;
   FILE           *file_list_fp;
   //FILE           *image_fp;
   //FILE           *script_data_file;
   FILE           *reliability_fp;
   //FILE           *shape_file;

   //float           hamming_penalty;
   float           float_result = 0;
   char            language_string[500];
   char            file_string[500];
   char            script_data_path[500];
   //char            computed_lang[50];
   //char            shell_command[500];
   char            string_val[500];
   int             best_cluster_no;
   //int dummy_int;
   int gray_index = 0;


   language_profile *curr_lang;
   language_profile *first_lang;
   //language_profile *prev_lang;
   language_profile *best_fit_language;
   language_profile *language_ptr;

   //language_results *first_lang_results;
   //language_results *lang_results;
   //language_results *prev_lang_results;

   jcluster       *cluster_ptr;
   //jcluster       *best_fit_cluster;
   //int             system_result;
   //int             process_flag;
   //int             shape_result;
   //int             pixels, upper, lower, left, right, min_size;
   //int             symbol_count;

   int             result;
   //int             return_code;
   int             file_symbol_count;
   int *gray_values;


   //int             w, h, d, t, e;
   //int             close_result;
   /* PointLong upper_left; */
   int             language_count = 0;
   int             bits_needed = 0;
   int             bytes_needed;
   int             i;
   int             j;
   int             on_pixels = 0;
   //DLCCdocument      doink;
   //int             errorTwoBit;
   int            *densities = 0;
   // unsigned char  *uscResized;
   unsigned char  *uscCompressed;
   int            *usiCompressed;
   //int             iCount;
   float           template_threshold;
   int             k;
   language_profile *lang_ptr;

   template_threshold = .5;

   bits_needed = scaled_height * scaled_width;	/* pixels in image */
   bytes_needed = (int)(ceil((double) bits_needed / 8.0));	/* bytes needed to store
                                                                 * bit image */
   /* read_script_data(language_list_path, &first_lang,&language_count,
    * scaled_height, scaled_width); */
   JScript::ReadScriptData(language_list_path, &first_lang, &language_count,
                           scaled_height, scaled_width, &template_threshold,&gray_values);


   lang_ptr = first_lang;
   for (k = 0; k < language_count; k++)
   {
      /* need to call this x times for each lang  */
      JMorphology::ConvertData32Bit(lang_ptr, scaled_height, scaled_width, &lang_ptr->cluster32);

      /* advance to the next language  */
      lang_ptr++;
   };


   /* at this point all template data has been read in */

   /* be careful on this call in NT */
   file_list_fp = fopen(file_list_path, "r+");
   if (file_list_fp == NULL)
   {
      dlLogger.dlLog(DL_CRITICAL, "couldn't open list of training files\n");
      exit(-1);
   }
   result = fscanf(file_list_fp, "%s %s ", file_string, language_string);

   while ((strlen(file_string) != 0) && (result == 2))
   {
      dlLogger.dlLog(DL_INFO, "file: %s script: %s\n", file_string, language_string);

      curr_lang = NULL;
      language_ptr = first_lang;
      while (curr_lang == NULL)
      {
         if (strcmp(language_ptr->identification, language_string)
             == 0)
            curr_lang = language_ptr;
         language_ptr++;
      }

      /* allocate and associate */

      DLImage image(file_string);

      DLConnectedComponents cc;
      cc.dlCreateConnectedComponents(image);


      cc.dlFilterCCHeightBetween( min_component_size,  max_height);
      cc.dlFilterCCWidthBetween( min_component_size,  max_width);

      file_symbol_count = 0;

      // ** need to loop through all components!
      vector <DLComponent> ccList = cc.dlGetConnectedComponents();
      vector<DLComponent>::iterator iterf = ccList.begin();
      for(; iterf != ccList.end(); iterf++) 
      {	
         DLComponent component = (DLComponent)*iterf;
         curr_lang->total_symbol_count++;

         unsigned char  *uscResized = NULL;
         /* extract the component, and resize */

         JargonTBRoutines::tb_GetResizedComponent(&component, &uscResized, scaled_width, scaled_height);
         //int error = 
         JargonTBRoutines::tb_CalcQuadrantMasses(uscResized,  scaled_width, scaled_height, &densities);

         best_fit_language = first_lang;
         best_cluster_no = 1;

         delete[] densities;
         /* as long as we are using a square this is ok, will need to change row
          * and width, depending on if 90/270 or 180 */
         on_pixels = JargonTBRoutines::tb_CalcQuadrantMasses(uscResized,  scaled_height, scaled_width, &densities);

         JargonTBRoutines::tbCompressUCArray(uscResized, &uscCompressed, scaled_width, scaled_height);


         /* if usign 32 bit need to convert to int */
         /* allocate memory for the conversion */

//             usiCompressed = (int *) calloc(13, sizeof(int));
         usiCompressed = new int[13];
         for(int i = 0; i < 13; i++)
         {
            usiCompressed[i] = 0;
         }
         JargonUtils::ConvertUSCInt((unsigned int*)usiCompressed, uscCompressed);

         JStatistics::GetBestFitAllWithDens32(first_lang,
                                              &best_fit_language, &best_cluster_no,
                                              language_count, (unsigned int*)usiCompressed, bytes_needed,
                                              bits_needed, NULL,
                                              (unsigned int*)hamming_lookup, on_pixels, densities, &float_result);

         /* update counts for language */
         cluster_ptr = (jcluster *) best_fit_language->first_cluster;
         cluster_ptr = cluster_ptr + best_cluster_no;
         if (strncmp(language_string, best_fit_language->identification,
                     strlen(language_string)) == 0)
         {
            best_fit_language->correctly_matched++;
            cluster_ptr->correct_matches++;
         } else
         {
            best_fit_language->incorrectly_matched++;
            cluster_ptr->incorrect_matches++;
         }
         /* increment the counter */
         // iList++;
         //file_symbol_count++;
         //free(usiCompressed);
         //free(densities);
         //free(uscResized);
         //free(uscCompressed);

         delete[] usiCompressed;
         delete[] densities;
         densities = NULL;
         delete[] uscResized;
         delete[] uscCompressed;
      }
      result = fscanf(file_list_fp, "%s %s ", file_string, language_string);
   }

   /* compute the reliability number for each cluster */
   language_ptr = first_lang;
   for (j = 0; j < language_count; j++)
   {
      cluster_ptr = (jcluster *) language_ptr->first_cluster;
      for (i = 0; i < language_ptr->cluster_count; i++)
      {
         if ((cluster_ptr->correct_matches > 0) &&
             (cluster_ptr->incorrect_matches > 0))
            cluster_ptr->reliability = (float) cluster_ptr->correct_matches /
               ((float) cluster_ptr->correct_matches + (float) cluster_ptr->incorrect_matches);
         else if (cluster_ptr->correct_matches > 0)
            cluster_ptr->reliability = 1.0;
         else if (cluster_ptr->correct_matches == 0)
            cluster_ptr->reliability = 0.0;
         cluster_ptr++;
      }
      language_ptr++;
   }

   /* store the clusters (with reliability) to the indicated directory */

   /* for simplicities sake copy the existing file to the target and then just
    * rewrite the language and cluster pointer structures (not the clusters
    * themselves) */

   language_list_fp = fopen(language_list_path, "r");
   if (language_list_fp == NULL)
   {
      dlLogger.dlLog(DL_CRITICAL, "could not open list of scripts");
      exit(-1);
   }
   gray_index = 0; 
   language_ptr = first_lang;
   result = fscanf(language_list_fp, "%s %s", language_string, script_data_path);
   while (result > 0)
   {
      char dest_path [500];
      sprintf(string_val, "%s%s/%dx%dreliability.%s",
              reliability_directory_path, language_string,
              scaled_height, scaled_width, identifier);

      sprintf(dest_path, "%s%s",reliability_directory_path, language_string);

      makeDirectory(dest_path);
      copyFile(script_data_path, string_val);

      sprintf(string_val, "%s%s/%dx%dreliability.%s",
              reliability_directory_path, language_string,
              scaled_height, scaled_width, identifier);

      reliability_fp = fopen(string_val, "r+");
      if (reliability_fp == NULL)
      {
         dlLogger.dlLog(DL_CRITICAL, "couldn't open output reliability file(%s) for %s\n",
                        string_val,
                        language_string);
         exit(-1);
      }
      /* overwrite the language structure */
      fprintf(reliability_fp,"%s %d %d %d %d %d %d %d %d %d %d %d %d ",
              language_ptr-> identification,
              language_ptr-> cluster_count,
              language_ptr-> filtered_cluster_count,
              language_ptr-> file_count,
              language_ptr-> hamming,
              language_ptr-> symbol_count,
              language_ptr-> small_cluster_threshold,
              language_ptr-> min_size,
              language_ptr-> max_height,
              language_ptr-> max_width,
              language_ptr-> total_symbol_count,
              language_ptr-> correctly_matched,
              language_ptr-> incorrectly_matched);
      fprintf(reliability_fp,"%u %u %f ",
              language_ptr->  hamming_sum,
              language_ptr->  best_hamming,
              language_ptr-> matched_sum);


      dlLogger.dlLog(DL_DEBUG, "file pointer after language structure write: %d\n",
                     ftell(reliability_fp));
      cluster_ptr = (jcluster *) language_ptr->first_cluster;
      for (i = 0; i < language_ptr->cluster_count; i++)
      {
         /* write the cluster structure */
         fprintf(reliability_fp,"%d %d %d %f %u %d %d ",
                 cluster_ptr-> count,
                 cluster_ptr-> correct_matches,
                 cluster_ptr-> incorrect_matches,
                 cluster_ptr-> reliability,
                 cluster_ptr-> similar_symbol_count,
                 cluster_ptr-> matches_per_image,
                 cluster_ptr-> on_pixels);
//            cout << "i: " << cluster_ptr->reliability << endl;

         for (j =0; j< 400; j++)
         {
            fprintf(reliability_fp,"%d ",gray_values[gray_index]);
            gray_index++;

         }
         cluster_ptr++;
      }
      fclose(reliability_fp);

      language_ptr++;
      result = fscanf(language_list_fp, "%s %s", language_string, script_data_path);

   }
   fclose(file_list_fp);

   JMorphology::freeLanguageProfileStruct(first_lang, language_count);
   delete [] first_lang;
   delete[] gray_values;
    
}
