#ifndef JARGONRELIABILITY_H
#define JARGONRELIABILITY_H

#include "JMorphology.h"
#include "JStatistics.h"
#include "JScript.h"

#include "DLConnectedComponentsUtils.h"
#include "DLImage.h"
#include "DLConnectedComponents.h"
#include "DLComponent.h"
#include "JargonUtils.h"

#include <vector>
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <iostream>
#include <fstream>
#include <vector>
#include <string>
using namespace std;
/***
 * Jargon Reliability class is used to computer the "reliability" metric for each cluster
 * formed during the training (cluster formation) process.  The reliability number is between
 * 0 and 1 and serves as a measure for how unique a particular cluster is to a script.  
 * output.
 */

/// Contains all methods used to compute the reliability metric for clusters.

class JargonReliability
{
   public:
   ///Constructor
   JargonReliability(void);
   /// Destructor
   ~JargonReliability(void);

   /**
    * compute_reliability is the main method used to compute the 
    * reliability for clusters formed during the training process
    * /param file_list_path a filename to a file contains a list of images used in training
    * /param language_list_path a filename to a file which contains the list of the output of the training application
    * /param max_height maximum height of connected components used during both the training and reliability processes
    * /param max_width maximum width of connected components used during both the training and reliability processe
    * /param min_component_size minimum component size used during training and reliability computation
    * /param reliability_directory_path  directory where output files will be stored
    * /param identifier string used to identify output generated
    * /param hamming_lookup 
    */ 
   static void compute_reliability(char *file_list_path, char *language_list_path,
                                   int max_height, int max_width, int min_component_size,
                                   int scaled_height, int scaled_width,
                                   char *reliability_directory_path, char *identifier,
                                   int *hamming_lookup); 
   private:

   /**
    * makeDirectory creates a directory that is used to store the output reliability files 
    * /param dir directory path to make the directory
    * /return bool success or failed
    */
   static bool makeDirectory(string dir);
   /**
    * Tokenize takes a string and split it into tokens
    * \param str string to tokenize
    * \param tokens vector storing the result of tokenization
    * \param delimiters delimiter use to divide the string
    **/
   static void Tokenize(const string& str,
                        vector<string>& tokens,
                        const string& delimiters);

   /**
    * CopyFile is a function to copy a file
    * \param org filename to source file
    * \param dest filename to target file
    * \return status fail/success
    **/
   static bool copyFile(string org, string dest);
};

#endif
