#include "DLLogger.h"

#include "JargonTBRoutines.h"

int JargonTBRoutines::iLUT[8] = {128,64,32,16,8,4,2,1};
int JargonTBRoutines::tb_CalcLineOrientation(DLImage &dimage, vector<DLComponent> * activecc, int iNumber, int iPctDiff) 
{ 

   int     max_node_weight;

   int      i,ii,j,jj;        /* various pointers */
   int      nrow;             /* number rows with CC seed points */
   int      *frow;            /* list of first CC on rows    */
   TB_stats stat;             // previously TBstats
   TB_nodes *N;               // previously TBnodes
   float    dx,dy;

   // float    dd;
   int      dd;

   int      comp_count;      // corresponds initial iComponentCount
   int      adj_ccount;      // number of components less 1
   bool     iterateagain;
   int      p_row_max;
   int      save_rmax_height;   // save CC datum

   int      xy_dist_lim_sq= (INTMAXDISTBETWEENCOMP * (INTMAXDISTBETWEENCOMP-1) +1); 


   if (activecc == NULL)
      return( -2 );

   // Establish number of components to process as mimimum of the number
   // of active components in the activecc vector and the iNumber value
   comp_count = (*activecc).size();
   if (comp_count > iNumber) 
      comp_count = iNumber; 

   /* check to make sure we have enough components */
   if (comp_count < (MINCOMP-1))
      return( -2 );

   adj_ccount = comp_count -1;    // skip one component

   /* PUT ONLY REASONABLY SIZED CC'S INTO LIST */

   /*********************************************************************/
   /* get index of first component on each row of image with CC's */
   // frow = (int*)calloc(1+LIST.iRows,sizeof(int));
   frow = new int [adj_ccount];   // Allocate elements for frow
   if (frow == NULL)
      return (INSUFFICIENT_MEMORY);


   p_row_max = -1;

   vector<DLComponent>::iterator iterf1 = (*activecc).begin();

   /* get index of first component on each row of image with CC's */

   // Note - original logic did not utilize the first 
   //        component even after Filtering, so
   //        adjust iterf1 to skip the first component.
   for (iterf1= iterf1+1, nrow=0, i=1; (i<adj_ccount) && (iterf1 != (*activecc).end()); i++, iterf1++)
   {
      DLComponent cpnt = (DLComponent) *iterf1;

      if(cpnt.dlGetRowMax() != p_row_max)
      {
         p_row_max = cpnt.dlGetRowMax();
         frow[nrow++] =i;
      }
   }
   frow[nrow] = adj_ccount;     

   i = 1;
   while (frow[i] != adj_ccount) {
      if (frow[i] <= frow[i-1])
         printf("<=\n");
      i++;
   }

   /*********************************************************************/
   /* allocate enough memory to hold links to hto_edges of all lengths */

   // max_node_weight = 1+ceil(sqrt(2.0*MAXDISTBETWEENCOMP*MAXDISTBETWEENCOMP));

   max_node_weight = 1 + static_cast <int> (ceil(sqrt( static_cast <float> (2 * INTMAXDISTBETWEENCOMP * INTMAXDISTBETWEENCOMP))));

   N = new TB_nodes [adj_ccount];  
   if (N == NULL)
   {
      delete []frow;
      return (INSUFFICIENT_MEMORY);
   }

   for(i=0; i<adj_ccount; i++)
   {
      N[i].hnode_wt = max_node_weight;
      N[i].vnode_wt = max_node_weight;
      N[i].hto_edge= 0;
      N[i].hfrom_edge= 0;
      N[i].vto_edge= 0;
      N[i].vfrom_edge= 0;
   }


   /******************************************/
   /* set stopping condition in list of CC's */
   // 
   // Don't want to permanently alter CC data, so save the current value of
   //   last component's Height before changing it to stop value

   vector<DLComponent>::reverse_iterator iterf_r = (*activecc).rbegin();
   DLComponent lcpnt = (DLComponent) *iterf_r;
   save_rmax_height = lcpnt.dlGetRowMax();
   lcpnt.dlSetRowMax(dimage.dlGetHeight() + 10 * INTMAXDISTBETWEENCOMP);  


   /* GET HTO_EDGE DISTANCE BETWEEN NODES AND IF SMALL ENOUGH  */
   /*   SAVE ON  SORTED LIST  */
   for(i=0; i<nrow; i++)
   {  /* loop on the rows with segments */
      /* LOOK AT ADJACENT CC'S ON ROW i */
      for( ii=frow[i] ; ii<frow[i+1]-1 ;ii++ )  
      {
         jj=ii+1;
         dx = (*activecc)[jj].dlGetColCentroid() - (*activecc)[ii].dlGetColCentroid();
         dy = (*activecc)[jj].dlGetRowCentroid() - (*activecc)[ii].dlGetRowCentroid();
         put_on_list_dl(N, ii, jj, dx, dy);
      }

      /* LOOK AT CC'S ON ROW frow(i) AND frow(j) if rows closer than DY */

      j=i+1;
      iterateagain = (j<nrow) && (((*activecc)[frow[j]].dlGetRowMax() - (*activecc)[frow[i]].dlGetRowMax() )< INTMAXDISTBETWEENCOMP); 
      for( ; iterateagain; )
      { 
         /* LOOK AT CC'S ON ROW frow()i AND frow(j) IF CLOSER THAN DX */
         for(ii=frow[i],jj=frow[j] ; (jj<frow[j+1])&&(ii<frow[i+1]) ; )
         {
            dx = (*activecc)[jj].dlGetColCentroid() - (*activecc)[ii].dlGetColCentroid();
            dy = (*activecc)[jj].dlGetRowCentroid() - (*activecc)[ii].dlGetRowCentroid();
            // dd = (float)round(sqrt(dx*dx + dy*dy));
            dd = static_cast <int> (dx*dx + dy*dy +0.5);

            // if(dd<MAXDISTBETWEENCOMP)
            if(dd < xy_dist_lim_sq)
            {
               put_on_list_dl(N, ii,jj, dx, dy);
            }

            if(dx<0)
               jj++;
            else
               ii++;

         } /* ii jj */	

         j++;
         iterateagain = (j<nrow) && (((*activecc)[frow[j]].dlGetRowMax() - (*activecc)[frow[i]].dlGetRowMax() )< INTMAXDISTBETWEENCOMP);
      } /* for iterateagain */
   } /* for i */

   stat.hor_num = 0;   
   stat.ver_num = 0;   

   for(i=1; i<adj_ccount; i++)
   {
      if(N[i].hto_edge != 0)
         stat.hor_num++;
      if(N[i].vto_edge != 0) 
         stat.ver_num++;
   }  

   /* Exit Housekeeping ****************************************/
   /* restore the last component's height value                */ 
   lcpnt.dlSetRowMax(save_rmax_height);  

   //  printf("JTBR put1, put2  nrow  frow[0 1 nr-1 nr]  %d %d %d %d %d %d %d\n", cntr_debug_put1,cntr_debug_put2, nrow, frow[0], frow[1], frow[nrow-1], frow[nrow]);
   //  printf("JTBR hor, ver, sum, dif, Pctdiff counter %7.2f %7.2f %7.2f %7.3f %d %d\n", stat.hor_num, stat.ver_num, stat.hor_num+stat.ver_num, stat.hor_num-stat.ver_num, iPctDiff, cntr_debug);

   /* unallocate data */
   if ( N != NULL)
      delete []N;

   if (frow != NULL)
      delete []frow;

   /* do we have enough links to determine line orientation */
   if ((stat.hor_num < MIN_LINKS) || (stat.ver_num < MIN_LINKS))
      return( -1 );

   /* are the numbers too close */
   if ( static_cast <float> (abs( static_cast <int> (stat.hor_num-stat.ver_num))) / (stat.hor_num+stat.ver_num)*100.0 < static_cast <float> (iPctDiff) )
   {
      return( -2 );
   }

   /* now use the method for determining orientation */
   if(stat.hor_num > stat.ver_num) 
      return( 0 );    // horizontal
   else 
      return( 1 );    // vertical

} /* tb_CalcLineOrientation */



void JargonTBRoutines::put_on_list_dl(TB_nodes *N, int i, int j, float dx, float dy)
/* test thresholds and put of list if a pass */
/* increment pointers to row ii */ 
// 	Callers now provide dx and dy values (this makes it faster)
// 	Added parameters 
//             dx  ccentroid difference, j ccentroid - i ccentroid
//             dy  rcentroid difference, j rcentroid - i rcentroid
{
   int   dd; 
   int   ii, jj; 
   float ady, adx; 


   adx = fabs(dx);
   ady = fabs(dy);

   //   dd = round(sqrt(dx*dx + dy*dy));
   dd = static_cast <int> (sqrt(dx*dx + dy*dy) + 0.5);


   if(dx<0){ 
      jj=i;  
      ii=j;
   } else {
      ii=i;  
      jj=j;
   }

   if(  (ady < adx*SLOPE_CHAR)               &&  /* not to far vertically */  
        (dd < N[ii].hnode_wt)                &&  /* less than before */
        (dd < N[ N[jj].hfrom_edge ].hnode_wt)  ) /* less than anyone else */
   {
      N[N[jj].hfrom_edge].hto_edge = 0;
      N[N[ii].hto_edge].hfrom_edge  = 0;
      N[ii].hnode_wt                = dd;
      N[ii].hto_edge                = jj;
      N[jj].hfrom_edge              = ii;
   }

   if(dy<0){
      jj=i;  
      ii=j;
   } else {
      ii=i;  
      jj=j;
   }
   if( (adx < SLOPE_CHAR*ady)                 && /* not to far horizontally */
       (dd < N[ii].vnode_wt)                  && /* less than before */
       (dd < N[ N[jj].vfrom_edge ].vnode_wt)  )  /* less than anyone else */
   {
      N[N[jj].vfrom_edge].vto_edge = 0;
      N[N[ii].vto_edge].vfrom_edge  = 0;
      N[ii].vnode_wt                = dd;
      N[ii].vto_edge                = jj;
      N[jj].vfrom_edge              = ii;
   }  
} /* put_on_list_dl */

// ( End of tbCalcLineOrientation.c)       //////////////////////////////////




// ( Beginning of tbCalcQuadrantMasses.c)  //////////////////////////////////
//  Changed 02/20/04, instead of allocating iQuadMasses the caller now
//  provides it.
//  (previously  included  "TB.h")

int JargonTBRoutines::tb_CalcQuadrantMasses(unsigned char* uscImage, int iRows, int iCols, int** iQuadMasses) {
   /* calcluate the masses of the quadrants 
    * This function is primarily for Jargon
    *
    * Return value is Number of on pixels
    */
   int iHalfRows = iRows/2;
   int iHalfCols = iCols/2;
   int iTotalPixels = 0;
   int iR,iC,iByteWidth;
   int rowsub;

   // (*iQuadMasses) = (int*)calloc(4, sizeof(int));
   (*iQuadMasses) = new int[4];

   // the QuadrantMasses must be initialized to zero
   for (iR=0; iR<4; iR++) 
      (*iQuadMasses)[iR] = 0;

   /* byte width */
   iByteWidth = (iCols+7)/8;

   for (iR = 0; iR < iRows; iR++) 
   {
      rowsub = iR * iByteWidth;
      for (iC = 0; iC < iCols; iC++) 
      {
         if ( (uscImage[rowsub + iC/8] & iLUT[iC%8]) > 0) 
         {
            if ((iR < iHalfRows) && (iC < iHalfCols))
               (*iQuadMasses)[0]+= 1;
            else
               if ((iR < iHalfRows) && (iC >= iHalfCols))
                  (*iQuadMasses)[1]+= 1;
               else
                  if ((iR >= iHalfRows) && (iC < iHalfCols))
                     (*iQuadMasses)[2]+= 1;
                  else
                     (*iQuadMasses)[3]+=1;
            /* add a pixel */
            iTotalPixels++;
         }
      }
   }

   /* return the total number of pixels */
   return( iTotalPixels );
}
// ( End of tbCalcQuadrantMasses.c)        //////////////////////////////////




// ( Beginning of tbPackUCArray.c)         //////////////////////////////////
//  Returns 0 for success, or -1 for allocation failure.
//  Changed to use new instead of calloc  01/26/04
//  (previously  included  "TB.h")

int JargonTBRoutines::tb_PackUCArray(unsigned char* image, unsigned char** newimage,
                                     int iRow, int iCol) 
{
   /* this function removes the padding at the end */

   int r, c,iByte;
   int cols_round_up;
   int rowsub;


   (*newimage) = new unsigned char [(iRow*iCol+7)/8];
   if ((*newimage) == NULL)
      return (-1);

   for (iByte=0; iByte < ((iRow*iCol+7)/8); iByte++)
      (*newimage)[iByte] = 0;

   cols_round_up = (iCol+7)/8;
   iByte = 0;
   for (r = 0; r < iRow; r++) 
   {
      rowsub = r * cols_round_up;
      for (c = 0; c < iCol; c++) 
      {
         if ( (image[rowsub + c/8] & iLUT[c%8]) > 0)
         {
            (*newimage)[iByte/8]+=iLUT[iByte%8]; 
         }
         iByte++;
      }
   }
   return( 0 );
}
// ( End of tbPackUCArray.c)               //////////////////////////////////


// ( tbGetResizedComponent.c)              //////////////////////////////////
// int tbGetResizedComponent(TBdocument* doink, int iComponent, unsigned char** image,int iResizeRow, int iResizeCol) 
int JargonTBRoutines::tb_GetResizedComponent(DLComponent * ccomp, unsigned char** image, int iResizeRow, int iResizeCol) 
{
   /* the component image is extacted,  then resized to 
    * iResizeCol, iResizeRow, and returned in unsigned char** image..
    * Returns 0 for success, or -1 for allocation failure.
    *
    * the image is uncompressed. meaning if you have extra on the end 
    * byte it stays their 
    *
    * 01110xxx
    * 10001xxx
    * 01110xxx
    * 
    * the above stays like this you can call CompressComponent and it will give
    * 01110100
    * 010110xx
    * 
    * ok?
    */

   int   iWidth;
   int   iHeight;
   int   iByteWidth;
   int   iBW;
   int   iBytesNeeded;
   int   colstart, colstop, row, col;
   int   iByte;
   float fXfact, fYfact;
   unsigned char* uscTemp;    // for initial image extraction (not resized)
   int    rowsub;
   int    temp_part_sub;
   int    colsub;


   /* set up variables */
   iWidth = (*ccomp).dlGetWidth();
   iHeight= (*ccomp).dlGetHeight();
   fXfact = static_cast <float> (iResizeCol) / static_cast <float> (iWidth);
   fYfact = static_cast <float> (iResizeRow) / static_cast <float> (iHeight);

   iBytesNeeded = iHeight * ((iWidth+7)/8);    // for uscTemp (not resized)

   uscTemp = new unsigned char [iBytesNeeded];
   if (uscTemp == NULL)
   {
      (*image) = NULL;
      return (-1);   
   }

   for (iByte=0; iByte<iBytesNeeded; iByte++)
      uscTemp[iByte]= 0;


   iByteWidth = (iResizeCol+7)/8;    // for re-sized image

   /* going to without packing the bytes */
   iBytesNeeded = iByteWidth * iResizeRow;     // for re-sized image

   (*image) = new unsigned char [iBytesNeeded];    // allocate re-sized image
   if ((*image) == NULL)
      return (-1);

   for (iByte=0; iByte<iBytesNeeded; iByte++)
      (*image)[iByte] = 0;

   // extract the image (original size) into temporary area

   iByteWidth = (iWidth+7)/8;    // for uscTemp (not resized)

   vector<DLDatarun> dataruns = (*ccomp).dlGetRuns();
   vector<DLDatarun>::iterator iterf = dataruns.begin();

   for (; iterf != dataruns.end(); iterf++)
   {
      DLDatarun dr = (DLDatarun) *iterf;

      row = dr.run_row - (*ccomp).dlGetRowMin();
      rowsub = row * iByteWidth;

      colstart = dr.run_start - (*ccomp).dlGetColMin();
      colstop =  dr.run_stop - (*ccomp).dlGetColMin();

      for (col = colstart; col < colstop; col++) 
         uscTemp[rowsub + col/8] += iLUT[col%8];
   }


   // re-size image and fill in *image

   iByteWidth = (iResizeCol+7)/8;
   iBW = (iWidth+7)/8;
   for (row = 0; row < iResizeRow; row++) 
   {
      rowsub = row * iByteWidth;
      temp_part_sub = static_cast <int> (static_cast <float> (row)/fYfact) * iBW;
      for (col = 0; col < iResizeCol; col++) 
      {
         colsub = static_cast <int> (static_cast <float> (col)/fXfact);
         if ((uscTemp[temp_part_sub + colsub / 8] & iLUT[colsub %8]) > 0)
            (*image)[rowsub + col/8] += iLUT[col%8];
      }
   }

   delete [] uscTemp;
   return( 0 );
}
// ( End of tbGetResizedComponent.c)       //////////////////////////////////



// ( tbRotateComponent.c)                  //////////////////////////////////
// Returns 0 for success;  or -1 for allocation failure or parameter error
// iRows and iWidth are the number of pixels not the bytes 
//  previously included "TB.h"

// int tbRotateComponent(unsigned char* origimage,unsigned char** image, int iRows, int iCols, int iDegrees) 
int JargonTBRoutines::tb_RotateComponent(unsigned char* origimage, unsigned char** rimage, int iRows, int iCols, int iDegrees) 
{

   int iBytesNeeded;
   int iByteWidthOld,iByteWidthNew,r,c,iNr,iNc;
   int isub;
   int jsub;
   int ksub;
   int lsub;

   switch (iDegrees) 
   {

      /* rotate the image 90 degrees */
      case 90:
         iBytesNeeded = iCols*((iRows+7)/8);
         (*rimage) = new unsigned char [iBytesNeeded];
         if ((*rimage) == NULL)
            return (-1);

         for (isub=0; isub<iBytesNeeded; isub++)
            (*rimage)[isub]= 0;
         iByteWidthNew = (iRows+7)/8;
         iByteWidthOld = (iCols+7)/8;

         for (r = 0; r < iRows; r++) 
         {
            isub = r*iByteWidthOld;
            iNc = r;
            jsub = r / 8;
            ksub = r % 8;

            for (c = 0; c < iCols; c++) 
            {
               iNr = iCols-1-c;

               if ((origimage[isub + c/8] & (unsigned char)iLUT[c%8]) > 0)
                  (*rimage)[iNr*iByteWidthNew + jsub] += iLUT[ksub];
            }
         }
         break;

         /* rotate the image 180 degrees */
      case  180:
         iBytesNeeded = iRows*((iCols+7)/8);
         (*rimage) = new unsigned char [iBytesNeeded];
         if ((*rimage) == NULL)
            return (-1);

         for (isub=0; isub<iBytesNeeded; isub++)
            (*rimage)[isub]= 0;
         iByteWidthNew = (iCols+7)/8;
         iByteWidthOld = (iCols+7)/8;

         for (r = 0; r < iRows; r++) 
         {
            isub = r * iByteWidthOld;
            iNr = iRows-r-1;
            lsub = iNr * iByteWidthNew;

            for (c = 0; c < iCols; c++) 
            {
               iNc = iCols-c-1;

               if ((origimage[isub + c/8] & (unsigned char)iLUT[c%8]) > 0)
                  (*rimage)[lsub + iNc/8] += iLUT[iNc%8];
            }
         }
         break;


      case 270:
         iBytesNeeded = iCols*((iRows+7)/8);
         (*rimage) = new unsigned char [iBytesNeeded];
         if ((*rimage) == NULL)
            return (-1);

         for (isub=0; isub<iBytesNeeded; isub++)
            (*rimage)[isub]= 0;
         iByteWidthNew = (iRows+7)/8;
         iByteWidthOld = (iCols+7)/8;

         for (r = 0; r < iRows; r++) 
         {
            isub = r * iByteWidthOld;
            iNc = iRows - r - 1;
            jsub = iNc / 8;
            ksub = iNc % 8;
            for (c = 0; c < iCols; c++) 
            {
               iNr = c;

               if ((origimage[isub + c/8] & (unsigned char)iLUT[c%8]) > 0)
                  (*rimage)[iNr*iByteWidthNew + jsub] += iLUT[ksub];
            }
         }
         break;


      default :
         dlLogger.dlLog(DL_CRITICAL, "RotateComponent ERROR, degrees (%d) not supported yet",iDegrees );
         (*rimage) = NULL;
         return(-1);
   }

   return(0);
}



int JargonTBRoutines::tbCompressUCArray(unsigned char* image, unsigned char** newimage,
                                        int iRow, int iCol) 
{
   int r, c,iByte;
   int iLUT[8] = {128,64,32,16,8,4,2,1};

   //   (*newimage) = (unsigned char*)calloc((iRow*iCol+7)/8, 
   //       sizeof(unsigned char));
   (*newimage) = new unsigned char[(iRow*iCol+7)/8]; 

   memset (*newimage, 0, (iRow*iCol+7)/8);

   iByte = 0;
   for (r = 0; r < iRow; r++) {
      for (c = 0; c < iCol; c++) {
         if ( (image[r*((iCol+7)/8) + c/8] & iLUT[c%8]) > 0) {
            (*newimage)[iByte/8]+=iLUT[iByte%8]; 
         };
         iByte++;
      };
   };
   return( 0 );
}
