#ifndef _JARGONTBROUTINES_H_
#define _JARGONTBROUTINES_H_

/* //////////////////////////////////////////////////////////////////////////
// JargonTBRoutines.cpp  07/02/04  Removed Printfs
//
// These modules were ported to Jargon from tb:
//    tbCalcLineOrientation.c ,(removed Edges)
//    tbCalcQuadrantMasses.c,  
//    tbPackUCArray.c         
//    tbGetResizedComponent.c
//    tbRotateComponent.c
//
//  Component Filtering for Jargon is done by using dlFilterCCxxx methods
//  Memory allocation in JargonTBRoutines is via new
*/
#include <cstdlib>    
#include <cstring>
#include <iostream>

extern "C" 
{
#include <math.h>
}//  fabs

#include <vector>
#include "DLImage.h"
#include "DLComponent.h"

/// Data structures used in the calculation of line orientation

typedef struct {
   int  hnode_wt,   vnode_wt;                       
   int  hto_edge,   vto_edge;    
   int  hfrom_edge, vfrom_edge;
} TB_nodes;

/// for the calc line orientation  routine (previously TBstats in TBTypes.h */
typedef struct {
   float hor_mean;
   float ver_mean;
   float asr;
   float hor_num;
   float ver_num;
   float hor_dx;
   float ver_dy;
} TB_stats;

#define  INSUFFICIENT_MEMORY -4
#define  INTMAXDISTBETWEENCOMP 100 /* integer max distance between adj. comp. */
#define  SLOPE_CHAR 0.5
#define  MIN_LINKS 15
#define  MINCOMP 20    // for tb_CalcLineOrientation

// common data
//int iLUT[8]= {128,64,32,16,8,4,2,1};

/** Class to contain a variety of legacy code from earlier image processing library.
 */
/// Class to contain a variety of legacy code from earlier image processing library.

class JargonTBRoutines 
{
   //friend class JMorphology;


   public:
   static int iLUT[8];

   /** <PRE>
    * FILE: rget_lines.c
    * DESCRIPTION: Uses  dress-right-dress algorithm to form the
    *              components into lines of text. in the horizontal and
    *              in the vertical direction then gets the mean of the
    *              distance between components in hor and vet direction 
    * USAGE: rget_lines(Document &D)
    * RETURN: void
    * NOTES:This version uses a triangle region to the right of CC.
	                                                                  
	                                                                 
    -     
    -        
    -           
    -    |          
    -  _____| SLOPE     
    -                    
    -                       
    -                          
    -         TO THE RIGHT        
    x_________________________> DD (radius)  
    -                             
    |  \   -                          
    -                       
    |      \       -                    
    -                 
    |          \           -              
    -           
    |  BELOW       \               -        
    -     
    |                  \                       
	                                                                      
    |                      \                       
	                                                                     
    |                          \                       
    </PRE>
    *RETURN VALUES : 0 means the lines are horizontal,
    *                 1 lines run vertical
    *                -1 not enough links on page
    *                -2 percent diff not great enough to make determination
    *                -4 insufficient memory 
    *
    * iNumber - previously the subscript 0 component was not used,
    *           and (iNumber - 1) components were processed, having
    *           subscripts 1 .. (iNumber-1).
    *
    * iPctDiff : % difference necessary to make the call, ie 5, 1, 20, etc
    *
    **/

   /// Calculates the line direction (horizontal vs. vertical()
   static int tb_CalcLineOrientation(DLImage &dimage, vector<DLComponent> * activecc, int iNumber, int iPctDiff) ;
   static void  put_on_list_dl(TB_nodes *N, int i, int j, float dx, float dy);
   /** Calculates the number of on pixels in the four quadrants of a region.  This routine operates
    * on an usigned char array
    */
   /// Calculates the number of on pixels in the four quadrants of a region
   static int tb_CalcQuadrantMasses(unsigned char* uscImage, int iRows, int iCols, int** iQuadMasses);
   /** This routine takes a 1 byte/pixel unsigned char array and packs it into a new 1 bit/pixel 
    * representation 
    */
   /// Compress an 1byte/pixel to a 1 bit/pixel array

   static int tb_PackUCArray(unsigned char* image, unsigned char** newimage,int iRow, int iCol) ;

   /// Creates an array (1 byte/pixel) represented a resized version of a connected component

   static int tb_GetResizedComponent(DLComponent * ccomp, unsigned char** image, int iResizeRow, int iResizeCol);

   /** Rotates a connected component.  Both the input and the output are unsigned char representations 
    * of the component
    */

   static int tb_RotateComponent(unsigned char* origimage, unsigned char** rimage, int iRows, int iCols, int iDegrees);

   /// Compresses an unsigned char (1 byte/pixel) to a 1 bit/pixel representation


   static int tbCompressUCArray(unsigned char* image, unsigned char** newimage,
                                int iRow, int iCol) ;


};
#endif //_JARGONTBROUTINES_H_
