#include "Trainer.h"
#include "ClusterUtils.h"
#include "Utils.h"

Trainer::Trainer(void)
{
}

Trainer::~Trainer(void)
{
}


int Trainer::train(char *training_file_list, char *language,
                   int hamming_distance, int min_component_size,
                   int max_height, int max_width, int blob_percent,
                   int symbol_cnt, int scaled_width, int scaled_height,
                   int small_cluster_threshold, char *results_file)
{
   int             file_symbol_count;
   int             cluster_count = 0;
   jcluster       *first_cluster;
   jcluster       *clust = 0;
   language_profile language_info;
   FILE           *list_file;
   char            current_file[800];
   int             scanf_result;
   int             symbol_count;


   /* TwoBit */
   int             errorTwoBit;
   //DLCCdocument strDocument;
   // int            *iList;
   //int            *iListStart;
   unsigned char  *uscResized;
   unsigned char  *uscCompressed;
   int             iCount;
   //int             iCurrentComponent;
   char           *cExpanded;
   int             iLUT[8] = {128, 64, 32, 16, 8, 4, 2, 1};
   int             iBytesNeeded;
   //int             i, x, bit;
   struct stat     stat_buf;
   FILE           *fp;
   char           *tiff_ptr;


   /* create language structure */

   symbol_count = 0;
   strcpy(language_info.identification, language);
   language_info.cluster_count = 0;
   language_info.file_count = 0;
   language_info.symbol_count = 0;
   language_info.max_height = max_height;
   language_info.max_width = max_width;
   cluster_count = 0;
   first_cluster = NULL;


   /* open file list */
   list_file = fopen(training_file_list, "r");
   if (list_file == NULL)
   {
      dlLogger.dlLog(DL_CRITICAL, "unable to open file listing training files");
      exit(0);
   }
   /* for each file */
   scanf_result = fscanf(list_file, "%s", current_file);

   while ((scanf_result > 0) && (strlen(current_file) > 0)) 
       
   {
      dlLogger.dlLog(DL_INFO, "file: %s\n", current_file);
      language_info.file_count++;

      /* Open the file and read it into memory */
      stat(current_file, &stat_buf);
      fp = fopen(current_file, "r");
      //       tiff_ptr = (char *) calloc(1, stat_buf.st_size);
      tiff_ptr = new char[stat_buf.st_size];

      fread(tiff_ptr, 1, stat_buf.st_size, fp);
      fclose(fp);
      delete [] tiff_ptr;

      /* create a new document */
      DLImage image(current_file);
      DLConnectedComponents cc;
      cc.dlCreateConnectedComponents(image);


      cc.dlFilterCCHeightBetween( min_component_size,  max_height);
      cc.dlFilterCCWidthBetween( min_component_size,  max_width);


      /* should we put an error code in here for making sure we have enough
       * chars on the page */

      file_symbol_count = 0;

      vector <DLComponent> ccList = cc.dlGetFilteredCCs();
      vector<DLComponent>::iterator iterf = ccList.begin();
      file_symbol_count = 0;

      for(; iterf != ccList.end() ; 
          iterf++) 
      {	
         DLComponent component = (DLComponent)*iterf;

         errorTwoBit = JargonTBRoutines::tb_GetResizedComponent(&component, &uscResized, scaled_width, scaled_height);

         JargonTBRoutines::tbCompressUCArray(uscResized, &uscCompressed, scaled_width, scaled_height);

         iBytesNeeded = scaled_width * scaled_height;
//            cExpanded = (char *) calloc(iBytesNeeded, sizeof(char));
         cExpanded = new char[iBytesNeeded];
         for(int idx = 0; idx < iBytesNeeded; idx++)
            cExpanded[idx] = 0;

         for (iCount = 0; iCount < iBytesNeeded; iCount++)
         {
            if ((uscCompressed[iCount / 8] & iLUT[iCount % 8]) > 0)
               cExpanded[iCount] = 1;
         }

         if (first_cluster == NULL)
         {
            first_cluster = ClusterUtils::create_cluster(cExpanded, NULL,
                                                         &cluster_count, scaled_height, scaled_width);
            clust = first_cluster;
         } else
         {
            /* add to the closes cluster based on hamming distance  */
            if (ClusterUtils::match_to_clusters(cExpanded,
                                                first_cluster, hamming_distance,
                                                scaled_height, scaled_width) < 0)
            {
               /* no match - create new cluster  */
               clust = ClusterUtils::create_cluster(cExpanded,
                                                    clust,
                                                    &cluster_count, scaled_height, scaled_width);
            }
         }
         symbol_count++;
         file_symbol_count++;

         delete[] uscResized;
         delete[] uscCompressed;
         delete[] cExpanded;
      }
      scanf_result = fscanf(list_file, "%s", current_file);
   }

   language_info.hamming = hamming_distance;
   language_info.symbol_count = symbol_count;
   language_info.min_size = min_component_size;
   language_info.small_cluster_threshold = small_cluster_threshold;
   first_cluster = ClusterUtils::filter_small_clusters(first_cluster, small_cluster_threshold,
                                                       &cluster_count);
   language_info.cluster_count = cluster_count;
   language_info.filtered_cluster_count = cluster_count;



/* */
   language_info.total_symbol_count = 0;
   language_info.correctly_matched = 0;
   language_info.incorrectly_matched = 0;
   language_info.hamming_sum = 0;
   language_info.best_hamming = 0;
   language_info.matched_sum = 0;
   language_info.cluster32 = NULL;
   language_info.cluster_data = NULL;
   language_info.first_cluster = NULL;
   language_info.prev = NULL;
   language_info.next = NULL;

   for(int i = 0; i < 18; i++)
   {
      language_info.spare[i] = 0;
   }


   Utils::save_script_data(results_file,
                           &language_info, first_cluster, scaled_height, scaled_width);

   /* free memory */
   while (first_cluster != NULL)
   {
      delete[] (first_cluster->first_image);
      delete[] (first_cluster->centroid);
      clust = first_cluster;
      first_cluster = (jcluster *) first_cluster->next;
      delete (clust);
   }
   return 1;
}
