#ifndef TRAINER_H
#define TRAINER_H

#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "JScript.h"

#include "DLConnectedComponentsUtils.h"
#include "DLImage.h"
#include "DLConnectedComponents.h"
#include "DLComponent.h"

#include "JargonUtils.h"

/**
 * Trainer is a class used to compute the clusters for a particular script
 * It is provided a list of images and generates the clusters based on 
 * the connected components within the images.
 */

/// The Trainer class provides for computing the clusters for a script
class Trainer
{
   public:
   /// Constructor
   Trainer(void);
   /// Destrucotr
   ~Trainer(void);

   /**
    * Train is the main function to train 
    * language scripts and generate data to be
    * used for the Jargon script ID
    *
    * /param training_file_list file containing list of training files for the script
    * /param language language
    * /param hamming_distance hamming distance
    * /param min_component_size minimum component size
    * /param max_height max_height of component
    * /param max_width max_width of component
    * /param blob_percent min_black_area (percentage value used to filter blob-like symbols - not currently used in training.  This is used in id.
    * /param symbol_cnt ymbol count (for debut only normally all symbols are processed
    * /param scaled_width escale_width
    * /param scaled_height rescale_height
    * /param small_cluster_threshold  small_cluster_threshold
    * /param results_file  file to which results should be stored 
    */

   static int train(char *training_file_list, char *language,
                    int hamming_distance, int min_component_size,
                    int max_height, int max_width, int blob_percent,
                    int symbol_cnt, int scaled_width, int scaled_height,
                    int small_cluster_threshold, char *results_file);
};

#endif
