#include "Utils.h"
#include "DLLogger.h"

Utils::Utils(void)
{
}

Utils::~Utils(void)
{
}


/* Returns number of pixel agreements between two bands of multi-band bit
   images.  Within the function, 'score' is the number of disagreements.
   Returns -1 on failure.  Makes a table to use in calculating scores the
   first time it is called */

#define SIDE 30
#define NUM_ENTRIES 256 /* size of the table */
#define NUM_BYTES ((SIDE + 7)/8) /* needed to hold one row's worth of bits */
#define MASK 0x3F /* needed to take care of last empty two bits in row */

int Utils::hamming(char *img1,  char *img2 )
{
   int i, j;
   int score = 0;
   unsigned char *ptr_uc1, *ptr_uc2; /* point to imagedata */
   unsigned char uctmp;		/* holds one btye during comparison */

   static int *bit_table;	/* table of bit counts */
   static int table_present = 0; /* is the table present? */

   /* initialize table if not done yet */
   if (!table_present) {

      /* Allocate memory for the table */
      bit_table = (int *) malloc (sizeof(int) * NUM_ENTRIES);
      if (!bit_table) {
         dlLogger.dlLog(DL_CRITICAL, "ERROR get_score:");
         dlLogger.dlLog(DL_CRITICAL,  "MEMORY ALLOCATION FAILURE\n");
         return(-1);
      }

      /* fill the table iwth the number of 1 bits in each unsigned char value */
      for (i = 0; i < NUM_ENTRIES; i++){
         bit_table[i] = 0;
         if (i & 0x01) bit_table[i]++;
         if (i & 0x02) bit_table[i]++;
         if (i & 0x04) bit_table[i]++;
         if (i & 0x08) bit_table[i]++;
         if (i & 0x10) bit_table[i]++;
         if (i & 0x20) bit_table[i]++;
         if (i & 0x40) bit_table[i]++;
         if (i & 0x80) bit_table[i]++;
      }

      /* set flag to indicate that the table is now present */
      table_present = 1;
   } /* done making table */

   /* access the imagedata part of both input images */
   ptr_uc1 = (unsigned char *) img1;
   ptr_uc2 = (unsigned char *) img2;
  
   /* go through two imagedatas byte by byte, updating score */
   for (i = 0; i < SIDE; i++){	/* for each row */
    
      /* general case */
      for (j = 0; j < NUM_BYTES-1; j++){
         uctmp = ptr_uc1[i*NUM_BYTES + j] ^ ptr_uc2[i*NUM_BYTES + j];
         /* uctmp now holds 1 for disagree, 0 for agree */
         score += bit_table[uctmp];
         /* adds appropriate number of 1's to score */
      }

      /* last byte: has padding beyond last significant bit */
      uctmp = (ptr_uc1[(i*NUM_BYTES) + (NUM_BYTES - 1)] ^ 
               ptr_uc2[(i*NUM_BYTES) + (NUM_BYTES - 1)]) & MASK;
      score += bit_table[uctmp];
   } /* done for each row */

   return SIDE*SIDE - score;
}

void Utils::save_script_data(char *file_path, 
                             language_profile *language, 
                             jcluster *first_cluster,int height, int width)
{

   FILE *fp;
   jcluster *cluster_ptr;
   int image_size;

   int idx;

   image_size = height*width;

   fp = fopen(file_path,"w");

   if (fp == NULL)
   {
      dlLogger.dlLog(DL_CRITICAL, "could not open script_data file: %f\n",file_path);
      return;
   }
   fprintf(fp, "%s %d %d %d %d %d %d %d %d %d %d %d %d %u %u %f ",
           language->identification,
           language->cluster_count,
           language->filtered_cluster_count,
           language->file_count,
           language->hamming,
           language->symbol_count,
           language->small_cluster_threshold,
           language->min_size,
           language->max_height,
           language->max_width,
           language->total_symbol_count,
           language->correctly_matched,
           language->incorrectly_matched,
           language->hamming_sum,
           language->best_hamming,
           language->matched_sum);

   cluster_ptr = first_cluster;
   while (cluster_ptr != NULL)
   {


      //fwrite(cluster_ptr, sizeof(jcluster), 1, fp);
      fprintf(fp, "%d %d %d %f %u %d %d ", cluster_ptr->count,
              cluster_ptr->correct_matches,
              cluster_ptr->incorrect_matches,
              cluster_ptr->reliability,
              cluster_ptr->similar_symbol_count,
              cluster_ptr->matches_per_image,
              cluster_ptr->on_pixels);
    
      //fwrite(cluster_ptr, sizeof(jcluster), 1, fp);
    
      //for(int idx = 0; idx < image_size; idx++)
      //{
      //    fprintf(fp, "%d ", luster_ptr->first_image[idx]);
      //}

      /* centroid is integer instead of character */

      for(idx = 0; idx < image_size; idx++)
      {
         fprintf(fp, "%d ", cluster_ptr->centroid[idx]);
      }

      cluster_ptr = (jcluster *) cluster_ptr->next;
   }

   fclose(fp);


}
