/* layout.h
   Class Clayout is used to form 5 dimenstion feature vectors (objvector) 
   for each textline in a document page. 
*/

#if !defined(AFX_LAYOUT_H__41A5307C_A00B_418A_A668_D1C2E20618DF__INCLUDED_)
#define AFX_LAYOUT_H__41A5307C_A00B_418A_A668_D1C2E20618DF__INCLUDED_

#include <string>
#include "../ObjectVector_hm/objcluster.h"	
#include "../PageLayoutLib/objectInfoClass.h"

struct rectangle
{
   long left;
   long top;
   long right;
   long bottom;

   rectangle & operator = (rectangle b)
   {
      left = b.left;
      top = b.top;
      right = b.right;
      bottom = b.bottom;
      return *this;
   }


};


class Clayout  
{
   public:
   Clayout & operator = (Clayout & a);
	
   double			m_pageheight;          // height between the toppest and lowest textline 
   double			m_pagewidth;           // width between the leftmost endingpoint and the rightmost endingpoint
	
   objvector *		m_pObjectList;          
   int				m_nbObjects;

   objcluster*		m_pClusterList;
   int				m_nbClusters;
	
	
   void	BoundingBox(objectInfo * obj, int objcount, struct rectangle &r); //finding the minimum bounding box of all the objects
   void	Rotate(CTMatrix<double> &obj);						   //Rotate the page so that the bounding box is upright
   void	ImportObjects(objectInfo *obj, int objcount); 
	

   // Kmeans clustering of object 5D vectors
   void	KmeansObjClustering(Clayout * pLayouts, int nObj, int nDoc, int nCluster, int maxiteration, objvector sigma, bool use_zero_flag = false);
	
   // compute the mean and variance of all samples, for usage of Kmeans clustering
	
   void	NormalizeObjects(Clayout * pLayouts, int nDoc, objvector &avg, objvector &sigma);
	
   // weighting wanted 5D object clusters
   void	Weight_Clusters_against(Clayout * pLayout, double probability_range = 0.5);

   // disk operation
   void SaveClustersToDisk(string fname, bool saveweights = true);
   void LoadClustersFromDisk(string fname);
		
   Clayout();
   virtual ~Clayout();
   void Reset();

   private:
   CTArray<int>    m_index;				//designate the cluster index, which the object belongs to
		
   //objvector	m_avg;				// used for objClustering		
   //objvector	m_sigma;
   //objvector	m_weight;
				
};

#endif // !defined(AFX_LAYOUT_H__41A5307C_A00B_418A_A668_D1C2E20618DF__INCLUDED_)
