// objcluster.h: interface for the objcluster class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_OBJCLUSTER_H__4B0D7A50_3E37_454B_A627_0C159E376149__INCLUDED_)
#define AFX_OBJCLUSTER_H__4B0D7A50_3E37_454B_A627_0C159E376149__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <fstream>

#include "math.h"
#include "../Utility_hm/TMatrix.h"
using namespace std;

/* objvector captures all essential information for a single textline,
 * derived from objectInfoClass.
 * When only use single textline/object as feature primitive of a document page,
 * the five double data form the vector for feature similarity computation.
 * When pair objects/two textlines are used to form feature primitive, the vector
 * of the polygon is formed in a way to capture the shape information.(See pairshape.h)

*/
struct objvector 
{
   int		Type;			 // type of the object: textline, tableline, etc. Currently 
   // only textline is considered

   // 5-Dim vector
   double	FontHeight;
   double	Length;			 // Euclidean distance between two ending points
   double	Centroid_X;      // Position of the middle points of the line
   double	Centroid_Y;
   double	Angle;           // angle between the horizontal line and the textline

   /** 
    * Constructor
    **/
   objvector()
   {
      Type		= 0;
      FontHeight	= 0;
      Length		= 0;
      Centroid_X	= 0;
      Centroid_Y	= 0;
      Angle       = 0;
   }

   /** 
    * Constructor
    **/
   objvector(int type, double font, double length, double x, double y, double angle)
   {
      Type		= type;
      FontHeight	= font;
      Length		= length;
      Centroid_X	= x;
      Centroid_Y	= y;
      Angle       = angle;
   }

   /** 
    * write_stream outputs instance to file
    * \param f output file stream object
    **/
   void write_stream( ofstream & f )
   {
      f << Type << endl;	 // type of the object: textline, tableline, etc. Currently 
      // only textline is considered

      // 5-Dim vector
      f << FontHeight << endl;
      f << Length << endl;			// Euclidean distance between two ending points
      f << Centroid_X << endl;		// Position of the middle points of the line
      f << Centroid_Y << endl;
      f << Angle << endl;				// angle between the horizontal line and the textline
   }

   /** 
    * read_stream loads instance from file
    * \param f input file stream object
    **/
   void read_stream( ifstream &f )
   {
      f >> Type;					// type of the object: textline, tableline, etc. Currently 
      // only textline is considered

      // 5-Dim vector
      f >> FontHeight;
      f >> Length;				// Euclidean distance between two ending points
      f >> Centroid_X ;			// Position of the middle points of the line
      f >> Centroid_Y ;
      f >> Angle;					// angle between the horizontal line and the textline
	
   }

   objvector operator - (objvector a)
   {
      objvector b;
      //ASSERT(Type == a.Type || Type * a.Type == 0);
      b.Type		 = Type;
      b.FontHeight = FontHeight - a.FontHeight;
      b.Length	 = Length	  - a.Length;
      b.Centroid_X = Centroid_X - a.Centroid_X;
      b.Centroid_Y = Centroid_Y - a.Centroid_Y;
      b.Angle		 = Angle	  - a.Angle;
		
      return b;
   }
   objvector operator + (objvector a)
   {
      objvector b;
      //ASSERT(Type == a.Type || Type * a.Type == 0);
      b.Type		 = Type;
      b.FontHeight = FontHeight	+ a.FontHeight;
      b.Length	 = Length		+ a.Length;
      b.Centroid_X = Centroid_X	+ a.Centroid_X;
      b.Centroid_Y = Centroid_Y	+ a.Centroid_Y;
      b.Angle		 = Angle		+ a.Angle;
		
      return b;
   }


   objvector operator * (double a)
   {
      objvector b;
      b.Type		 = Type; 
      b.FontHeight = FontHeight * a;
      b.Length	 = Length * a;
      b.Centroid_X = Centroid_X * a;
      b.Centroid_Y = Centroid_Y * a;
      b.Angle		 = Angle * a;		
      return b;
   }

   objvector operator / (double a)
   {
      //ASSERT(a!=0);

      objvector b;
      b.Type		 = Type; 
      b.FontHeight = FontHeight / a;
      b.Length	 = Length / a;
      b.Centroid_X = Centroid_X / a;
      b.Centroid_Y = Centroid_Y / a;
      b.Angle		 = Angle / a;
      return b;
   }

   objvector dotproduct(objvector a)
   {
      //ASSERT(Type == a.Type);

      objvector b		= *this;
      b.Angle			*= a.Angle;
      b.Centroid_X	*= a.Centroid_X;
      b.Centroid_Y	*= a.Centroid_Y;
      b.Length		*= a.Length;
      b.FontHeight	*= a.FontHeight;
      return b;
   }

   objvector dotsqrt()
   {
      //ASSERT(Type == a.Type);

      objvector b		= *this;
      b.Angle			= sqrt(Angle);
      b.Centroid_X	= sqrt(Centroid_X);
      b.Centroid_Y	= sqrt(Centroid_Y);
      b.Length		= sqrt(Length);
      b.FontHeight	= sqrt(FontHeight);
      return b;
   }

   objvector dotdivide(objvector a, bool flag = true)
   {
      //ASSERT(Type == a.Type);

      objvector b		= *this;
      b.Angle			/= a.Angle;
      if(flag)
      {
         b.Centroid_X	/= a.Centroid_X;
         b.Centroid_Y	/= a.Centroid_Y;
      }
      b.Length		/= a.Length;
      b.FontHeight	/= a.FontHeight;
      return b;
   }

   objvector objsqrt()
   {
      objvector b;
      b.Type			= Type;
      b.Angle			= sqrt(Angle);
      b.Centroid_X	= sqrt(Centroid_X);
      b.Centroid_Y	= sqrt(Centroid_Y);
      b.FontHeight	= sqrt(FontHeight);
      b.Length		= sqrt(Length);

      return b;
   }

   objvector& operator = (objvector a)
   {
      Type			= a.Type;
      FontHeight		= a.FontHeight;
      Length			= a.Length;
      Centroid_X		= a.Centroid_X;
      Centroid_Y		= a.Centroid_Y;
      Angle			= a.Angle;

      return *this;
   }

   bool operator == (objvector a)
   {
      if (Type != a.Type) return false;
      if (FontHeight != a.FontHeight ) return false;
      if (Length != a.Length) return false;
      if (Centroid_X != a.Centroid_X) return false;
      if (Centroid_Y != a.Centroid_Y) return false;
      if (Angle != a.Angle) return false;

      return true;
   }

   objvector dotabs()
   {
      objvector b = *this;
      if(b.Angle <0) b.Angle = -b.Angle;
      if(b.Centroid_X <0) b.Centroid_X = -b.Centroid_X;
      if(b.Centroid_Y <0) b.Centroid_Y = -b.Centroid_Y;
      if(b.FontHeight <0) b.FontHeight = -b.FontHeight;
      if(b.Length <0) b.Length = -b.Length;

      return b;
   }

   double norm()
   {
      return sqrt(normsquare());
   }

   double normsquare()
   {
      return FontHeight*FontHeight + Length*Length+Centroid_X*Centroid_X + Centroid_Y*Centroid_Y + Angle*Angle;
   }

   /* weighted norm */
   double norm_2(double f_r=1, double a_r=1, double l_r=1, double x_r=1, double y_r=1)
   {
      double ratio = f_r + a_r+ l_r + x_r + y_r;
      double dist = FontHeight*FontHeight * f_r + Length*Length * l_r
         + Centroid_X*Centroid_X * x_r + Centroid_Y*Centroid_Y * y_r 
         + Angle*Angle * a_r;

      return sqrt(dist/ratio);
   }

   public:
   void clear()
   {
      Type = 0;
      FontHeight = 0;
      Length = 0;
      Centroid_X = 0;
      Centroid_Y = 0;
      Angle = 0;
   }
};

struct objcluster  
{
   public:
objvector	m_centroid;
   objvector	m_sigma;		    //standard diveation: multivariate gaussian
   int			m_nbObjects;

   double		m_weight;
   double		m_avgnbObjperdoc;   // #objects of this cluster averaged by the number of training tiff images 
   double		m_sigmaObjperdoc;   // standard deviation of #objects of this cluster accross the training tiffs

   objcluster();
   virtual ~objcluster();

   double Probability(objvector& obj);
	
   // void   SaveToDisk(CString fname, bool saveWeights = true);
   // void   LoadFromDisk(CString fname );

   void write_stream(ofstream & of, bool saveweights);
   void read_stream(ifstream &f);
};


//struct objclusterset
//{
//public:
//	int m_nClusters;
//	objcluster * m_pClusters;
//
//	objclusterset() { m_nClusters = 0; m_pClusters = NULL; }
//	virtual ~objclusterset() 
//	{
//		for(int i=0; i<m_nClusters; i++)
//			delete m_pClusters[i];
//	}
//
//	void SaveToDisk(char* path, bool saveweights = true);
//	void   LoadFromDisk(CString fname );
//};



//#define INCREAMENTAL_AMOUNT 12
//
//struct a_Cluster
//{
//	int				centroid[2]; // index of centroid object/pair, page_index/member_index
//	double			sum_dist;    // sum of distance from each object pair to centroid pair
//	CTMatrix<int>	memberlist;
//	int				nMember;
//
//	a_Cluster()
//	{
//		sum_dist = 0;
//		nMember = 0;
//		centroid[0] = centroid[1] = -1;
//	}
//
//	~a_Cluster()
//	{
//		memberlist.Destruction();
//	}
//	
//	void Add(int page_index, int member_index)
//	{
//		if(nMember == memberlist.GetRows())
//		{
//			CTMatrix<int> tmp = memberlist;
//			memberlist.Construction(nMember + INCREAMENTAL_AMOUNT,2);
//			for(int i=0;i<nMember;i++)
//				memberlist[i] = tmp[i];
//		}
//
//		memberlist[nMember][0] = page_index;
//		memberlist[nMember][1] = member_index;
//		nMember ++;
//	}
//};

#endif // !defined(AFX_OBJCLUSTER_H__4B0D7A50_3E37_454B_A627_0C159E376149__INCLUDED_)
