
#include "objectPairClass.h"
#include "polyClass.h"


#include <cstdio>
#include <iostream>
#include <fstream>
#include <string>
#include <cstring>
#include <vector>
#include <algorithm>
#include <stdlib.h>
#include <math.h>



/***************************************************/
objectPairClass::~objectPairClass()
{
   if (_objectPairs != NULL)
   {
      free(_objectPairs);
      _objectPairs = NULL;
   }
}


/***************************************************/
// updated by hm
// neighbor : 0-- Best Neighbor; 1 -- nearest neighbor; 2-- Gauss_neighborhood
// for more detail, see definitions in utility.h
void objectPairClass::scoreImage(objectPairClass *objectPairs,float *score, int neighbor) //used in testing
{
   /* for each object pair on the image find the best matching 
      object pair (if there is a match ).  Increment cummulative
      score by the score of the best match */

   *score = 0.0;

   int cnt = objectPairs->getPairCount();
   objectPairInfo *pairs = objectPairs->getPairs();

   int instanceSum = 0;
   //for each polygon in the test document
   int i,j;
   for (i = 0; i < cnt; i++)
   {
      float best_fit_score = 0;		
		
      //for nearest neighbor
      double min_dist = 10000; 

      //for Gaussian KNN
      double sum_weighted_score = 0;
      double sum_probability = 0;
		
      for (j=0;j < _numPairsAfterPrune; j++)
      {
         if(_objectPairs[j].type1 == nullObject) continue;

         double dist;
         int result = matches(&pairs[i],&_objectPairs[j],&dist);
         if (result > 0 ) 				
         {
            //Best Neighbor : Original method
            if(neighbor ==0 && _objectPairs[j].score > best_fit_score )
               best_fit_score = _objectPairs[j].score;				

            //Nearest Neighbor
            else if(neighbor ==1 &&dist < min_dist)
            {
               best_fit_score = _objectPairs[j].score;
               min_dist = dist;
            }

            else //Gaussian weighted KNN
            {
               double  delta = POLY_MATCH_THRESHOLD/3; 
               double  gaussian_probability= exp(-dist*dist/(2*delta*delta))/(2.50663*delta);				
               sum_weighted_score += _objectPairs[j].score * gaussian_probability;
               sum_probability += gaussian_probability;
            }
         }
      }

      //for Best Neighbor & NN
      if(neighbor != 2)
         pairs[i].score = best_fit_score;

      else //for Gaussian KNN
         pairs[i].score = sum_probability==0 ? 0 : sum_weighted_score/sum_probability;
		
      *score += pairs[i].score * pairs[i].totalInstanceCount ;
      instanceSum += pairs[i].totalInstanceCount;
   }

   // normalization
   *score   /= (float)max(1,instanceSum); 

}

/***************************************************/

void objectPairClass::dumpHighScoringObjects(struct objectInfo *obj1,float *threshold)
{
   int i;
   FILE *fp = fopen("tmpscore.dat","w");
   for (i=0;i<_numPairs;i++)
   {
      if (_objectPairs[i].score >= *threshold)
      {
         printf("dumping to tmpscore threshold: %f pair: %d score: %f\n",*threshold, i, _objectPairs[i].score );
         fprintf(fp,"%d \n",_objectPairs[i].object1);
         fprintf(fp,"%d \n",_objectPairs[i].object2);
         obj1[_objectPairs[i].object1].occuranceCount ++;
         obj1[_objectPairs[i].object2].occuranceCount ++;
      }
   }

   fclose(fp);

}


/***************************************************/
/******weighting wanted clusters *********************/
void objectPairClass::assignScore(objectPairClass *otherClass) //( int nDoc_wanted, int nDoc_unwanted)
{	
   int i,j;
   objectPairInfo *other_struct = otherClass->getPairs();
   int other_cnt = otherClass->getPairCount();	

   for (i=0;i<_numPairsAfterPrune;i++)
   {
      if(_objectPairs[i].totalInstanceCount <1) //a virtue cluster 
      {
         _objectPairs[i].score = 0;
         continue;
      }

      int near_count = 0;		 
      for (j=0;j<other_cnt;j++)
      {
         double dist;
         int result = matches(&_objectPairs[i],&other_struct[j],&dist);
         if (result > 0)	
            near_count += other_struct[j].totalInstanceCount;
							
      }

      // updated by hm
      _objectPairs[i].score = (float)_objectPairs[i].totalInstanceCount/
         ((float)_objectPairs[i].totalInstanceCount + (float)near_count); //corrective_factor2);
	

   }

}

/***************************************************/

objectPairClass::objectPairClass()
{
   _frequencyPrunedLevel = 0;
   _coref_file = NULL;
   _numPairs = 0;
   _numPairsAfterPrune = 0;

   _objectPairs = (objectPairInfo *) calloc ( CALLOC_BLOCK_SIZE, sizeof(objectPairInfo));
   /* initialize list of documents containing pair */

   int i;
   for(i=0;i<CALLOC_BLOCK_SIZE;i++)_objectPairs[i].docsCount = 0;

   _availableEntries = CALLOC_BLOCK_SIZE;
}

/***************************************************/

objectPairClass::objectPairClass(FILE *coref_file)
{
   _frequencyPrunedLevel = 0;
   _coref_file = coref_file;
   _numPairs = 0;
   _availableEntries = CALLOC_BLOCK_SIZE;
   _objectPairs = (objectPairInfo *) calloc ( CALLOC_BLOCK_SIZE, sizeof(objectPairInfo));

   /* initialize list of documents containing pair */

   int i;
   for (i=0;i<CALLOC_BLOCK_SIZE;i++)_objectPairs[i].docsCount = 0;

   _numPairsAfterPrune = 0;

}


/**********************************************/
void objectPairClass::readDataWithScores(FILE *fp)
{
   polyMatchClass pm;

   _frequencyPrunedLevel = 0;
   int i = 0;
   int result = fscanf(fp,"%*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s  %*s  %*s  %*s ");
   while (result == 0)
   {
      i++;
      result = fscanf(fp,"%*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s ");
   }

   fseek(fp,0,SEEK_SET);

   _numPairs = i;
   _objectPairs =	(objectPairInfo *) calloc( _numPairs, sizeof(objectPairInfo));

   /* initialize list of documents containing pair */
   for (i=0;i<_numPairs;i++)
   {
      result = fscanf(fp,"%*s %*s %*s %d %*s %d %*s %d %*s %f %*s %d %*s %d %*s %d %*s %d %*s %d %*s %d %*s %d %*s %d %*s %d %*s %d %*s %d %*s %f %*s %s %*s %s %*s %d %*s %d",
                      &_objectPairs[i].type1,
                      &_objectPairs[i].type2,
                      &_objectPairs[i].crossing,
                      &_objectPairs[i].area,
                      &_objectPairs[i].pageInstanceCount,
                      &_objectPairs[i].totalInstanceCount,
                      &_objectPairs[i].docsCount,
                      &_objectPairs[i].vertices[0].x,
                      &_objectPairs[i].vertices[0].y,
                      &_objectPairs[i].vertices[1].x,
                      &_objectPairs[i].vertices[1].y,
                      &_objectPairs[i].vertices[2].x,
                      &_objectPairs[i].vertices[2].y,
                      &_objectPairs[i].vertices[3].x,
                      &_objectPairs[i].vertices[3].y,
                      &_objectPairs[i].score,
                      &_objectPairs[i].obj1_qualifier,
                      &_objectPairs[i].obj2_qualifier,
                      &_objectPairs[i].obj1_int_qualifier,
                      &_objectPairs[i].obj2_int_qualifier);

      /* store turning representation to speed up later matching */
      pm.poly_to_turn_rep(_objectPairs[i].vertices,4,&_objectPairs[i].t);
   }
}




/***************************************************/

objectPairClass::objectPairClass(const char *pair_data_file,FILE *coref_file)
{
   int i,k;

   polyMatchClass pm;
   char string_val[200];

   _availableEntries = 0;
   _frequencyPrunedLevel = 0;

   _coref_file = coref_file;
   FILE *fp = fopen(pair_data_file,"r");

   int result = fscanf(fp,"%*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %s %*s %*s %*s %*s %*s %*s %*s",string_val); 
   if (strcmp(string_val,"score:") == 0)
   {
      fseek(fp,0,SEEK_SET);
      readDataWithScores(fp);
   }
   else
   {
      i = 0;
      fseek(fp,0,SEEK_SET);
      result = fscanf(fp,"%*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s"); 
      while (result == 0)
      {
         i++;
         result = fscanf(fp,"%*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s %*s"); 
      }
	
      fseek(fp,0,SEEK_SET);

      _numPairs = i;
      _objectPairs = 	(objectPairInfo *) calloc ( _numPairs, sizeof(objectPairInfo));

      /* initialize list of documents containing pair */
      for (i=0;i<_numPairs;i++)
      {
         result = fscanf(fp,"%*s %*s %*s %d %*s %d %*s %d %*s %f %*s %d %*s %d %*s %d %*s %d %*s %d %*s %d %*s %d %*s %d %*s %d %*s %d %*s %d %*s %s %*s %s %*s %d %*s %d",
                         &_objectPairs[i].type1,
                         &_objectPairs[i].type2,
                         &_objectPairs[i].crossing,
                         &_objectPairs[i].area,
                         &_objectPairs[i].pageInstanceCount,
                         &_objectPairs[i].totalInstanceCount,
                         &_objectPairs[i].docsCount,
                         &_objectPairs[i].vertices[0].x,
                         &_objectPairs[i].vertices[0].y,
                         &_objectPairs[i].vertices[1].x,
                         &_objectPairs[i].vertices[1].y,
                         &_objectPairs[i].vertices[2].x,
                         &_objectPairs[i].vertices[2].y,
                         &_objectPairs[i].vertices[3].x,
                         &_objectPairs[i].vertices[3].y,
                         _objectPairs[i].obj1_qualifier,
                         _objectPairs[i].obj2_qualifier,
                         &_objectPairs[i].obj1_int_qualifier,
                         &_objectPairs[i].obj2_int_qualifier);

         /* store turning representation to speed up later matching */
         if (_objectPairs[i].totalInstanceCount == 0) k = 1;
         pm.poly_to_turn_rep(_objectPairs[i].vertices,4,	&_objectPairs[i].t);

      }//end for
   }//end else

   fclose(fp);
   _numPairsAfterPrune  = _numPairs;
}


/***************************************************/
objectPairClass::objectPairClass(struct objectInfo *obj1,int object_count, int documentId, FILE *coref_file)
{
   polyMatchClass pm;
   int i,j;

   /* this class creates an array of object pair structures
      based on the objects received as a parameter */

   _frequencyPrunedLevel = 0;
   _coref_file = coref_file;

   /* number of object pairs = n*(n-1)/2  */
   if (object_count < 1)
   {
      _numPairs = 0;
      return;
   }
   else	_numPairs = object_count*(object_count-1)/2;

   _numPairsAfterPrune = _numPairs;
   _availableEntries = 0;

   _objectPairs = (objectPairInfo *) calloc(_numPairs, sizeof(objectPairInfo));

   /* initialize list of documents containing pair */

   for (i=0;i<_numPairs;i++)	/* initialized document entries for those entries which will be used at this time */
   {
      _objectPairs[i].docsCount = 1;
      _objectPairs[i].doc_index = documentId; //hm
      _objectPairs[i].pageInstanceCount = 1;
   }

   int index=0;
   for (i=0;i<object_count-1;i++)
   {
      for (j=i+1;j<object_count;j++)
      {
         /* store info in pairs structure */
         _objectPairs[index].type1 = obj1[i].type;
         sprintf(_objectPairs[index].obj1_qualifier,"%s",obj1[i].qualifier);
			
         _objectPairs[index].type2 = obj1[j].type;
         sprintf(_objectPairs[index].obj2_qualifier,"%s",obj1[j].qualifier);

         _objectPairs[index].obj1_int_qualifier = obj1[i].int_qualifier;
         _objectPairs[index].obj2_int_qualifier = obj1[j].int_qualifier;

         _objectPairs[index].object1 = i;
         _objectPairs[index].object2 = j;
         _objectPairs[index].pageInstanceCount = 1;
         _objectPairs[index].totalInstanceCount = 1;

         _objectPairs[index].vertices[0].x =	obj1[i].location[0].x;
         _objectPairs[index].vertices[0].y =	obj1[i].location[0].y;
         _objectPairs[index].vertices[1].x =	obj1[i].location[1].x;
         _objectPairs[index].vertices[1].y =	obj1[i].location[1].y;

         _objectPairs[index].vertices[2].x =	obj1[j].location[0].x;
         _objectPairs[index].vertices[2].y =	obj1[j].location[0].y;
         _objectPairs[index].vertices[3].x =	obj1[j].location[1].x;
         _objectPairs[index].vertices[3].y =	obj1[j].location[1].y;


         /* create a vector of the 4 points defining the 2 objects */
         vector<pointLocation> pvector(4);
         pvector[0] =  _objectPairs[index].vertices[0];
         pvector[1] =  _objectPairs[index].vertices[1];
         pvector[2] =  _objectPairs[index].vertices[2];
         pvector[3] =  _objectPairs[index].vertices[3];
         polyClass *poly = new polyClass(pvector);
         _objectPairs[index].crossing = poly->linesIntersect();

         if (_objectPairs[index].crossing >= 0)		
            _objectPairs[index].area = poly->getArea();		
         else		
            _objectPairs[index].area = 0;
			
         delete poly;

         /* store turning representation to speed up later matching */
         pm.poly_to_turn_rep(_objectPairs[index].vertices,4,&_objectPairs[index].t);
			
         index++;
      }
   }

   prune();


} /* create method */

/************************************************************/

void objectPairClass::printInfoSummary(char *filename)
{
   FILE *fp = fopen (filename,"w+");
   if(fp==NULL) 
      printf("_numPairsAfterPrune: %d availableEntries: %d numPairs: %d afterPrune: %d\n",
             _numPairsAfterPrune,
             _availableEntries, _numPairs, _numPairsAfterPrune);
   else {
      fprintf(fp, "_numPairsAfterPrune: %d availableEntries: %d numPairs: %d afterPrune: %d\n",_numPairsAfterPrune,
              _availableEntries,_numPairs,  _numPairsAfterPrune);
      fclose (fp);
   }
}
/************************************************/
void objectPairClass::printScoreInfo(string filename)
{
   int i;
   FILE *fp = fopen (filename.c_str(),"a");
   if(!fp) return;

   for (i=0;i<_numPairsAfterPrune;i++)
   {
      fprintf(fp,"pair: %d type1: %d type2: %d crossing: %d area: %f pageInst: %1.0f totalInst: %d docCount: %d v1x: %d v1y: %d v2x: %d v2y: %d ",
              i, 	
              _objectPairs[i].type1,
              _objectPairs[i].type2,
              _objectPairs[i].crossing,
              _objectPairs[i].area,	
              _objectPairs[i].pageInstanceCount,
              _objectPairs[i].totalInstanceCount,
              _objectPairs[i].docsCount,
              _objectPairs[i].vertices[0].x,
              _objectPairs[i].vertices[0].y,
              _objectPairs[i].vertices[1].x,
              _objectPairs[i].vertices[1].y);
    
    
      fprintf(fp,"v3x: %d v3y: %d v4x: %d v4y: %d score: %f intqual1: %d intqual2: %d \n",
              _objectPairs[i].vertices[2].x,
              _objectPairs[i].vertices[2].y,
              _objectPairs[i].vertices[3].x,
              _objectPairs[i].vertices[3].y,
              _objectPairs[i].score,
              //	    _objectPairs[i].obj1_qualifier,
              //	    _objectPairs[i].obj2_qualifier,
              _objectPairs[i].obj1_int_qualifier,
              _objectPairs[i].obj2_int_qualifier);
    
      /* string q1 (_objectPairs[i].obj1_qualifier);
         string q2 ( _objectPairs[i].obj2_qualifier);
         fprintf(fp,"pair: %d type1: %d type2: %d  crossing: %d area: %f pageInst: %d  totalInst: %d docCount: %d v1x: %d v1y: %d v2x: %d v2y: %d v3x: %d v3y: %d v4x: %d v4y: %d score: %f qual1: %s qual2: %s intqual1: %d intqual2: %d\n",
         i,
         _objectPairs[i].type1,
         _objectPairs[i].type2,
         _objectPairs[i].crossing,
         _objectPairs[i].area,	
         _objectPairs[i].pageInstanceCount,
         _objectPairs[i].totalInstanceCount,
         _objectPairs[i].docsCount,
         _objectPairs[i].vertices[0].x,
         _objectPairs[i].vertices[0].y,
         _objectPairs[i].vertices[1].x,
         _objectPairs[i].vertices[1].y,
         _objectPairs[i].vertices[2].x,
         _objectPairs[i].vertices[2].y,
         _objectPairs[i].vertices[3].x,
         _objectPairs[i].vertices[3].y,
         _objectPairs[i].score,
         q1.c_str(), // _objectPairs[i].obj1_qualifier,
         q2.c_str(), // _objectPairs[i].obj2_qualifier,
         _objectPairs[i].obj1_int_qualifier,
         _objectPairs[i].obj2_int_qualifier); */
   }
  
   fclose(fp);
}
/************************************************************/


/************************************************************/

void objectPairClass::printInfo(string filename)
{
   /* the printInfo and printScoreInfo routines are virtually identical
      except the printScoreInfo includes the score field in the output */

   int i;
   FILE *fp = fopen (filename.c_str(),"w");
   if(!fp) return;

   for (i=0;i<_numPairsAfterPrune;i++)
   {
      /*
        fprintf (fp, "pair: %d ", i);
        fprintf (fp, "type1: %d  ", _objectPairs[i].type1); 
        fprintf (fp, "type2: %d  ", _objectPairs[i].type2);
        fprintf (fp, "crossing: %d  ", _objectPairs[i].crossing);
        fprintf (fp, "area: %f  ", _objectPairs[i].area);
        fprintf (fp, "pageInst: %d  ", _objectPairs[i].pageInstanceCount);
        fprintf (fp, "totalInst: %d  ", _objectPairs[i].totalInstanceCount);
        fprintf (fp, "docCount: %d  ", _objectPairs[i].docsCount);
        fprintf (fp, "v1x: %d  ", _objectPairs[i].vertices[0].x);
        fprintf (fp, "v1y: %d  ", _objectPairs[i].vertices[0].y);
        fprintf (fp, "v2x: %d  ", _objectPairs[i].vertices[1].x);
        fprintf (fp, "v2y: %d  ", _objectPairs[i].vertices[1].y);
        fprintf (fp, "v3x: %d  ", _objectPairs[i].vertices[2].x);
        fprintf (fp, "v3y: %d  ", _objectPairs[i].vertices[2].y);
        fprintf (fp, "v4x: %d  ", _objectPairs[i].vertices[3].x);
        fprintf (fp, "v4y: %d  ", _objectPairs[i].vertices[3].y);
        fprintf (fp, "qual1: %s  ", _objectPairs[i].obj1_qualifier);
        fprintf (fp, "qual2: %s  ", _objectPairs[i].obj2_qualifier);
        fprintf (fp, "intqual1: %d  ", _objectPairs[i].obj1_int_qualifier);
        fprintf (fp, "intqual2: %d \n", _objectPairs[i].obj2_int_qualifier);
      */	  

      fprintf(fp,"pair: %d type1: %d type2: %d crossing: %d area: %f pageInst: %1.0f totalInst: %d docCount: %d v1x: %d v1y: %d v2x: %d v2y: %d ",
              i, 	
              _objectPairs[i].type1,
              _objectPairs[i].type2,
              _objectPairs[i].crossing,
              _objectPairs[i].area,	
              _objectPairs[i].pageInstanceCount,
              _objectPairs[i].totalInstanceCount,
              _objectPairs[i].docsCount,
              _objectPairs[i].vertices[0].x,
              _objectPairs[i].vertices[0].y,
              _objectPairs[i].vertices[1].x,
              _objectPairs[i].vertices[1].y);
	  

      fprintf(fp,"v3x: %d v3y: %d v4x: %d v4y: %d qual1: %s qual2: %s intqual1: %d intqual2: %d \n",
              _objectPairs[i].vertices[2].x,
              _objectPairs[i].vertices[2].y,
              _objectPairs[i].vertices[3].x,
              _objectPairs[i].vertices[3].y,
              _objectPairs[i].obj1_qualifier,
              _objectPairs[i].obj2_qualifier,
              _objectPairs[i].obj1_int_qualifier,
              _objectPairs[i].obj2_int_qualifier);
 
   }
   fclose(fp);

}
/************************************************************/

void objectPairClass::printVertices()
{
   int i;
   for (i=0;i<_numPairs;i++)
   {
      int j;
      for(j=0;j<4;j++)
      {
         printf("%d %d \n",
                _objectPairs[i].vertices[j].x,
                _objectPairs[i].vertices[j].y);
      }
      printf("\n");
   }
}


// Same function as PrintScoreInfo(), added by hm 
void objectPairClass::SaveToDisk(string fname)
{
   ofstream f;
   f.open(fname.c_str(),ios::out);
   if( f.rdstate() & ios::failbit ) 
   {
      printf("open file error!");
      f.close();
      return;
   }	

   f << "numPairsAfterPrune: " <<_numPairsAfterPrune << endl;

   int i;
   for (i=0;i<_numPairsAfterPrune;i++)
   {
      f << "pair: " << i;
      f << " type1: " << _objectPairs[i].type1 ;
      f << " type2: " << _objectPairs[i].type2;
      f << " crossing: " << _objectPairs[i].crossing ;
      f << " area: " << _objectPairs[i].area;
      f << " pageInst: " << _objectPairs[i].pageInstanceCount;
      f << " totalInst: "<< _objectPairs[i].totalInstanceCount;
      f << " docsCount: "<< _objectPairs[i].docsCount;
      f << " v1x: "<<_objectPairs[i].vertices[0].x;
      f << " v1y: "<<_objectPairs[i].vertices[0].y;
      f << " v2x: "<<_objectPairs[i].vertices[1].x;
      f << " v2y: "<<_objectPairs[i].vertices[1].y;
      f << " v3x: "<<_objectPairs[i].vertices[2].x;
      f << " v3y: "<<_objectPairs[i].vertices[2].y;
      f << " v4x: "<<_objectPairs[i].vertices[3].x;
      f << " v4y: "<<_objectPairs[i].vertices[3].y;
      f << " score: " <<_objectPairs[i].score;
      f << " qual1: " <<_objectPairs[i].obj1_qualifier;
      f << " qual2: " <<_objectPairs[i].obj2_qualifier;
      f << " intqual1: " << _objectPairs[i].obj1_int_qualifier;
      f << " intqual2: " << _objectPairs[i].obj2_int_qualifier;
      f <<endl;
			
   }

   f.close();

}

// added by hm 
void objectPairClass::LoadFromDisk(string fname)
{
   ifstream f;
   f.open(fname.c_str(),ios::in);
   if( f.rdstate() & ios::failbit ) 
   {
      printf("open file error!");
      f.close();
      return;
   }	

   char buf[80];
   f >> buf ;
   f >> _numPairsAfterPrune ;
   _numPairs = _numPairsAfterPrune;

   if(_numPairsAfterPrune > CALLOC_BLOCK_SIZE)
   {
      free(_objectPairs);
      _objectPairs = (objectPairInfo *) calloc ( _numPairsAfterPrune, sizeof(objectPairInfo));
      _availableEntries = 0;
   }

   polyMatchClass pm;

   int i;
   for (i=0;i<_numPairsAfterPrune;i++)
   {
      int type1, type2;
      f >> buf; f >> type1; // this is the index
      f >> buf; f >> type1; _objectPairs[i].type1 = objectType(type1) ;
      f >> buf; f >> type2; _objectPairs[i].type2 = objectType(type2);
      f >> buf; f >> _objectPairs[i].crossing ;
      f >> buf; f >> _objectPairs[i].area;
      f >> buf; f >> _objectPairs[i].pageInstanceCount;
      f >> buf; f >> _objectPairs[i].totalInstanceCount;
      f >> buf; f >> _objectPairs[i].docsCount;
      f >> buf; f >> _objectPairs[i].vertices[0].x;
      f >> buf; f >> _objectPairs[i].vertices[0].y;
      f >> buf; f >> _objectPairs[i].vertices[1].x;
      f >> buf; f >> _objectPairs[i].vertices[1].y;
      f >> buf; f >> _objectPairs[i].vertices[2].x;
      f >> buf; f >> _objectPairs[i].vertices[2].y;
      f >> buf; f >> _objectPairs[i].vertices[3].x;
      f >> buf; f >> _objectPairs[i].vertices[3].y;
      f >> buf; f >> _objectPairs[i].score;
      f >> buf; f >> _objectPairs[i].obj1_qualifier;
      f >> buf; f >> _objectPairs[i].obj2_qualifier;
      f >> buf; f >> _objectPairs[i].obj1_int_qualifier;
      f >> buf; f >> _objectPairs[i].obj2_int_qualifier;	

      /* store turning representation to speed up later matching */
      pm.poly_to_turn_rep(_objectPairs[i].vertices,4,	&_objectPairs[i].t);
			
   }

   f.close();

}

/************************************************************/
int objectPairClass::matches(objectPairInfo *pair1,	objectPairInfo *pair2, double *score)
{	int i,j;
 float percent_dif;
 float delta_area;
 polyMatchClass pm;
 pointLocation *pt1 = NULL;
 pointLocation *pt2 = NULL;
 pointLocation *pt3 = NULL;
 pointLocation *pt4 = NULL;

 /* if objects pairs don't have common area, it is only necessary for
    the types to agree and, if either object has point identifiers, 
    the length of the segment connecting must be within a threshold.
    Right now this is coded to assume they don't match*/

 /* make sure types, string, and integer qualifiers agree */

 /* the only integer qualifiler used right now is the line height.  It has to match within 5 pixels. */

 if ((pair1->type1 == pair2->type1 && pair1->type2 == pair2->type2 &&
      (strcmp(pair1->obj1_qualifier,pair2->obj1_qualifier) == 0) &&
      (strcmp(pair1->obj2_qualifier,pair2->obj2_qualifier) == 0) &&
      (abs(pair1->obj1_int_qualifier - pair2->obj1_int_qualifier) <= 5) &&
      (abs(pair1->obj2_int_qualifier - pair2->obj2_int_qualifier) <= 5)) ||
     (pair1->type1 == pair2->type2 && pair1->type2 == pair2->type1  &&
      (strcmp(pair1->obj1_qualifier,pair2->obj2_qualifier) == 0) &&
      (strcmp(pair1->obj2_qualifier,pair2->obj1_qualifier) == 0) &&
      (abs(pair1->obj1_int_qualifier - pair2->obj1_int_qualifier) <= 5) &&
      (abs(pair1->obj2_int_qualifier - pair2->obj2_int_qualifier) <= 5)  ) )
 {

    /* one of the items has no area: parallel textlines */ 
    if ((pair1->area <= 0) || (pair2->area<=0 ))
    {
       /* see if there is a line segment present */
       for (i=0;i<4;i++)
       {
          if (pair1->vertices[i].x > 0)
          {
             if (pt1 == NULL)
                pt1 = &pair1->vertices[i];
             else 
                pt2 = &pair1->vertices[i];
          }
					
          if (pair2->vertices[i].x > 0)
          {
             if (pt3 == NULL)
                pt3 = &pair2->vertices[i];
             else 
                pt4 = &pair2->vertices[i];
          }
       }
					

       /* if any of the points are null the compare can't	be done  assume a match */
       if ((pt1 == NULL) || (pt2 == NULL) ||(pt3 == NULL) || (pt4 == NULL))
          return (1);

       float length1 = (float) (pt1->x - pt2->x)*(pt1->x - pt2->x) + (pt1->y - pt2->y)*(pt1->y - pt2->y) ;
       float length2 = (float) (pt3->x - pt4->x)*(pt3->x - pt4->x) + (pt3->y - pt4->y)*(pt3->y - pt4->y) ;

					
       /* match if length is withing threshold	percentage  
          This needs to be modified to take into account absolute differences*/
       delta_area = length1-length2;
       percent_dif = delta_area >0 ? delta_area/ length1: -delta_area/length2;
						 
       *score = 1 - percent_dif;
       return delta_area >  LINE_MATCH_THRESHOLD_PIXEL_DISTANCE ? -1 : 1; //??
    }//end if one of the polygons has no area


    /* objects are declared a match if the shapes match and
       the difference in area is less then AREA_MATCH_THRESHOLD
       percent of the larger of the areas */

    /* if objects match in type, area, and in shape return a true value */
    delta_area = pair1->area - pair2->area;
    percent_dif = delta_area > 0? delta_area/ pair1->area : -delta_area/ pair2->area;

    /* don't bother with the shape match if the areas are too different */
    /* tbd add code to use the larger of the areas as a basis */
    if (percent_dif > AREA_MATCH_THRESHOLD) return -1;
	
    *score = pm.matchPoly(&pair1->t,&pair2->t);
			
    return (*score < POLY_MATCH_THRESHOLD)? 1 : -1;
			
 }
 /* objects have no common type */
 return (-1);
	
}
/************************************************************/
void objectPairClass::merge(objectPairClass *mergeData,int document_number) 
{
   int merged;
   int source_count = mergeData->getPairCount();

   objectPairInfo *sourceDataStruct = mergeData->getPairs();

   int index = max(0, _numPairsAfterPrune);

   int i,j;
   for (i=0;i<_numPairsAfterPrune;_objectPairs[i++].tag = 0);

   for (i = 0;i<source_count; i++)
   {
      /* see if entry can be merged into an existing entry */
      for (j = 0, merged = 0;j<_numPairsAfterPrune;j++)
      {
         double dist;
         int result  = matches(&_objectPairs[j],	&sourceDataStruct[i],&dist);
         if (result > 0)/* add to this entry */
         {  
            /*tagging type1 as null will signify
              to the destruction instance that he 
              space for the documentIdList can be freed */				

            merged = 1;
				
            _objectPairs[j].totalInstanceCount += sourceDataStruct[i].pageInstanceCount;				
				
            if (_coref_file != NULL)
               fprintf(_coref_file,"%d %d\n",	j,document_number);

            /* up the count of documents this relationship occured on */
            if (_objectPairs[j].tag == 0)
            {
               _objectPairs[j].docsCount  ++;
					
               /* set tag to prevent multiple increments */
               _objectPairs[j].tag = 1;
            }

            break;  //hm
         }
			
      }//end for j


      /* if this entry was not merged create a new entry */
      if (merged == 0)
      {	
         if (_availableEntries == 0)
         {
            /* out of room; allocate space for another block of entries */
            _availableEntries = CALLOC_BLOCK_SIZE;
            _objectPairs = (objectPairInfo *) realloc ( _objectPairs, (CALLOC_BLOCK_SIZE +
                                                                       index)* sizeof(objectPairInfo));
         }

         if (_coref_file != NULL)
            fprintf(_coref_file,"%d %d\n", index,document_number);	

         _availableEntries --;
         memcpy(&_objectPairs[index],&sourceDataStruct[i],sizeof(sourceDataStruct[i]));

         _objectPairs[index].docsCount=1; //hm

         index ++;
      }
   }//end for i

   _numPairsAfterPrune = index;
}

/************************************************************/
void objectPairClass::frequencyBasedPrune(int threshold,FILE *coref_file)
{
/* this method is used when doing scoring based on the polygon frequency of occurance */
/*  It prunes the object pair list down to only those pairs whose occurance count is 
    greater then a threshold.  This will allow the scoring to run much faster.

    At this point pruning occurs on the first 1/2 of the polygons created since the last pruning */


   int cnt = (_numPairsAfterPrune - _frequencyPrunedLevel)/2;

   int pruned_count = 0;

//cout << "Frequency Based Prune " << "Threshold: " << threshold << endl;
//cout << "numPairs before: " << _numPairsAfterPrune << endl;

/* tag all pairs with scores below the threshold for deletion */
   int upper_limit = _frequencyPrunedLevel+ cnt;
//cout << "_frequencyPrunedLevel: " << _frequencyPrunedLevel << " cnt: " << cnt << "  upper_limit: " << upper_limit << endl;

   int i;
   for (i=_frequencyPrunedLevel;i<=upper_limit;i++)
   {
      if (_objectPairs[i].totalInstanceCount <= threshold)
      {
/*		cout << "pruning: " << i << endl; */
         _objectPairs[i].type1 = nullObject;
         pruned_count++;
      }
   }

/* now collapse the entries to that less space is used */
   int index = 0;
   for (i=0;i<_numPairsAfterPrune;i++)
   {
      if (i == upper_limit) _frequencyPrunedLevel = index;
      if (_objectPairs[i].type1 != nullObject)
      {

         if (coref_file != NULL)
            fprintf(coref_file,"oldPolyNumb: %d  newPolyNumb: %d\n",i,index); 

         /* copy to next index */
         if (i != index)
            memcpy(&_objectPairs[index], &_objectPairs[i],sizeof(objectPairInfo));

         index++;
      }
      else
      {
         if (coref_file != NULL)
            fprintf(coref_file,"deletedPoly: %d\n",i);
      }
   }
   /*_frequencyPrunedLevel = upper_limit; */
   _numPairsAfterPrune = index;
   _availableEntries += pruned_count;
//	cout << "numPairs afteR: " << _numPairsAfterPrune << endl;

}


/************************************************************/
void objectPairClass::frequencyBasedPruneForScoring(int threshold)
{
/* this method is used when doing scoring based on the polygon frequency of occurance */
/*  It prunes the object pair list down to only those
    pairs whose occurance count is greater then a threshold.  This will
    allow the scoring to run much faster 

    At this point pruning occurs on the first 1/2 of the polygons created since the last pruning */

   int pruned_count = 0;

//cout << "Frequency Based Prune " << "Threshold: " << threshold << endl;
//cout << "numPairs before: " << _numPairsAfterPrune << endl;
/* tag all pairs with scores below the threshold for deletion */

   int upper_limit = _numPairsAfterPrune;
//cout << "_frequencyPrunedLevel: " << _frequencyPrunedLevel <<"  upper_limit: " << upper_limit << endl;
   int i;
   for (i=_frequencyPrunedLevel;i<=upper_limit;i++)
   {
      if (_objectPairs[i].totalInstanceCount <= threshold)
      {
/*		cout << "pruning: " << i << endl; */
         _objectPairs[i].type1 = nullObject;
         pruned_count++;
      }
   }

/* now collapse the entries to that less space is used */
   int index = 0;
   for (i=0;i<_numPairsAfterPrune;i++)
   {
      if (i == upper_limit) _frequencyPrunedLevel = index;
      if (_objectPairs[i].type1 != nullObject)
      {
         /* copy to next index */
         if (i != index)
            memcpy(&_objectPairs[index], &_objectPairs[i],sizeof(objectPairInfo));
         index++;
      }
      else
      {
         if (_coref_file != NULL)
            fprintf(_coref_file,"deletedPoly: %d\n",i);
      }

   }
   /*_frequencyPrunedLevel = upper_limit; */
   _numPairsAfterPrune = index;
   _availableEntries += pruned_count;
   //cout << "numPairs afteR: " << _numPairsAfterPrune << endl;

}

/************************************************************/
void objectPairClass::scoreBasedPrune(float threshold)
{
   /* this method is used when doing scoring based on object
      pairs.  It prunes the object pair list down to only those
      pairs whose "score" is greater then a threshold.  This will
      allow the scoring to run much faster */

   int pruned_count = 0;
   /* tag all pairs with scores below the threshold fordeletion */

   int i;
   for (i=0;i<_numPairsAfterPrune;i++)
      if (_objectPairs[i].score < threshold)/* if the object pairs match combine them */
      {
         _objectPairs[i].type1 = nullObject;	
         _objectPairs[i].score = 0;
      }


   // now collapse the entries to that less space is used 
   int index = 0;
   for (i=0;i<_numPairsAfterPrune;i++)
      if (_objectPairs[i].type1 != nullObject)
      {
         // copy to next index 

         if (i != index)		
            memcpy(&_objectPairs[index], &_objectPairs[i],sizeof(objectPairInfo));
         index++;
      }


   _numPairsAfterPrune = index;
   _availableEntries += pruned_count;


}

/************************************************************/
void objectPairClass::prune()
{
   objectPairInfo tmpPair;

   int pruned_count = 0;

   /* first combine like entries */
   int i,j;
   for (i=0;i<_numPairs-1;i++) //changed from i<_numPairs-2 : hm
   {
      /* skip pruned entries */
      if (_objectPairs[i].type1 == nullObject) continue;
			
      for (j=i+1;j<_numPairs;j++)
         if (_objectPairs[j].type1 != nullObject)  /* skip pruned entries */
         {
            /* if the object pairs match, combine them */
            double score;
            int result = matches(&_objectPairs[i],&_objectPairs[j],&score) ;
            if (result > 0)
            {
               /* combine the second into the first and nullify the second entry */
               _objectPairs[i].pageInstanceCount ++;
               _objectPairs[j].type1 = nullObject;
               pruned_count++;
            }
         }		

   }//end for i

   /* now collapse the entries so that less space is used */
   int index = 0;
   for (i=0;i<_numPairs;i++)
   {
      if (_objectPairs[i].type1 != nullObject)
      {
         /* copy to next index */
         if (i != index)
         {

            /* make sure not to overwrite entries and swap
               correctly since each entry has space allocated
               for a document occurance list */

            memcpy(&tmpPair,&_objectPairs[index],sizeof(objectPairInfo));
			
            memcpy(&_objectPairs[index], &_objectPairs[i],sizeof(objectPairInfo));

            memcpy(&_objectPairs[i],&tmpPair,sizeof(objectPairInfo));

            _objectPairs[index].totalInstanceCount = _objectPairs[index].pageInstanceCount;
         }
         index++;
      }
   }
   _numPairsAfterPrune = _numPairs - pruned_count;

   /* adjust the number of available entries */
   _availableEntries += pruned_count;

}

/**
 * prune2 cluster/mergy object pairs: when adding a new pair to an existing cluster which is 
 * the nearest one to the new pair and their distance is within a threshold, the cluster center
 * is dynamically updated to its member who has the minimum sum of distance to all other members.
 * \param isMerge flag indicating whether the source pairs are from different documents. If true,
 * the function will use an array to record cluster member occurence in each document.
 * \param reallocmemory when set true, reallocated memory size can hold 
 * (CALLOC_BLOCK_SIZE + #clusters) number of clusters; if false, the reallocated memory can only 
 * hold (#clusters number)of clusters.
 **/
void objectPairClass::prune2(bool isMerge, bool reallocmemory) //hm
{
   if(_numPairs <=1) return;

   //get initial cluster centers
   int cluster_count = 1;
   pairCluster * pClusters = new pairCluster[_numPairs];
   pClusters[0].memberlist.Construction(10);
   pClusters[0].Add(0);
   pClusters[0].centroid = 0;

   polyMatchClass pm;
   int i,j;
   for (i=1;i<_numPairs;i++) 
   {
      double min_dist = 1;
      int nearest_cluster = -1;
      int former_nMember = 0;

      //looking for the nearest cluster
      for (j=0;j<cluster_count;j++)
      {
         int centroid_index = pClusters[j].centroid;
         double dist;
         int result = matches(&_objectPairs[i],&_objectPairs[centroid_index],&dist) ;
         if (result > 0 && (dist < min_dist || dist == min_dist && pClusters[j].nMember > former_nMember))
         {
            nearest_cluster = j;
            min_dist = dist;
            former_nMember = pClusters[j].nMember;
         }
      }

      if(nearest_cluster >=0) //add polygon i to the nearest cluster
      {
         //modify centroid
         double sum = min_dist;
         int k;
         for(k=0; k<pClusters[nearest_cluster].nMember; k++)
         {
            int index = pClusters[nearest_cluster].memberlist[k];
            if( index == pClusters[nearest_cluster].centroid ) continue;
            sum += pm.matchPoly(&(_objectPairs[i].t), &(_objectPairs[index].t));
         }
         if(sum < pClusters[nearest_cluster].sum_dist)
         {
            pClusters[nearest_cluster].centroid = i;
            pClusters[nearest_cluster].sum_dist = sum;
         }
         pClusters[nearest_cluster].Add(i);			
      }			
      else //forms a new cluster
      {
         pClusters[cluster_count].memberlist.Construction(10);
         pClusters[cluster_count].Add(i);
         pClusters[cluster_count].centroid = i;
         cluster_count++;
      }
   }

   //redistribute polygons

   //clear member list of each cluster, just keep the centroid index
   for (i=0;i<cluster_count;i++)
   {
      pClusters[i].memberlist.Construction(10);
      pClusters[i].nMember = 0;
   }

   for(i =0; i<_numPairs; i++)
   {
      double min_dist = 1000;
      int nearest_cluster = -1;
      int former_nMember = 0;

      //looking for the nearest cluster
      for (j=0;j<cluster_count;j++)
      {
         int centroid_index = pClusters[j].centroid;
         double dist;
         int result = matches(&_objectPairs[i],&_objectPairs[centroid_index],&dist) ;
         if (result > 0 && (dist < min_dist || dist == min_dist && pClusters[j].nMember > former_nMember))
         {
            nearest_cluster = j;
            min_dist = dist;
            former_nMember = pClusters[j].nMember;
         }
      }

      if(nearest_cluster >=0) //put polygon i into its nearest bin/cluster
         pClusters[nearest_cluster].Add(i);
      else //forms a new cluster
      {
         pClusters[cluster_count].memberlist.Construction(10);
         pClusters[cluster_count].Add(i);
         pClusters[cluster_count].centroid = i;
         cluster_count++;
      }
   }

   //dump clusters to member variable _objectPairs
   _numPairsAfterPrune = cluster_count;	

   int dest_memory = reallocmemory?  CALLOC_BLOCK_SIZE+cluster_count: cluster_count ;
   _availableEntries = dest_memory - cluster_count;

   objectPairInfo * tmp = (objectPairInfo *) calloc(dest_memory, sizeof(objectPairInfo));
	
   for(i=0;i<cluster_count;i++)
   {
      int centroid = pClusters[i].centroid;
      tmp[i] = _objectPairs[centroid];
		
      tmp[i].totalInstanceCount = 0;
      int j;
      for(j=0; j<pClusters[i].nMember;j++)
      {
         tmp[i].totalInstanceCount += _objectPairs[pClusters[i].memberlist[j]].pageInstanceCount;
         if(isMerge) //Merge polygon clusters from different documents
         {				
            if(tmp[i].doc_array.IsNull())
               tmp[i].doc_array.Construction(_numDoc);
				
            //label the document in which the polygon appears
            int index_doc = _objectPairs[pClusters[i].memberlist[j]].doc_index;
            tmp[i].doc_array[index_doc] ++;
            tmp[i].pageInstanceCount = tmp[i].totalInstanceCount/(double)_numDoc;
         }
      }
      if(!isMerge)
         tmp[i].pageInstanceCount = tmp[i].totalInstanceCount;

   }

   free(_objectPairs);
   _objectPairs = tmp;

   delete []pClusters;
}

double polydistance(pointLocation* p1, pointLocation* p2) //obsolete , hm
{
   return 1;
}


//Nov. 2004, hm
bool objectPairClass::match2(objectPairInfo *pair1, objectPairInfo *pair2, double* dist)
{
   // only parallel and no-intersection have the potential to be matched
   // any other types of crossing should be one to one correspond 
   // crossing : -3(notdefined), -2(intersect), -1(colinear), 0(parallel), 1(no-intersection)
   if(pair1->crossing * pair2->crossing <0 ) 
      return false;

   if(pair1->crossing * pair2->crossing == 0 && pair1->crossing + pair2->crossing <0 )
      return false;

   /* make sure types, string, and integer qualifiers agree */
   /* the only integer qualifiler used right now is the line height.  It has to match within 5 pixels. */
   if ((pair1->type1 == pair2->type1 && pair1->type2 == pair2->type2 &&
        (abs(pair1->obj1_int_qualifier - pair2->obj1_int_qualifier) <= 5) &&
        (abs(pair1->obj2_int_qualifier - pair2->obj2_int_qualifier) <= 5)) ||
       (pair1->type1 == pair2->type2 && pair1->type2 == pair2->type1  &&
        (abs(pair1->obj1_int_qualifier - pair2->obj1_int_qualifier) <= 5) &&
        (abs(pair1->obj2_int_qualifier - pair2->obj2_int_qualifier) <= 5)  ) )
   {

      /* both are colinear lines */ 
      if (pair1->crossing == coincident )
      {
         //length1 : length of the first segment of polygon 1
         float length1 =  sqrt((float)(pair1->vertices[0].x - pair1->vertices[1].x)*(pair1->vertices[0].x - pair1->vertices[1].x)
                               +(pair1->vertices[0].y - pair1->vertices[1].y)*(pair1->vertices[0].y - pair1->vertices[1].y));

         //length2 : length of the second segment of polygon 1
         float length2 =  sqrt((float)(pair1->vertices[2].x - pair1->vertices[3].x)*(pair1->vertices[2].x - pair1->vertices[3].x)
                               +(pair1->vertices[2].y - pair1->vertices[3].y)*(pair1->vertices[2].y - pair1->vertices[3].y));
				
         //length3 : length of the first segment of polygon 2
         float length3 = sqrt((float) (pair2->vertices[0].x - pair2->vertices[1].x)*(pair2->vertices[0].x - pair2->vertices[1].x)
                              +(pair2->vertices[0].y - pair2->vertices[1].y)*(pair2->vertices[0].y - pair2->vertices[1].y));
				
         //length4 : length of the second segment of polygon 2
         float length4 =  sqrt(((float)pair2->vertices[2].x - pair2->vertices[3].x)*(pair2->vertices[2].x - pair2->vertices[3].x)
                               +(pair2->vertices[2].y - pair2->vertices[3].y)*(pair2->vertices[2].y - pair2->vertices[3].y));
			
         float segment_diff1 = fabs(length1-length3) + fabs(length2 - length4);
         float segment_diff2 = fabs(length1-length4) + fabs(length2 - length3);

         float segment_diff = min(segment_diff1, segment_diff2);

         // gap between the two segments of polygon1
         float gap1 =  sqrt((float)(pair1->vertices[1].x - pair1->vertices[2].x)*(pair1->vertices[1].x - pair1->vertices[2].x)
                            +(pair1->vertices[1].y - pair1->vertices[2].y)*(pair1->vertices[1].y - pair1->vertices[2].y));

         // gap between the two segments of polygon1
         float gap2 =  sqrt((float)(pair2->vertices[1].x - pair2->vertices[2].x)*(pair2->vertices[1].x - pair2->vertices[2].x)
                            +(pair2->vertices[1].y - pair2->vertices[2].y)*(pair2->vertices[1].y - pair2->vertices[2].y));
				
         float gap_diff = abs(gap1 - gap2);
					
         * dist = sqrt(segment_diff * gap_diff);
						 
         return *dist < 0.5 ; //threshold need to be further considered
      }					

      //both are normal polygons			
      float delta_area = pair1->area - pair2->area;
      float percent_dif = delta_area > 0? delta_area/ pair1->area : -delta_area/ pair2->area;

      /* don't bother with the shape match if the areas are too different */			
      if (percent_dif > AREA_MATCH_THRESHOLD) return false;
	
      *dist = polydistance(pair1->vertices, pair2->vertices); //Jan, 25, 2005. unfinished
			
      return (*dist < 0.6)? true : false; //threshold should be further tested
			
   }
	 
   return false;

}


/*
  double objectPairClass::MyTestScore(objectPairClass *pTest)
  {
  int nTestObj = pTest->_numPairsAfterPrune;

  CTArray<int>	 hit_histo(_numPairsAfterPrune);
  CTArray<double>  hit_score(_numPairsAfterPrune);  // infact, hit distance 

  int unhitted_obj=0;
  for(int i=0;i<nTestObj;i++)
  {
  objectPairInfo *obj = &pTest->_objectPairs[i];

  double mindist = 0.5; 
  int index = -1;
  for(int j=0;j<_numPairsAfterPrune;j++)
  {
  // look for the cluster that current object hit
  double dist;
  int result = matches(&_objectPairs[j],obj,&dist) ;
  if (result > 0 && dist < mindist )
  {
  mindist  = dist;
  index = j;
  }
  }

  if(index >=0) // some cluster is hit
  {
  hit_histo[index] += pTest->_objectPairs[i].totalInstanceCount;// record how many objects hit the same cluster
			
  hit_score[index] += exp(-mindist*mindist/(2*0.1)); // record the average hitting score
  }
  else
  unhitted_obj += pTest->_objectPairs[i].totalInstanceCount;	
  }

  //use x^2 statistics to measure histogram difference -- inter cluster difference
  double x_square = 0;
  for(i=0;i<_numPairsAfterPrune;i++)
  x_square +=   (hit_histo[i] - _objectPairs[i].pageInstanceCount) 
  * (hit_histo[i] - _objectPairs[i].pageInstanceCount)
  / (2* _objectPairs[i].nPairPerDoc_sigma);

  x_square += (unhitted_obj - nTestObj*0.1)*
  (unhitted_obj- nTestObj*0.1)/(2.0* 30);

  x_square /= _numPairsAfterPrune;
	

  double intra_cluster_diff = 0; // the sum of the score got for each individual cluster
  int n_hitCluster = 0;
  for(i=0;i<_numPairsAfterPrune;i++)
  {
  if(hit_histo[i] <1) continue;
  intra_cluster_diff += (hit_score[i]/hit_histo[i]);
  n_hitCluster++;
  }

  if(n_hitCluster == 0) return 0;

  double final_prob = intra_cluster_diff/n_hitCluster 
  * exp(-x_square) // - fabs(m_nAvgVperDoc - nTestObj)/(m_nAvgVperDoc+nTestObj)); 
  * ((nTestObj - unhitted_obj)/(double)nTestObj) ;
	
  return final_prob;
  }

*/

//void objectPairClass::ComputeStat()
//{
//	//statistic infomation
//	for(int i=0; i<_numPairsAfterPrune; i++)
//	{
//		double avg = _objectPairs[i].pageInstanceCount;
//		_objectPairs[i].nPairPerDoc_sigma = 0;
//		for(int j=0;j <_numDoc; j++)
//			_objectPairs[i].nPairPerDoc_sigma +=  (_objectPairs[i].doc_array[j] - avg) 
//												* (_objectPairs[i].doc_array[j] - avg);
//		_objectPairs[i].nPairPerDoc_sigma /= _numDoc;
//		_objectPairs[i].nPairPerDoc_sigma = sqrt(_objectPairs[i].nPairPerDoc_sigma);
//
//		if(_objectPairs[i].nPairPerDoc_sigma < 1e-4)
//			_objectPairs[i].nPairPerDoc_sigma = 1e-4;
//		
//	}
//}


/************************************************************/
void objectPairClass::setNormalizationFactor(float *normalizationFactor) 
{
   _normalizationFactor = *normalizationFactor;

}

/************************************************************/
void objectPairClass::setLookupBins(valarray <float> precision)
{

   _lookupBins.resize(precision.size(),0);
//_lookupBins =  valarray < float > (precision.size());
   _lookupBins = precision;

}
/************************************************************/

float objectPairClass::lookupScore(float *index)

{
   float fval;

   fval = _lookupBins[(int) *index];
   return(fval);

}
