#ifndef __objectPairClass_H
#define __objectPairClass_H

#ifndef __objectInfoClass_H
#include "objectInfoClass.h"
#endif

#include <valarray>
#include "../Utility_hm/TArray.h"

#define CALLOC_BLOCK_SIZE 20000
#define DOCLIST_INIT  500
#define AREA_MATCH_THRESHOLD  .1
#define AREA_MATCH_THRESHOLD_PIXEL_AREA  .1
#define POLY_MATCH_THRESHOLD  .5
#define LINE_MATCH_THRESHOLD_PERCENT  .1

/* 50 pixels equates to about 1/4 inch */
#define LINE_MATCH_THRESHOLD_PIXEL_DISTANCE 50

/*
  #ifndef __polyClass_H
  #include "polyClass.h"
  #endif
*/
#ifndef __polyMatchClass_H
#include "polyMatchClass.h"
#endif

#define MAX_QUALIFIER_LEN 80

struct objectPairInfo
{
   int			object1;	/* used to store the first object number in the pair -not stored */
   int			object2;	/* used to store the second object number in the pair -not stored*/
   objectType	type1;		/* note the types are defined in objectInfoClass.h> */
   objectType	type2;
   char		obj1_qualifier[MAX_QUALIFIER_LEN];
   char		obj2_qualifier[MAX_QUALIFIER_LEN];
   int			obj1_int_qualifier;
   int			obj2_int_qualifier;
   pointLocation      vertices[4];

   float		pageInstanceCount;     /* occurances on page; avg_per doc after merge*/
   int			docsCount;             /* # docs containing this pair */
   int			totalInstanceCount;
   int			availableInstanceCount;
   float		area;
   float		area_upper_thresh;
   float		area_lower_thresh;
   int			crossing;
   int			tag;
   float		score;		 /* score used to measure how unique pair is to data set relative to another data set */

   TURN_REP_REC t;

   public:
   double nPairPerDoc_sigma;

   int          doc_index;  //hm	
   CTArray<int> doc_array;  //hm, for clustering 

   objectPairInfo& operator = (objectPairInfo a)
   {
      object1 = a.object1;
      object2 = a.object2;
      type1   = a.type1;
      type2   = a.type2;
      strcpy(obj1_qualifier, a.obj1_qualifier);
      strcpy(obj2_qualifier, a.obj2_qualifier);
      obj1_int_qualifier = a.obj1_int_qualifier;
      obj2_int_qualifier = a.obj2_int_qualifier;
      int i;
      for(i=0;i<4;i++)
         vertices[i] = a.vertices[i];

      pageInstanceCount = a.pageInstanceCount;
      docsCount = a.docsCount;
      totalInstanceCount = a.totalInstanceCount;
      availableInstanceCount = a.availableInstanceCount;
      area = a.area;
      area_upper_thresh = a.area_upper_thresh;
      area_lower_thresh = a.area_lower_thresh;
      crossing = a.crossing;
      tag = a.tag;
      score = a.score;
      memcpy(&t, &a.t, sizeof(TURN_REP_REC));
		
      doc_index = a.doc_index;
      doc_array = a.doc_array;

      return *this;
   }
};


// local structure used in objectPairClass::prune2();   --- hm 
struct pairCluster
{
   int				centroid;			// index of centroid object pair
   int				nMember;
   double			sum_dist;			// sum of distance from each object pair to centroid pair
   CTArray<int>	memberlist;			// index of all members belong to one cluster
   pairCluster()
   {
      sum_dist = 0;
      nMember = 0;
      centroid = -1;
   }
   void Add(int index)				   // appending a new member into current cluster member list
   {
      if(nMember == memberlist.GetDimension())
      {
         CTArray<int> tmp = memberlist;
         memberlist.Construction(nMember + 11);
         int i;
         for(i=0;i<nMember;i++)
            memberlist[i] = tmp[i];
      }
      memberlist[nMember++] = index;
   }
};


class objectPairClass 
{
   public:	

   // constructor and distructor
   objectPairClass();
   objectPairClass(FILE *fp);
   objectPairClass(const char *data_path,FILE *fp); /* opens data from file*/
   objectPairClass(struct objectInfo *obj1, int object_count,int documentId,FILE *fp);
   ~objectPairClass();
	
   // loading data from disk, called in constructor
   void readDataWithScores(FILE *fp);

   // retrieve routines
   int  getPairCount() const {return( _numPairsAfterPrune);}
   objectPairInfo *getPairs() const {return(_objectPairs);}

   // weighting wanted polygon clusters against unwanted clusters
   void assignScore (objectPairClass *otherClass); // int nDoc_wanted, int nDoc_unwanted); //updated by hm
   void scoreImage(objectPairClass *objects,float *score, int neighbor);  //updated by hm
   void dumpHighScoringObjects(struct objectInfo *obj1,float *threshold);

   // Save training results to disk
   void printInfoSummary(char *filename);
   void printVertices();
   void printInfo(string filename);
   void printScoreInfo(string filename);

   // merge similar polygons
   void merge(objectPairClass *newData, int new_doc_number);
   void prune();	
   void prune2(bool isMerge = false, bool reallocmemory = true); //added by hm
   //double	MyTestScore(objectPairClass *pTest);

   // post prune: to delete low weight wanted clusters
   void scoreBasedPrune(float threshold);
   void frequencyBasedPrune(int threshold, FILE * coref_file);
   void frequencyBasedPruneForScoring(int threshold);	
	
   // compute similarity between two polygons
   int  matches(objectPairInfo *pair1,  objectPairInfo *pair2, double *score = 0);
   bool match2(objectPairInfo *pair1, objectPairInfo *pair2, double* dist); //hm, obsolete

   float	getNormalizationFactor() const {return( _normalizationFactor);}
   void	setNormalizationFactor(float *normalizationFactor); 
   void	setLookupBins(valarray <float> precision);
   float	lookupScore(float *normalizedScore);

   float				corrective_factor;
   objectPairInfo *	_objectPairs;
   FILE *				_coref_file;
   int					_numPairs;
   int					_numPairsAfterPrune;
   int					_availableEntries;			//used for memory reallocation
   int					_frequencyPrunedLevel;

   int					_numDoc; //number of doc where _objectPairs come from

   float				_normalizationFactor;
   valarray <float>	_lookupBins;

   void SaveToDisk(string fname);    // added by hm
   void LoadFromDisk(string fname);
};

class objectPairMatrix 
{

   public:

   objectPairMatrix (vector<std::string> inputVector,
                     FILE *fp,char *directory_path, char *final_pair_filename,
                     JScript *scriptId);

   private:


};


#endif
