#ifndef __objectPairClass_H
#define __objectPairClass_H

#ifndef __objectInfoClass_H
#include <objectInfoClass.h>
#endif

#ifndef __polyClass_H
#include <polyClass.h>
#endif

#ifndef __polyMatchClass_H
#include <polyMatchClass.h>
#endif

#define MAX_QUALIFIER_LEN 20
/* note the types are defined in objectInfoClass.h> */

struct objectPairInfo
{
   objectType type1;
   objectType type2;
   char obj1_qualifier[MAX_QUALIFIER_LEN];
   char obj2_qualifier[MAX_QUALIFIER_LEN];
   int obj1_int_qualifier;
   int obj2_int_qualifier;
   pointLocation vertices[4];
   int pageInstanceCount;     /* occurances on page; meaningless after merge*/
   int docsCount;             /* # docs containing this pair */
   int totalInstanceCount;
   int availableInstanceCount;
   float area;
   float area_upper_thresh;
   float area_lower_thresh;
   int crossing;
   int tag;
   float score;		/* score used to measure how unique pair is to data set relative
                           to another data set */
   TURN_REP_REC t;
   int object1;  /* used to store the first object number in the pair -not stored */
   int object2;  /* used to store the first object number in the pair -not stored*/
};

class objectPairClass {

   public:

   objectPairClass(FILE *fp);
   objectPairClass(char *data_path,FILE *fp); /* opens data from file*/
   objectPairClass(struct objectInfo *obj1,
                   int object_count,int documentId,FILE *fp);
   objectPairClass::~objectPairClass();
   int getPairCount() const {return( _numPairsAfterPrune);}
   void printInfo(char *filename);
   void printScoreInfo(char *filename);
   void assignScore (objectPairClass *otherClass);
   void scoreImage(objectPairClass *objects,float *score,int storeScoreFlag);
   void objectPairClass::dumpHighScoringObjects(struct objectInfo *obj1,
                                                float *threshold);
   void deleteDocList();
   void printInfoSummary(char *filename);
   void printVertices();
   objectPairInfo *getPairs() const {return(_objectPairs);}
   void merge(objectPairClass *newData, int new_doc_number);
   void objectPairClass::scoreBasedPrune(float *threshold);
   void objectPairClass::frequencyBasedPrune(int threshold, FILE * coref_file);
   void objectPairClass::frequencyBasedPruneForScoring(int threshold);

   private:


   void objectPairClass::readDataWithScores(FILE *fp);
   int objectPairClass::matches(objectPairInfo *pair1,
                                objectPairInfo *pair2);
   void objectPairClass::prune();
   struct objectPairInfo *_objectPairs;
   FILE *_coref_file;
   int _numPairs;
   int _numPairsAfterPrune;
   int _availableEntries;
   int _frequencyPrunedLevel;


};

class objectPairMatrix {

   public:

   objectPairMatrix (vector<std::string> inputVector,
                     FILE *fp,char *directory_path, char *final_pair_filename);

   private:


};


#endif




