#ifndef __polyMatchClass_H
#define __polyMatchClass_H


/*
 * Largest polygon to be handled.
 */
#define MAX_PTS 4


/* Typedefs */

//typedef double COORD;       /* Cartesian coordinate */

typedef struct leg {        /* single leg of a turning rep polygon */
   double theta;           /* heading of the leg */
   double len;             /* length in original coordinates */
   double s;               /* cumulative arc length in [0,1] of start */
} LEG;

typedef struct turnrep {    /* polygon in turning rep */
   int n;
   double total_len;
   LEG leg[MAX_PTS];
} TURN_REP_REC, *TURN_REP;

typedef struct event {      /* critical event */
   double t;               /* "f shift" parameter of the event */
   int fi, gi;             /* pointers into turn reps f and g */
}   EVENT_REC, *EVENT,
   HEAP[MAX_PTS+1];

/*
 * These are to make a turning rep look infinite.
 * Given the rep and the "index" of a discontinuity,
 * 0,1,..., these return s and theta adjusted by
 * the correct number of turns around the polygon.
 */
/*#define tr_n(tr)        tr->n */
#define tr_n(tr)        4
#define tr_i(tr, i)     ((i) % tr->n)
#define tr_len(tr, i)   tr->leg[tr_i(tr, (i))].len
#define tr_s(tr,i)      (tr->leg[(i) % tr->n].s + (i)/ tr->n)
#define tr_theta(tr,i)  (tr->leg[(i) % tr->n].theta + (i)/tr->n*2*M_PI)
#define sqr(x)          ((x) * (x))
#define tr_smt(tr,i, t) ((tr->leg[(i) % tr->n].s -(t)) + (i)/tr->n)
#define pmin(x,y)       (x < y ? x : y)


class polyMatchClass 
{

   public:

   polyMatchClass();
   double matchPoly(TURN_REP f, TURN_REP g, int cnt=4); 
   double getScore() const{return(_score);}
   void poly_to_turn_rep(pointLocation *vertices,	int polySideCount,TURN_REP t);

   private:

   double	tr_s_old(TURN_REP tr, int i); 
   double	tr_smt_old(TURN_REP tr, int i, double t); 
   double	tr_theta_old(TURN_REP tr, int i); 
   double	sqr_old(double x); 
   int		pmin_old(int x, int y); 
   int		ilog2(int x);
   double	turn(double a, double base);
   void	rotate_turn_rep(TURN_REP t, int to, TURN_REP r);
   void	init_vals(TURN_REP f, TURN_REP g,
                  double *ht0_rtn, double *slope_rtn, double *a_rtn);
   void	reinit_vals(TURN_REP f, TURN_REP g, 
                    int fi, int gi, double *ht0_rtn, double *slope_rtn);
   int		reinit_interval(TURN_REP f, TURN_REP g);
   void	add_event(TURN_REP f, TURN_REP g, int fi, int gi);
   EVENT_REC next_event(void);
   void	init_events(TURN_REP f, TURN_REP g);
   double	h_t0min(TURN_REP f, TURN_REP g,
                        double hc0, double slope, double alpha, int d_update,
                        double *theta_star_rtn, EVENT e_rtn,
                        double *hc0_err_rtn, double *slope_err_rtn);

   private:
   double _score;

};

#endif

