
#include "pairshape.h"
#include "math.h"

#include "../Layout_hm/layout.h"
#include "../ObjectVector_hm/objcluster.h"
#include "../Utility_hm/TArray.h"
#include "../Utility_hm/TMatrix.h"

//static pairshape_vector default_sigma(2e-8,2e-8,2e-8, 1e-8, 1e-8);
static pairshape_vector default_sigma(1e-6,1e-6,1e-6, 1e-6, 1e-6);

/**
 * MidPoint calculates the midle 2D point's coordinates between two points
 * \param p1 one point
 * \param p2 the second point
 * \return the middle point of the two
 **/
pointLocation Midpoint(pointLocation p1, pointLocation p2)
{
   pointLocation ret = p1 + p2;
   ret.x = ( ret.x + 1)/2;
   ret.y = (ret.y + 1)/2;
	
   return ret;
}

/**
 * LengthBetween calculates the length between two points
 * \param p1 one point
 * \param p2 the second point
 * \return the length
 **/
double LengthBetween(pointLocation p1, pointLocation p2)
{
   return sqrt((double)(p1.x - p2.x) * (p1.x -p2.x) +
               (p1.y - p2.y) * (p1.y -p2.y) );
}

/*********************************************************************
 ***** struct pairshape_vector implementations*************************
 **********************************************************************/
/**
 * Construction generates an instance from two text line object
 * \param O1 one text line object
 * \param O2 the second text line object
 **/
void pairshape_vector::Construction(struct objectInfo& O1, struct objectInfo& O2)
{
   objectInfo obj1 = O1;
   objectInfo obj2 = O2;
   pointLocation center1 = Midpoint(O1.location[0] , O1.location[1]);
   pointLocation center2 = Midpoint(O2.location[0] , O2.location[1]);
	
   if( center1.y > center2.y || 
       center1.y == center2.y && center1.x > center2.x)
   {
      obj1 = O2; obj2 = O1;
   }	

   type1 = obj1.type;
   type2 = obj2.type;

   FontHeight1 = obj1.int_qualifier;
   FontHeight2 = obj2.int_qualifier;
			
   Len1 =	LengthBetween(obj1.location[0], obj1.location[1]);			
   Len2 =	LengthBetween(obj2.location[0], obj2.location[1]);
			
   pointLocation midpoint1 = Midpoint(obj1.location[0] , obj1.location[1]);
   pointLocation midpoint2 = Midpoint(obj2.location[0] , obj2.location[1]);
   Len3 =	LengthBetween(midpoint1, midpoint2);

   // for 5 lengths vector
   pointLocation left1 = obj1.location[0];
   pointLocation right1 = obj1.location[1];
   if(left1.x > obj1.location[1].x) 
   {
      left1 = obj1.location[1];
      right1 = obj1.location[0];
   }
   pointLocation left2 =  obj2.location[0];
   pointLocation right2 = obj2.location[1];
   if(left2.x > obj2.location[1].x) 
   {
      left2 = obj2.location[1];
      right2 = obj2.location[0];
   }
   alpha = LengthBetween(left1, right2);
   belta = LengthBetween(right1, left2);

	
/*	// angle between first textline and the midpoint-line, range [0,2PI)
	CPoint right1 = obj1.location[0].x < obj1.location[1].x ?
        obj1.location[1] : obj1.location[0];
	double Len_tmp = LengthBetween(right1, midpoint2);

	double a = Len1/2.0;
	double b = Len3; 
	double c = Len_tmp;
	double s = (a + b + c )/2.0;
	double tmp = s*(s-a)*(s-b)*(s-c);
	if(tmp <0) tmp = 0;
	double area = sqrt(tmp);	

	tmp = 2*area/a/b; 
	if( tmp >1) tmp = 1;
	alpha = asin(tmp)+ PI/2;

	//angle between second textline and the midpoint-line	
	CPoint right2 = obj2.location[0].x < obj2.location[1].x ?
        obj2.location[1] : obj2.location[0];
	Len_tmp = LengthBetween(right2, midpoint1);

	a = Len2/2.0;
	c = Len_tmp;
	s = (a + b + c )/2.0;
	tmp = s*(s-a)*(s-b)*(s-c);
	if(tmp <0) tmp = 0;
	area = sqrt(tmp);

	tmp = 2*area/a/b; 
	if( tmp >1) tmp = 1;
	belta = asin(tmp)+ PI/2;	

	ASSERT(belta>=0);
*/
   Normalization();

}

/**
 * Construction generates an instance using a text line object and a referrence line
 * \param obj the text line object
 * \param ref1 the leftpoint of the reference line
 * \param ref2 the rightpoint of the reference line
 **/
void pairshape_vector::Construction(objectInfo &obj, pointLocation &ref1, pointLocation &ref2)
{
   type1 = -1;
   type2 = obj.type;

   FontHeight1 = 0;
   FontHeight2 = obj.int_qualifier;
			
   Len1 =	LengthBetween(ref1, ref2);			
   Len2 =	LengthBetween(obj.location[0], obj.location[1]);
			
   pointLocation midpoint1 = Midpoint(ref1,ref2);
   pointLocation midpoint2 = Midpoint(obj.location[0] , obj.location[1]);
   Len3 =	LengthBetween(midpoint1, midpoint2);

   // for 5 lengths vector
   pointLocation left2 =  obj.location[0];
   pointLocation right2 = obj.location[1];
   if(left2.x > obj.location[1].x) 
   {
      left2 = obj.location[1];
      right2 = obj.location[0];
   }
   alpha = LengthBetween(ref1, right2);
   belta = LengthBetween(ref2, left2);

   Normalization();

}

/**
 * Normalization normalize each member variable to be within [0,1]
 * through dividing them by the maximum possible values
 **/
void pairshape_vector::Normalization()
{
/*
  #define MAX_LEN			2000
	
  Len1 /= MAX_LEN;
  Len2 /= MAX_LEN;
  Len3 /= MAX_LEN;

  alpha /= PI;
  belta /= PI;
*/

/*
  #define LENGTH_MEAN		1000  //0.5
  #define LENGTH_DEV		50    //30

  #define ANGLE_MEAN   PI/2			//0.5
  #define ANGLE_DEV	 PI/36 			//1.0/36

  Len1 = (Len1- LENGTH_MEAN)/LENGTH_DEV;
  Len2 = (Len2- LENGTH_MEAN)/LENGTH_DEV;
  Len3 = (Len3- LENGTH_MEAN)/LENGTH_DEV;

  alpha = (alpha - ANGLE_MEAN) / ANGLE_DEV;
  belta = (belta - ANGLE_MEAN) / ANGLE_DEV;
*/

   Len1 /= 2500;
   Len2 /= 2500;
   Len3 /= 2500;
   alpha /= 2500;
   belta /= 2500;
}	


/**
 * Reverse reverses the position of the two text line objects that formed the instance
 * \return the reversed instance
 **/
pairshape_vector pairshape_vector::Reverse()
{
   pairshape_vector b=*this;

   b.FontHeight1 = FontHeight2;
   b.FontHeight2 = FontHeight1;

   b.type1 = type2;
   b.type2 = type1;

   b.Len1 = Len2;
   b.Len2 = Len1;
   b.Len3 = Len3;
/*	b.alpha = 1- belta;
	b.belta = 1- alpha;
*/
/*	//undo Normalization
	double tmp = belta * ANGLE_DEV + ANGLE_MEAN;
	b.alpha = PI - tmp;//1- tmp;

	tmp = alpha * ANGLE_DEV + ANGLE_MEAN;
	b.belta = PI - tmp; //1-tmp;

	//redo Normalization for angles
	b.alpha = (b.alpha - ANGLE_MEAN) / ANGLE_DEV;
	b.belta = (b.belta - ANGLE_MEAN) / ANGLE_DEV;

*/

   return b;
}

/**
 * Mirror reflects the quadrilateral
 * \return the mirrored instance
 **/
pairshape_vector pairshape_vector::Mirror()
{
   pairshape_vector b = *this;

   b.alpha = 1 - alpha;
   b.belta = 1 - belta;
   return b;

}

/**
 * match calculates the distance between current instance and a given instance
 * \param a a given instance
 * \param upside a flag indicating whether consider the distance between the current 
 * instance and the reversed instance of the given one
 * \return the miniumun distance 
 **/
double pairshape_vector::match(pairshape_vector &a, bool upside)
{
   double dist1 = 1e10, dist2=1e10;
   if( abs(FontHeight1-a.FontHeight1) < 5 && abs(FontHeight2-a.FontHeight2) <5 )
   {
      pairshape_vector diff = *this - a;
      dist1 = diff.norm();
   }

   if(abs(FontHeight1-a.FontHeight2) < 5 && abs(FontHeight2-a.FontHeight1) <5 )
   {
      pairshape_vector r = Reverse();
      pairshape_vector diff = r - a;
      dist2 = diff.norm();
   }

   return min(dist1, dist2);

}

/**
 * operator = overides the assign operator
 **/
pairshape_vector pairshape_vector::operator = (pairshape_vector a)
{
   type1			= a.type1;
   type2			= a.type2;

   FontHeight1		= a.FontHeight1;
   FontHeight2		= a.FontHeight2;

   Len1			= a.Len1;
   Len2			= a.Len2;
   Len3			= a.Len3;
		
   alpha			= a.alpha;
   belta           = a.belta;

   doc_ID			= a.doc_ID;

   return *this;
}

/**
 * operator == overides the equal relational operator
 **/
bool pairshape_vector::operator == (pairshape_vector a)
{
   if (type1 != a.type1 || type2 != a.type2) return false;
   if (FontHeight1 != a.FontHeight1 ||
       FontHeight2 != a.FontHeight2 ) return false;
		
   if (Len1 != a.Len1 || Len2 != a.Len2 || Len3 != a.Len3) return false;
		
   if (alpha != a.alpha || belta != a.belta) return false;

   return true;
}

/**
 * norm returns the Euclidean norm of the vector
 * */
double pairshape_vector::norm()
{
   return sqrt(normsquare());
}

/**
 * normsquare returns the square of the vector norm
 **/
double pairshape_vector::normsquare()
{
   return Len1*Len1 + Len2*Len2+ Len3*Len3 + alpha*alpha + belta*belta;
}

/**
 * Enc_dist returns the distance between the current vector and a given instance
 * \param other the given instance
 **/
double pairshape_vector::Euc_dist(pairshape_vector &other)
{
   pairshape_vector tmp = *this - other;
   return tmp.norm();
}

/**
 * Hur_dist examines whether the given instance and the current instance satisfy
 * some huristic rules and return the distance between them
 * \param other the given instance
 **/
double pairshape_vector::Hur_dist(pairshape_vector &other)
{
   if(abs(FontHeight1- other.FontHeight1) >=5 || abs(FontHeight2 - other.FontHeight2)>=5)
      return 1e10;

   return Euc_dist(other);
}

/**
 * operator - overides the subtraction operator
 **/
pairshape_vector pairshape_vector::operator - (pairshape_vector a)
{
   pairshape_vector b = *this;
   //ASSERT(Type == a.Type || Type * a.Type == 0);
   b.Len1 -= a.Len1;
   b.Len2 -= a.Len2;
   b.Len3 -= a.Len3;

   b.alpha -= a.alpha;
   b.belta -= a.belta;

   b.FontHeight1 -= a.FontHeight1;
   b.FontHeight2 -= a.FontHeight2;
		
   return b;
}

/**
 * operator + overides the addition operator
 **/
pairshape_vector pairshape_vector::operator + (pairshape_vector a)
{
   pairshape_vector b = *this;
   //ASSERT(Type == a.Type || Type * a.Type == 0);
   b.Len1 += a.Len1;
   b.Len2 += a.Len2;
   b.Len3 += a.Len3;

   b.alpha += a.alpha;
   b.belta += a.belta;

   b.FontHeight1 += a.FontHeight1;
   b.FontHeight2 += a.FontHeight2;
		
   return b;
}

/**
 * operator * overides the multiply operator
 **/
pairshape_vector pairshape_vector::operator * (double a)
{
   pairshape_vector b = *this;
		
   b.Len1 	*= a;
   b.Len2  *= a;
   b.Len3  *= a;
   b.alpha *= a;
   b.belta *= a;

   b.FontHeight1 *= a;
   b.FontHeight2 *= a;

   return b;
}

/**
 * operator / overides the division operator
 **/
pairshape_vector pairshape_vector::operator / (double a)
{
//		ASSERT(a!=0);

   pairshape_vector b = *this;
		
   b.Len1 	/= a;
   b.Len2  /= a;
   b.Len3  /= a;
   b.alpha /= a;
   b.belta /= a;

   b.FontHeight1 /= a;
   b.FontHeight2 /= a;

   return b;
}

/**
 * dotproduct returns an instance who is the dot product of the current instance
 * and a given instance
 \ param a the given instance
**/
pairshape_vector pairshape_vector::dotproduct(pairshape_vector a)
{	
   pairshape_vector b		= *this;
   b.alpha *= a.alpha;
   b.belta *= a.belta;

   b.Len1 *= a.Len1;
   b.Len2 *= a.Len2;
   b.Len3 *= a.Len3;

   return b;
}

/**
 * dotsqrt returns an instance whose members are the square roots of the corresponding 
 * member of current instance
 **/
pairshape_vector pairshape_vector::dotsqrt()
{
   pairshape_vector b		= *this;
   b.alpha			= sqrt(alpha);
   b.belta			= sqrt(belta);
   b.Len1			= sqrt(Len1);
   b.Len2			= sqrt(Len2);
   b.Len3			= sqrt(Len3);
   return b;
}


/**
 * objsqrt returns an instance whose members are the square roots of the corresponding 
 * member of current instance
 **/

pairshape_vector pairshape_vector::objsqrt()
{
   pairshape_vector b;
   b.alpha			= sqrt(alpha);
   b.Len1			= sqrt(Len1);
   b.Len2			= sqrt(Len2);
   b.Len3			= sqrt(Len3);
   b.belta			= sqrt(belta);

   return b;
}

/**
 * dotdivide returns an instance whose members are the division results of the
 * corresponding members of the current instance and the given instance 
 * \param a the given instance
 **/
pairshape_vector pairshape_vector::dotdivide( pairshape_vector a)
{
   //ASSERT(Type == a.Type);

   pairshape_vector b		= *this;
   b.alpha			/= a.alpha;
   b.belta			/= a.belta;

   b.Len1			/= a.Len1;
   b.Len2			/= a.Len2;
   b.Len3			/= a.Len3;
		
//		b.FontHeight2	/= a.FontHeight2;
//		b.FontHeight1	/= a.FontHeight1;
   return b;
}

/*********************************************************************
 ***** struct pairshape_vector_list implementations********************
 **********************************************************************/
// form textline-pair shape vector from extracted textline objects
#ifndef PSV_N_1
#define PSV_N_1  1
#endif

/**
 * Construction builds the pairshape vector list
 * \param obj the text line object set
 * \param object_count the number of objects in the set
 * \param documentID index of the doucment where the objects dome from
 * \param style indicates whether use virtue line to form quadrilaterals or not
 **/
void pairshape_vector_list :: Construction(struct objectInfo *obj, int object_count,int documentId, int style)
{
   Reset();	

   if (object_count < 1)	return ;

   m_nVectors = style == PSV_N_1 ? object_count : object_count*(object_count-1)/2;

   m_pVectorList = new pairshape_vector[m_nVectors];

   /* initialize list of documents containing pair */
   int i;
   for (i=0;i<m_nVectors;i++)	/* initialized document entries for those entries which will be used at this time */
      m_pVectorList[i].doc_ID = documentId; 

   if(style == PSV_N_1)
   {
      pointLocation virtueLeft(0,0);
      pointLocation virtueRight(1000,0);
      for (i=0;i<object_count;i++)
         m_pVectorList[i].Construction(obj[i],virtueLeft, virtueRight);			
			
   }
   else 
   {
      int index=0;
      int j;
      for (i=0;i<object_count-1;i++)
         for ( j=i+1;j<object_count;j++)
         {
            m_pVectorList[index].Construction(obj[i], obj[j]);			
            index++;
         }
   }
}

/**
 * Append adds another vector list to the current vector list
 * \param addon the list to be appended
 **/
void pairshape_vector_list::Append(pairshape_vector_list * addon)
{
   if(addon == NULL || addon->m_nVectors == 0) return;

   pairshape_vector * pTemp = new pairshape_vector[m_nVectors + addon->m_nVectors];

   memcpy(pTemp, m_pVectorList, sizeof(pairshape_vector)*m_nVectors);
   memcpy(&(pTemp[m_nVectors]), addon->m_pVectorList, sizeof(pairshape_vector)*addon->m_nVectors);

   delete []m_pVectorList;
   m_pVectorList  = pTemp;

   m_nVectors += addon->m_nVectors;

}

/**
 * Reset releases memories and resets all working variables to the default value
 **/
void pairshape_vector_list::Reset()
{
   if(m_pVectorList) delete[]m_pVectorList;
   m_pVectorList = NULL;
   m_nVectors = 0;

}



/*********************************************************************
 ***** struct pairshape_cluster implementations************************
 **********************************************************************/
/**
 * Reset resets member list
 **/
void pairshape_cluster::Reset()
{
   m_MemberList.Reset();
}

/**
 * operator = overides the assign operator
 **/
pairshape_cluster pairshape_cluster::operator = (pairshape_cluster a)
{
   m_avgNum_perdoc = a.m_avgNum_perdoc;
   m_centroid = a.m_centroid;
   m_nMember = a.m_nMember;
   m_sigma   = a.m_sigma;
   m_sigmaNum_perdoc = a.m_sigmaNum_perdoc;
   m_weight = a.m_weight;

   m_MemberList.Append(&a.m_MemberList);

   return *this;
}

/**
 * write_stream save struct information to disk
 * \param f output file stream object instance
 **/
void pairshape_cluster::write_stream( ofstream &f )
{
   f << "---------pairshape_cluster-----------" << endl;
	
   f	<< "numMember: " << m_nMember << endl;
   f   << "centroid:" << endl;
   m_centroid.write_stream(f);
   f   << "sigma:" << endl;
   m_sigma.write_stream(f);

   f	<< "weight: " << m_weight << endl;
   f	<< "avgNb_perdoc: " << m_avgNum_perdoc << endl;
   f	<< "sigmaNb_perdoc: " << m_sigmaNum_perdoc << endl;
	
   f << "members:" << endl;
   m_MemberList.write_stream(f);	
};


/**
 * read_stream load struct information from file
 * \param f input file stream object instance
 **/
void pairshape_cluster::read_stream( ifstream &f )
{
   char	buf[100];
   f >> buf;	
   f >> buf;
   f	>> m_nMember;
   f >> buf;
   m_centroid.read_stream(f);
   f >> buf;
   m_sigma.read_stream(f);

   f	>> buf; f >> m_weight;
   f	>> buf; f >> m_avgNum_perdoc ;
   f	>> buf; f >> m_sigmaNum_perdoc;
	
   f >> buf;
   m_MemberList.read_stream(f);	
};


/*********************************************************************
 ***** struct pairshape_cluster_list implementations*******************
 **********************************************************************/
/**
 * dyn_Clustering clusters pairshape vectors; cluster centers are dynamically updated to be the
 * members that have the minimum sum of distance to all cluster members.
 * \param pDocLists document list where pairshape_vectors come from
 * \param nDoc number of documents inside the list
 * \param flag reserved parameter
 **/
void pairshape_cluster_list::dyn_Clustering(pairshape_vector_list * pDocLists, int nDoc,bool flag)
{
   Reset();

   if(pDocLists == NULL) return ;
	
   int i = 0;
   int nTotalVector = 0;
   for(;i<nDoc; nTotalVector+= pDocLists[i].m_nVectors, i++);

   m_nTotalVectors = nTotalVector;

#define BASE_SIZE 10

   //get initial cluster centers
   int cluster_count = 1;
   b_Cluster * pClusters = new b_Cluster[max(100,nTotalVector)];
	
   pClusters[0].memberlist.Construction(BASE_SIZE,2);
   pClusters[0].Add(0,0,pDocLists[0].m_pVectorList[0]);

   int times;
   for(times=0; times< this_iterations; times++)
   {
      int i;
      for (i=0;i<nDoc;i++) 
      {
         int nVector = pDocLists[i].m_nVectors;
         int j;
         for(j=0;j<nVector;j++)
         {
            if(j == 0 && i==0 && times ==0) continue;

            pairshape_vector& currentV = pDocLists[i].m_pVectorList[j];
				
//				ASSERT(currentV.norm()>=0);

            double min_dist = range_vector;
            int nearest_cluster = -1;
            int former_nMember = 0;

            //looking for the nearest cluster
            int k;
            for(k=0;k<cluster_count; k++)
            {
               if(pClusters[k].nMember == -1) continue;

               double dist = currentV.match( pClusters[k].centroid);
					
               if ( dist < min_dist || dist == min_dist && pClusters[k].nMember > former_nMember)
               {
                  nearest_cluster = k;
                  min_dist = dist;
                  former_nMember = pClusters[k].nMember;
               }						
            }

            if(nearest_cluster >=0) //add polygon i to the nearest cluster
            {
               //add current objects to memberlist
               pClusters[nearest_cluster].Add(i,j, currentV);	
					
            }			
            else //forms a new cluster
            {
               pClusters[cluster_count].memberlist.Construction(BASE_SIZE,2);
               pClusters[cluster_count].Add(i,j, currentV);

               cluster_count++;
            }

         }//end for j
      }//end for i

      //check dead cluster
      int s;
      for(s=0; s<cluster_count; s++)
         if(pClusters[s].nMember ==0)
            pClusters[s].nMember = -1; //place invalid flag
			

      if(times == this_iterations - 1) break;

      //prepare for redistributing polygons
      for (i=0;i<cluster_count;i++)//clear member list of each cluster, just keep the centroid index
      {
         if(pClusters[i].nMember == -1) continue;
         pClusters[i].memberlist.Construction(BASE_SIZE,2);						
         pClusters[i].nMember = 0;
         pClusters[i].nTotalMember = 0;
      }

   }//end for times

   //count dead cluster
   m_nClusters = cluster_count;
   int s;
   for( s=0; s<cluster_count; s++)
      if(pClusters[s].nMember <=0 )
         m_nClusters--; 

   //dump cluster info to objcluster point to be returned
   //for each cluster, compute the average number of object per documents
   m_pClusterList = new pairshape_cluster[m_nClusters];

   int index = 0; 
   for(i=0;i<cluster_count;i++)
   {
      if(pClusters[i].nMember <=0 ) continue;

      m_pClusterList[index].m_nMember = pClusters[i].nMember;
      m_pClusterList[index].m_centroid = pClusters[i].centroid;
      m_pClusterList[index].m_MemberList.m_nVectors = pClusters[i].nMember;
      m_pClusterList[index].m_MemberList.m_pVectorList = new pairshape_vector[pClusters[i].nMember];

      pairshape_vector	sigma;
      int j;
      for(j=0;j<pClusters[i].nMember;j++)
      {
         int page_index = pClusters[i].memberlist[j][0];
         int member_index = pClusters[i].memberlist[j][1];
         m_pClusterList[index].m_MemberList.m_pVectorList[j] = 
            pDocLists[page_index].m_pVectorList[member_index];

         pairshape_vector diff = pDocLists[page_index].m_pVectorList[member_index] - m_pClusterList[index].m_centroid ;
         sigma = sigma + diff.dotproduct(diff);
      }
		
      if(nDoc > 1) 
      {
         sigma = sigma/(double)pClusters[i].nMember;
         m_pClusterList[index].m_sigma = sigma.dotsqrt();
         CheckSTD(m_pClusterList[index].m_sigma);
      }

      index ++;
   }	

   delete []pClusters;
}




/**
 * SetDefaultSigma sets default standard deviation
 * \param sigma the default value
 **/
void pairshape_cluster_list::SetDefaultSigma(pairshape_vector &sigma)
{
   default_sigma = sigma;
}


/**
 * WeightingAgainst weights current cluster by referring to other/unwanted clusters.
 * weight is computed as the positive occurence ratio among all wanted and unwanted members
 * \param other the cluster lists used as referrences
 **/
void pairshape_cluster_list::WeightingAgainst(pairshape_cluster_list *Other)
{

   if(Other == NULL) return;

   int other_cnt = Other->m_nClusters;

   for (int i=0;i<m_nClusters;i++)
   {
      if(m_pClusterList[i].m_nMember <1) //a virtue cluster 
      {
         m_pClusterList[i].m_weight = 0;
         continue;
      }
		
      pairshape_vector& center = m_pClusterList[i].m_centroid;

      int near_count = 0;		 
      for (int j=0;j<other_cnt;j++)
      {
         pairshape_vector& other_center = Other->m_pClusterList[j].m_centroid;
			
//			ASSERT(other_center.norm() >=0 && center.norm() >=0 );

         double dist = center.match(other_center);
         if (dist < range_weightcluster)	
            near_count += Other->m_pClusterList[j].m_nMember;
							
      }
      m_pClusterList[i].m_weight = ((float)m_pClusterList[i].m_nMember)/
         (m_pClusterList[i].m_nMember + near_count); //corrective_factor2);

   }

}


/**
 * ScoreAPage score a testing document page
 * \param pTest pointer to the page layout (a cluster set) that to be scored
 * \param method indicate the scoring method
 * \param upside flag, if true, the testing document will be put upside down to compute the score again
 * and the final score is the larger one.
 * \return the score
 **/
double pairshape_cluster_list::ScoreAPage(pairshape_cluster_list *pTest, int method, bool upside )
{
   double score = ScoreOnClusterList(pTest, method);
   double score2 = 0;
   if(upside)
   {
      pairshape_cluster_list  pTest2;
      pTest2.m_nClusters = pTest->m_nClusters;
      pTest2.m_pClusterList = new pairshape_cluster[pTest->m_nClusters];
      int i;
      for(i=0; i<pTest->m_nClusters; i++)
      {
         pTest2.m_pClusterList[i].m_centroid = pTest->m_pClusterList[i].m_centroid.Reverse();
         pTest2.m_pClusterList[i].m_nMember = pTest->m_pClusterList[i].m_nMember;
      }

      score2 = ScoreOnClusterList(&pTest2, method);
   }

   return max(score, score2);

}


/**
 * dyn_Clustering clusters pairshape vectors; cluster centers are dynamically updated to be the
 * members that have the minimum sum of distance to all cluster members.
 * \param pDocLists document list where pairshape_vectors come from
 * \param nDoc number of documents inside the list
 * \param stat_flag if true, calculate the statistics for each cluster with respect to an average document
 * that is, the mean and variance of the member occurence in an average document for a cluster
 * \param flag reserved parameter
 **/
void pairshape_cluster_list::dyn_Clustering(pairshape_cluster_list **pDocLists, int nDoc, bool stat_flag, bool flag)
{
   Reset();

   if(pDocLists == NULL) return ;
   int i=0; 
   int nTotalCluster=0;
   for(;i<nDoc; nTotalCluster+= pDocLists[i]->m_nClusters, i++);

#define BASE_SIZE 10

   //get initial cluster centers
   int cluster_count = 1;
   b_Cluster * pClusters = new b_Cluster[nTotalCluster + 100];
   pClusters[0].memberlist.Construction(BASE_SIZE,2);
   pClusters[0].Add(0,0,pDocLists[0]->m_pClusterList[0].m_centroid ,pDocLists[0]->m_pClusterList[0].m_nMember);

   int times;
   for(times=0; times< this_iterations; times++)
   {
      for (int i=0;i<nDoc;i++) 
      {
         int nb_clusters = pDocLists[i]->m_nClusters;
         int j;
         for(j=0;j<nb_clusters;j++)
         {
            if(j==0 && i==0 && times==0) continue; //pass the first one which has form a cluster at the very beginning

            //get current vector which stands for a cluster center, that should be distributed out
            pairshape_vector& currentV = pDocLists[i]->m_pClusterList[j].m_centroid;
            int cur_nNewMember = pDocLists[i]->m_pClusterList[j].m_nMember;

            double min_dist = range_cluster;
            int nearest_cluster = -1;
            int former_nMember = 0;

            //considering all existing clusters
            int k;
            for(k=0;k<cluster_count;k++)
            {
               if(pClusters[k].nMember == -1) continue;

               double dist = currentV.match( pClusters[k].centroid);						
               if ( dist < min_dist || dist == min_dist && pClusters[k].nTotalMember > former_nMember)
               {
                  nearest_cluster = k;
                  min_dist = dist;
                  former_nMember = pClusters[k].nTotalMember;
               }						
            }

            if(nearest_cluster >=0) //add polygon i to the nearest cluster
            {
               //add current objects to memberlist
               pClusters[nearest_cluster].Add(i,j, currentV, cur_nNewMember);	
            }			
            else //forms a new cluster
            {
               pClusters[cluster_count].memberlist.Construction(BASE_SIZE,2);
               pClusters[cluster_count].Add(i,j, currentV,cur_nNewMember);
               cluster_count++;
            }

         }//end for j
      }//end for i

      //check dead cluster
      int s;
      for(s=0; s<cluster_count; s++)
         if(pClusters[s].nMember ==0)
            pClusters[s].nMember = -1; //place invalid flag
			

      if(times == this_iterations - 1) break;

      //prepare for redistributing polygons
      int index_i = 0;
      for (;index_i<cluster_count;index_i++)//clear member list of each cluster, just keep the centroid index
      {
         if(pClusters[index_i].nMember == -1) continue;
         pClusters[index_i].memberlist.Construction(BASE_SIZE,2);
         pClusters[index_i].nMember = 0;
         pClusters[index_i].nTotalMember = 0;
      }

   }//end for times

   //count dead cluster
   m_nClusters = cluster_count;
   int s;
   for(s=0; s<cluster_count; s++)
      if(pClusters[s].nMember ==-1)
         m_nClusters--; 


   //dump cluster info to objcluster point to be returned
   //for each cluster, compute the average number of object per documents
   m_pClusterList = new pairshape_cluster[m_nClusters];

   int index=0;
   for(i=0;i<cluster_count;i++)
   {
      if(pClusters[i].nMember <=0 ) continue;

      m_pClusterList[index].m_centroid = pClusters[i].centroid;
      m_pClusterList[index].m_nMember  = pClusters[i].nTotalMember;
        
      if(!stat_flag) 
      {
         index++;
         continue;
      }
		
      // build the document-object histogram for current cluster
      CTArray<int> histo(nDoc);
      pairshape_vector sigma;
      int j;
      for( j=0;j<pClusters[i].nMember;j++)
      {
         int page_index   = pClusters[i].memberlist[j][0];
         int member_index = pClusters[i].memberlist[j][1];
         int cur_nMember  = pDocLists[page_index]->m_pClusterList[member_index].m_nMember;
			
         m_pClusterList[index].m_MemberList.m_pVectorList = new pairshape_vector[cur_nMember];
         int k;
         for( k=0; k<cur_nMember; k++)
         {
            m_pClusterList[index].m_MemberList.m_pVectorList[k] = 
               pDocLists[page_index]->m_pClusterList[member_index].m_MemberList.m_pVectorList[k];
				
//				ASSERT(m_pClusterList[index].m_MemberList.m_pVectorList[k].doc_ID >=0
//					&& m_pClusterList[index].m_MemberList.m_pVectorList[k].doc_ID <nDoc);
			
         }

         histo[page_index] += cur_nMember;

         pairshape_vector diff = pDocLists[page_index]->m_pClusterList[member_index].m_centroid 
            - m_pClusterList[index].m_centroid;									 
			
         sigma = sigma + diff.dotproduct(diff)* (double)cur_nMember ;		
      }
      sigma = sigma/(double)m_pClusterList[index].m_nMember;
      m_pClusterList[index].m_sigma = sigma.dotsqrt();
      CheckSTD(m_pClusterList[index].m_sigma);
		
/*		int non_zeroDoc = 0;
		for(j=0;j<nDoc; j++)	if(histo[j] > 0) non_zeroDoc++;
		if(!flag) non_zeroDoc = nDoc;

		//compute the mean and variance of the #obj per doc
		m_pClusterList[index].m_avgNum_perdoc = m_pClusterList[index].m_nMember/(double)non_zeroDoc;
		m_pClusterList[index].m_sigmaNum_perdoc = 0;
		for(int k=0;k<nDoc;k++) 
		{
                if(flag && histo[k]==0) continue;
                m_pClusterList[index].m_sigmaNum_perdoc +=  (histo[k] - m_pClusterList[index].m_avgNum_perdoc)
                *(histo[k] - m_pClusterList[index].m_avgNum_perdoc);
		}

		m_pClusterList[index].m_sigmaNum_perdoc /= (double)non_zeroDoc;	
		if(m_pClusterList[index].m_sigmaNum_perdoc <1e-4 ) 
                m_pClusterList[index].m_sigmaNum_perdoc = 1e-4;

		ASSERT(m_pClusterList[index].m_centroid.norm() >=0 );
*/
      index++;
   }

   delete []pClusters;

}

/**
 * ScoreOnClusterList scores a testing document page, it is called by ScoreAPage()
 * \param pTest points to the testing page (a cluster set)
 * \param method indicates the method used to score a page
 **/
double pairshape_cluster_list::ScoreOnClusterList(pairshape_cluster_list *pTest, int method)
{
   int nTest = pTest->m_nClusters;	
   CTArray<double> scores(nTest);
   double sum_score = 0;
   int instanceSum = 0;

   //for each pair cluster in the test document
   for (int i = 0; i < nTest; i++)
   {
      double best_fit_score = 0;		
		
      //for nearest neighbor
      double min_dist = 1e10; 

      //for Gaussian KNN
      double sum_weighted_score = 0;
      double sum_probability = 0;

      pairshape_vector & center_test = pTest->m_pClusterList[i].m_centroid;
      for (int j=0;j < m_nClusters; j++) // searching all training clusters
      {
         pairshape_vector& center = m_pClusterList[j].m_centroid;			
         double dist= center.match(center_test);

         if (dist > range_score) continue;
			
         if(method == 0 ) // for Best Neighbor
            best_fit_score = max(best_fit_score, m_pClusterList[j].m_weight);
			
         else if(method == 1) //for Nearest Neighbor
         {
            if ( dist < min_dist) 			
            {				
               best_fit_score = m_pClusterList[j].m_weight;
               min_dist = dist;
            }
         }
         else // for Gaussian KNN 
         {				
            double probability = exp(-dist*dist);
            sum_weighted_score += probability * m_pClusterList[j].m_weight;
            sum_probability += probability;				
         }
      }

      if(method == 0 || method == 1)//for Best Neighbor & NN
         scores[i] = best_fit_score;

      else //for Gaussian KNN
         scores[i] = sum_probability==0 ? 0 : sum_weighted_score/sum_probability;

      sum_score += scores[i] * pTest->m_pClusterList[i].m_nMember ;
      instanceSum += pTest->m_pClusterList[i].m_nMember ;

   }

   sum_score /= (double)max(1,instanceSum); 

   return sum_score;


}

/**
 * Probability computes the probability that a testing document page is from the same layout class
 * as the current one. 
 * \param pTest points to the testing document (a cluster set)
 * \return the probability
 **/
double pairshape_cluster_list::Probability(pairshape_cluster_list *pTest)
{
   int nTestObj = pTest->m_nClusters;

   CTArray<int>	 hit_histo(m_nClusters);
   CTArray<double>  hit_score(m_nClusters);  // in fact, hit distance 

   int unhitted_obj=0;
   int i;
   for( i=0;i<nTestObj;i++)
   {
      pairshape_vector &obj = pTest->m_pClusterList[i].m_centroid;

      double mindist = range_vector; 
      int index = -1;
      int j;
      for( j=0;j<m_nClusters;j++)
      {
         // look for the cluster that current object hit
         double dist = m_pClusterList[j].m_centroid.Hur_dist(obj);
         if(dist < mindist)
         {
            mindist  = dist;
            index = j;
         }
      }

      if(index >=0) // some cluster is hit
      {
         hit_histo[index] += pTest->m_pClusterList[i].m_nMember;			 // record how many objects hit the same cluster
				
         pairshape_vector diff = obj - m_pClusterList[index].m_centroid;

         diff = diff.dotdivide(m_pClusterList[index].m_sigma);

         double power = diff.normsquare()/5;
			
         hit_score[index] += pTest->m_pClusterList[i].m_nMember * exp(-power/2);     // record the average hitting score
      }
      else
         unhitted_obj += pTest->m_pClusterList[i].m_nMember;
   }

   //use x^2 statistics to measure histogram difference -- inter cluster difference
   double x_square = 0;
   double TotalWeight = 0;
   for(i=0;i<m_nClusters;i++)
   {
      x_square +=   (hit_histo[i] - m_pClusterList[i].m_avgNum_perdoc) 
         * (hit_histo[i] - m_pClusterList[i].m_avgNum_perdoc)
         / (2* m_pClusterList[i].m_sigmaNum_perdoc)
         * m_pClusterList[i].m_weight;

      TotalWeight += m_pClusterList[i].m_weight;
   }

   x_square +=  (unhitted_obj - nTestObj*0.1)
      *(unhitted_obj- nTestObj*0.1)
      /(2.0 * 30) ; 
	
   x_square /= TotalWeight;
	

   double intra_cluster_diff = 0; // the sum of the score got for each individual cluster
   int n_hitCluster = 0;
   double  sumWeight = 0;
   for(i=0;i<m_nClusters;i++)
   {
      if(hit_histo[i] <1) continue;
      intra_cluster_diff += m_pClusterList[i].m_weight * (hit_score[i]/hit_histo[i]);
      n_hitCluster++;
      sumWeight += m_pClusterList[i].m_weight;
   }

   if(n_hitCluster == 0) return 0;

   double final_prob = intra_cluster_diff/sumWeight * exp(-x_square)
      * ((nTestObj - unhitted_obj)/(double)nTestObj) ;
	
   return final_prob;
}

/**
 * CheckSTD examines whether each dimension of a given pairshape vector is within a given range,
 * if not, substitue the value with a default value.
 * \param targ the vector to be examined
 **/
void pairshape_cluster_list::CheckSTD(pairshape_vector &targ)
{
   if(targ.Len1 < 1e-6) targ.Len1 = default_sigma.Len1;
   if(targ.Len2 < 1e-6) targ.Len2 = default_sigma.Len2;
   if(targ.Len3 < 1e-6) targ.Len3 = default_sigma.Len3;

   if(targ.alpha < 1e-6) targ.alpha = default_sigma.alpha;
   if(targ.belta < 1e-6) targ.belta = default_sigma.belta;

}

/**
 * Probability computes the probability that a testing document page is from the same layout class
 * as the current one. 
 * \param pTest points to the testing document (a vector set)
 * \return the probability
 **/
double pairshape_cluster_list::Probability(pairshape_vector_list *pTest)
{
   int nTestObj = pTest->m_nVectors;

   CTArray<int>	 hit_histo(m_nClusters);
   CTArray<double>  hit_score(m_nClusters);  // infact, hit distance 

   int unhitted_obj=0;
   int i;
   for(i=0;i<nTestObj;i++)
   {
      pairshape_vector &obj = pTest->m_pVectorList[i];

      double mindist = range_vector; 
      int index = -1;
      int j;
      for( j=0;j<m_nClusters;j++)
      {
         // look for the cluster that current object hit
         double dist = m_pClusterList[j].m_centroid.Hur_dist(obj);
         if(dist < mindist)
         {
            mindist  = dist;
            index = j;
         }
      }

      if(index >=0) // some cluster is hit
      {
         hit_histo[index] ++;			 // record how many objects hit the same cluster
				
         pairshape_vector diff = obj - m_pClusterList[index].m_centroid;

         diff = diff.dotdivide(m_pClusterList[index].m_sigma);

         double power = diff.normsquare()/5;
			
         hit_score[index] += exp(-power/2);     // record the average hitting score
      }
      else
         unhitted_obj ++;
   }

   //use x^2 statistics to measure histogram difference -- inter cluster difference
   double x_square = 0;
   double totalweights =0;
   for(i=0;i<m_nClusters;i++)
   {
      x_square +=   (hit_histo[i] - m_pClusterList[i].m_avgNum_perdoc) 
         * (hit_histo[i] - m_pClusterList[i].m_avgNum_perdoc)
         / (2* m_pClusterList[i].m_sigmaNum_perdoc)
         * m_pClusterList[i].m_weight;
      totalweights += m_pClusterList[i].m_weight;
   }

//	if(m_noiseVRatio < 1e-2) m_noiseVRatio = 0.1;
	
   x_square +=  (unhitted_obj - nTestObj*0.1)
      *(unhitted_obj- nTestObj*0.1)
      /(2.0 * 30) ; 
	
   x_square /= totalweights;

   double intra_cluster_diff = 0; // the sum of the score got for each individual cluster
   int n_hitCluster = 0;
   double sumWeights = 0;
   for(i=0;i<m_nClusters;i++)
   {
      if(hit_histo[i] <1) continue;
      intra_cluster_diff += m_pClusterList[i].m_weight * (hit_score[i]/hit_histo[i]);
      n_hitCluster++;
      sumWeights += m_pClusterList[i].m_weight;
   }

   if(n_hitCluster == 0) return 0;
		
   double final_prob = intra_cluster_diff/sumWeights 
      * exp(-x_square) 
      * ((nTestObj - unhitted_obj)/(double)nTestObj) ;
	
   return final_prob;

}

/**
 * ClusterStat computes the statistics of a cluster with respect to an average document.
 * \param nDoc number of documents that participate clustering
 * \param flag indicates whether the documents with no occurence of a cluster member will be
 * takin into account for computing average occurence per document.
 **/
void pairshape_cluster_list::ClusterStat(int nDoc, bool flag)
{
   m_nTotalVectors = 0;

   int i = 0; 
   int nIllegal = 0;
   for(; i<m_nClusters; i++)
   {
      CTArray<int> histo(nDoc);
      int nMember = m_pClusterList[i].m_nMember;
      pairshape_vector_list &pVectors = m_pClusterList[i].m_MemberList;
      int j;
      for(j=0;j<nMember;j++)
      {
         int docID = pVectors.m_pVectorList[j].doc_ID;
         histo[docID]++;
      }

      int non_zeroDoc = 0;
      for(j=0;j<nDoc; j++)	if(histo[j] > 0) non_zeroDoc++;
      if(!flag) non_zeroDoc = nDoc;

      m_pClusterList[i].m_avgNum_perdoc = nMember/(double)non_zeroDoc;
		
      //label the clusters whose frequency is too low
      m_nTotalVectors += nMember;
      if(m_pClusterList[i].m_avgNum_perdoc < 0.5)
      {
         m_pClusterList[i].m_nMember *= -1;  
         nIllegal ++;
         continue;
      }

      m_pClusterList[i].m_sigmaNum_perdoc = 0;
      int k;
      for( k=0;k<nDoc;k++) 
      {
         if(flag && histo[k]==0) continue;
         m_pClusterList[i].m_sigmaNum_perdoc +=  (histo[k] - m_pClusterList[i].m_avgNum_perdoc)
            *(histo[k] - m_pClusterList[i].m_avgNum_perdoc);
      }

      m_pClusterList[i].m_sigmaNum_perdoc /= (double)non_zeroDoc;	
      if(m_pClusterList[i].m_sigmaNum_perdoc <1e-4)
         m_pClusterList[i].m_sigmaNum_perdoc = 1e-4;

   }

   //remove illegal clusters
   pairshape_cluster * tmp = new pairshape_cluster[m_nClusters-nIllegal];
   int index = 0;
   for(i=0; i<m_nClusters; i++)
   {
      if(m_pClusterList[i].m_nMember <= 0) continue;

      tmp[index++] = m_pClusterList[i];

   }

   delete []m_pClusterList;
   m_nClusters -= nIllegal;
   m_pClusterList = tmp; 

//	m_noiseVRatio = (double)nIllegal/m_nTotalVectors;
   m_nAvgVperDoc = m_nTotalVectors/(double)nDoc;
}

/**
 * ProbScoreAPage computes the probability that a testing document is from the same layout class
 * \param pTest points to the testing document(a vector set)
 * \param upside a flag indicating whether consider the upsided document page
 * \param mirror a flag indicating whether consider the mirror image of the document
 **/
double pairshape_cluster_list::ProbScoreAPage(pairshape_vector_list *pTest, bool upside, bool mirror)
{
   double score1 = Probability(pTest);
   double score2 = 0;
   double score3 = 0;
   double score4 = 0;
   if(mirror)
   {
      int i;
      for( i=0; i<pTest->m_nVectors; i++)
         pTest->m_pVectorList[i] = pTest->m_pVectorList[i].Mirror();
		
      score3 = Probability(pTest);

      score1 = max(score1, score3);
   }
   if(upside)
   {
      pairshape_vector_list  pTest2;
      pTest2.Append(pTest);
      int i;
      for( i=0; i<pTest->m_nVectors; i++)
         pTest2.m_pVectorList[i] = pTest->m_pVectorList[i].Reverse();
			
      score2 = Probability(&pTest2);

      if(mirror)
      {
         int i;
         for( i=0; i<pTest->m_nVectors; i++)
            pTest2.m_pVectorList[i] = pTest2.m_pVectorList[i].Mirror();

         score4 = Probability(&pTest2);
      }

      score2 = max(score2, score4);

   }

   return max(score1, score2);
}

/**
 * SaveToDisk save information to disk
 * \param outfile the output file name
 * \param style indicating the formation style of the pairshape vector
 * \param saveweight flag indicating whether or not save weight, obsolete
 **/
void	pairshape_cluster_list::SaveToDisk(string outfile, int style, bool saveweight)
{
   ofstream	f;
   f.open( outfile.c_str(), ios::out );
   if( f.rdstate() & ios::failbit ) 
   {
      printf("open file error!");
      f.close();
      return;
   }

   f   << "pair_formation_style: " << style << endl;

   f	<< "numClusters: " << m_nClusters << endl;
   for( int i=0; i<m_nClusters; i++ )
   {
      f << "Cluster_" << i << endl;
      m_pClusterList[i].write_stream( f );
   }

   f	<< "avgNbVectorsperDoc: " << m_nAvgVperDoc << endl;
   //f	<< "noiseVRatio: "m_noiseVRatio << endl;	
   f	<< "numTotalVectors: " << m_nTotalVectors << endl;
	
   f.close();
}

/**
 * LoadFromDisk load an instance information from a file.
 * \param infile input file name
 * \param style used to store the pairshape vector formation style read from input file
 **/
void	pairshape_cluster_list::LoadFromDisk( string infile , int &style)
{
   if( m_pClusterList ) delete []m_pClusterList;
   m_pClusterList = NULL;

   ifstream	f;
   f.open( infile.c_str(), ios::in );
   if( f.rdstate() & ios::failbit ) 
   {
      printf("open file error!");
      f.close();
      return;
   }

   char buf[80];

   f	>> buf; f>> style;
   f	>> buf;	f >> m_nClusters;
   m_pClusterList	= new pairshape_cluster[m_nClusters];
   for( int i=0; i<m_nClusters; i++ )
   {	
      f>>buf;
      m_pClusterList[i].read_stream( f );
   }

   f	>> buf; f >> m_nAvgVperDoc ;
   // f	>> buf; f >> m_noiseVRatio ;	
   f	>> buf; f >> m_nTotalVectors ;
	
   f.close();
}



//// Construct textline shape vector, each textline is represented by a 5D vector, which in fact
//// defines a unique quadrilateral formed by this textline and the reference line.
//void pairshape_vector_list::Construction2(objectInfo *obj, int object_count, int documentId)
//{
//	Reset();	
//
//	/* number of object pairs = n*(n-1)/2  */
//	if (object_count < 1)	return ;
//
//	Clayout tmpLayout;
//	tmpLayout.ImportObjects(obj,object_count);
//	struct rectangle r;
//	tmpLayout.BoundingBox(obj,object_count, r);
//
//	m_nVectors = object_count;
//
//	m_pVectorList = new pairshape_vector[m_nVectors];
//
//	/* initialize list of documents containing pair */
//	for (int i=0;i<m_nVectors;i++)	/* initialized document entries for those entries which will be used at this time */
//		m_pVectorList[i].doc_ID = documentId; 
//
//	pointLocation ref1(r.left , r.top);
//	pointLocation ref2(r.right, r.top);
//	for (i=0;i<object_count;i++)
//		m_pVectorList[i].Construction(obj[i], ref1, ref2);			
//
//}
//

