/*
  Kmeans : class for kmeans clustering
  *
  */

#if !defined(AFX_KMEANS_H__A2716842_3DD9_4D85_882B_54EB8C40624A__INCLUDED_)
#define AFX_KMEANS_H__A2716842_3DD9_4D85_882B_54EB8C40624A__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "../Utility_hm/TArray.h"
#include "../Utility_hm/TMatrix.h"

#define MAXVECTDIM 12
#define MAXCLUSTER 500

#ifndef BOOL
#define BOOL bool
#endif

#ifndef FALSE 
#define FALSE false
#endif

#ifndef TRUE
#define TRUE true
#endif

/*
  struct aCluster is used only in Kmeans implementations. 
  One instance of aCluster records one cluster information.
*/
struct aCluster {
   double       Center[MAXVECTDIM];
   double       SDev[MAXVECTDIM];	    //Sample Deviation of this cluster
   CTArray<int> Member;					//Index of Vectors belonging to this cluster
   int          NumMembers;
   int			SizeVector;

   public:

   void Add(int);
   void Reset();
   BOOL Initiate(int);
   void ComputeSampleDev(CTMatrix<double> & pattern, CTArray<double> &def_std);
   const aCluster & operator = (aCluster b);
   aCluster(){NumMembers = 0; SizeVector=0; };
};


class CKmeans  
{
   public:
   CKmeans(int nPat,int patDim,int size_vector,bool dyn,int num);  //constructor
   virtual ~CKmeans();

//protected:
   CTMatrix<double>    Pattern;            //raw data
   aCluster     Cluster[MAXCLUSTER];       //cluster info

   CTArray<double> def_stdDev;
   int          NumPatterns;               // Number of patterns
   int          SizeVector;                // Number of dimensions in vector
   int			PatVectorDim;			   // vector dimension of each pattern ,most time the same as SizeVector	
   int          NumClusters;               // Number of clusters

   void			CalcSampleDev(int ClustID);
   void         DistributeSamples();				// Step 2 of K-means algorithm
   BOOL         CalcNewClustCenters();				// Step 3 of K-means algorithm
   inline   double       EucNorm(int, int);			// Calc Euclidean norm vector
   int          FindClosestCluster(int,double&);	// ret indx of clust closest to pattern whose index is arg
   public:
   void SetDefaultStdDev(CTArray<double> &def);
   void SetConstraint(int Dim, double c);
   bool ConditionSatisfied(double diff, int iDem);
   CTArray<double> maxRange;						//max range of each dimension
   CTArray<double> constraint;						//spetial constraint on each dimension
   BOOL bConverged;                                //convergy status flag
   BOOL bUseInitialValue;                          //if false, using random values as initial centroid of each cluster
   BOOL DynNumCluster;								//if true, dynamically decide the number of clusters
   int  runtime;
//	double threshold_ratio;                         
   double Je;										// value of Judge function
//	int nBestCluster;								

   void SetBound4EachDim(double *boundarray);      // set the value range of each dimension
   void SetInitialValue(const CTMatrix<double> &); // set the initial value of each cluster
	
   void Initiate();
   void CalcJe();									// evaluate Judge function
   void SetDynamic(BOOL,int);                      // set flag for dynamic determination of number of clusters, when
   // flag is false, the second parameter set the predefined number of clusters

   CKmeans();
   void Reset();
   void InitClusterCenter(bool useinitalcenter);   // Step 1 of K-means algorithm
   void RunKMeans(int maxiteration = 0, bool flag = false);            // Overall control K-means process

   private:
   CTMatrix<double> InitialValue;                   // initial value of each cluster
};

#endif // !defined(AFX_KMEANS_H__A2716842_3DD9_4D85_882B_54EB8C40624A__INCLUDED_)
