// TArray.cpp: implementation of the CTArray class.
//
//////////////////////////////////////////////////////////////////////
#include "TArray.h"

#include <memory.h>
#include <stdio.h>

#ifndef NULL
#define NULL	0
#endif

template class CTArray< short >;
template class CTArray< int >;
template class CTArray< long >;
template class CTArray< float >;
template class CTArray< double >;
//template class CTArray< CPoint >;


/**
 * Constructor without parameters
 **/
template <class TYPE>
inline CTArray< TYPE >::CTArray()
{
	
   m_memory      = NULL;  
   m_isAllocated = false; 
   m_dimension   = 0 ;    
   m_activeIndex = 0;     
}

/**
 * Constructor
 * \param dimension number of cells to be allocated
 **/
template <class TYPE>
inline CTArray< TYPE >::CTArray( long dimension )
{
	
   m_memory      = NULL;  
   m_isAllocated = false; 
   m_dimension   = 0 ;    
   m_activeIndex = 0;  

	
   if( dimension > 0 )
   {
      m_memory = new TYPE[ dimension ]; 

      if( m_memory != NULL ) 
      {
         m_isAllocated = true;      
         m_dimension   = dimension; 
         m_activeIndex = 0;         
         memset(m_memory,0,dimension*sizeof(TYPE)); 

      }
      else
      {
         Verification( "System Error << Can not allocate memory correctly >>" ); 
      }
   }
   else
   {
      Verification( "Program Error << Array dimension must be bigger than 0 >>" ); 
   }
}

/**
 * Constructor called by CTArray(long dimension)
 * \param memory points to the memory to be allocated
 * \param dimension number of array cells to be allocated
 * \param activeIndex index pointed to the current active cell
 **/
template <class TYPE>
inline CTArray< TYPE >::CTArray( TYPE* memory, long dimension, long activeIndex )
{	
   m_memory      = NULL;  
   m_isAllocated = false; 
   m_dimension   = 0 ;    
   m_activeIndex = 0;     

	
   if( memory != NULL && dimension > 0 ) 
   {
      m_isAllocated = false;       
      m_memory      = memory;      
      m_dimension   = dimension;   
      m_activeIndex = activeIndex; 
   }
   else
   {
      Verification( "Program Error << The assigned array memory pointer can not be null, and array dimension must be bigger thant 0 >>" ); // [ Ա ]
   }
}

/**
 * Copy Constructor
 * \param array the array to be copied
 **/
template <class TYPE>
inline CTArray< TYPE >::CTArray( const CTArray< TYPE >& array )
{	
   m_memory      = NULL;  
   m_isAllocated = false;
   m_dimension   = 0 ;    
   m_activeIndex = 0;     

   if( array.m_memory != NULL ) 
   {
      m_memory = new TYPE[ array.m_dimension ]; 

      if( m_memory != NULL ) 
      {
         m_isAllocated = true;                
         m_dimension   = array.m_dimension;   
         m_activeIndex = array.m_activeIndex; 

         long sizeInBytes = m_dimension * sizeof( TYPE ); 
         memcpy( m_memory, array.m_memory, sizeInBytes ); 	    
      }
      else
      {
         Verification( "System Error << Can not allocate memory correctly >>" ); 
      }	
   }
}

/**
 * Destructor
 **/
template <class TYPE>
inline CTArray< TYPE >::~CTArray(){
	
   if( m_isAllocated == true )
   {
      delete [] m_memory;
   }
}

/**
 * Rutine used to construct a CTArray instance
 * \param dimension the number of array cells
 **/
template <class TYPE>
inline void CTArray< TYPE >::Construction( long dimension )
{	  
	
   if( m_isAllocated == true )
   {
      delete [] m_memory;
   }	
	
   m_memory      = NULL; 
   m_isAllocated = false; 
   m_dimension   = 0 ;   
   m_activeIndex = 0;    

	
   if( dimension > 0 ) 
   {
      m_memory = new TYPE[ dimension ]; 

      if( m_memory != NULL ) 
      {
         m_isAllocated = true;      
         m_dimension   = dimension; 
         m_activeIndex = 0;         
         memset(m_memory,0,dimension*sizeof(TYPE)); 
      }
      else
      {
         Verification( "System Error << Can not allocate memory correctly >>" ); 
      }
   }
   else
   {
      Verification( "Program Error << Array dimension must be bigger than 0 >>" ); 
   }
}

/**
 * rutine used to construct a CTArray instance
 * \param memory points to the starting address of the memory to be allocated
 * \param dimension the number of array cells
 * \param activeIndex index of the current active cell
 **/
template <class TYPE>
inline void CTArray< TYPE >::Construction( TYPE* memory, long dimension, long activeIndex )
{
   if( m_isAllocated == true )
   {
      delete [] m_memory;
   }
	
   m_memory      = NULL;  
   m_isAllocated = false; 
   m_dimension   = 0 ;   
   m_activeIndex = 0;     

   if( memory != NULL && dimension > 0 ) 
   {
      m_isAllocated = false;       
      m_memory      = memory;      
      m_dimension   = dimension;   
      m_activeIndex = activeIndex; 
   }
   else
   {
      Verification( "Program Error << The assigned array memory pointer can not be null, and array dimension must be bigger than 0 >>" ); // [ Ա ]
   }
}

/**
 * routine used to construct a CTArray instance by copying another instance
 * \param array the instance to be copied
 **/
template <class TYPE>
inline void CTArray< TYPE >::Construction( const CTArray< TYPE >& array )
{
   if( array.m_memory != NULL ) 
   {
		
      if( m_isAllocated == true && m_dimension == array.m_dimension && m_memory != NULL ) 
      {
         m_activeIndex = array.m_activeIndex; 

         long sizeInBytes = m_dimension * sizeof( TYPE ); 
         memcpy( m_memory, array.m_memory, sizeInBytes ); 

         return; 
      }
	    
      if( m_isAllocated == true )
      {
         delete [] m_memory;
      }
	
      m_memory      = NULL;  
      m_isAllocated = false;
      m_dimension   = 0 ;    
      m_activeIndex = 0;     

	    
      m_memory = new TYPE[ array.m_dimension ]; 

      if( m_memory != NULL ) 
      {
         m_isAllocated = true;                
         m_dimension   = array.m_dimension;   
         m_activeIndex = array.m_activeIndex; 

         long sizeInBytes = m_dimension * sizeof( TYPE ); 
         memcpy( m_memory, array.m_memory, sizeInBytes ); 	    
      }
      else
      {
         Verification( "System Error << Can not allocate memory correctly >>" ); 
      }	
   }
   else
   {	    
      if( m_isAllocated == true )
      {
         delete [] m_memory;
      }	
	    
      m_memory      = NULL; 
      m_isAllocated = false; 
      m_dimension   = 0 ;    
      m_activeIndex = 0;     
   }
}

/**
 * routine used to destruct an instance
 **/
template <class TYPE>
inline void CTArray< TYPE >::Destruction()
{	
   if( m_isAllocated == true )
   {
      delete [] m_memory;
   }	
	
   m_memory      = NULL;  
   m_isAllocated = false; 
   m_dimension   = 0 ;    
   m_activeIndex = 0;     
}

/**
 * filling an instance's content by memory copying
 * \param source points to the memory block to be copied
 **/
template <class TYPE>
inline void CTArray< TYPE >::ImportFrom( const void* source )
{
   if( m_memory != NULL && source != NULL ) 
   {
      long sizeInBytes = m_dimension * sizeof( TYPE ); 
      memcpy( m_memory, source, sizeInBytes );         
   }
}

/**
 * ExportTo exports the memory content to a memory block
 * \param destination the targeted memory block
 **/
template <class TYPE>
inline void CTArray< TYPE >::ExportTo( void* destination ) const
{
   if( m_memory != NULL && destination != NULL ) 
   {
      long sizeInBytes = m_dimension * sizeof( TYPE ); 
      memcpy( destination, m_memory, sizeInBytes );    
   }
}

/**
 * operator [] accesses a cell by index 
 **/
template <class TYPE>
inline TYPE& CTArray< TYPE >::operator[]( long index ) const
{
#ifdef _DEBUG

   static TYPE error; 

   if( !( index >= ( - m_activeIndex ) && index <= ( m_dimension - m_activeIndex - 1 ) ) ) 
   {
      Verification( "Program Error << Array subscript beyond bound >>" ); 

      return error; 
   }

#endif // _DEBUG

   return m_memory[ index + m_activeIndex ]; 
}

/**
 * operator * returns the content of a cell
 **/
template < class TYPE >
inline TYPE& CTArray< TYPE >::operator*() const
{
#ifdef _DEBUG

   static TYPE error; 

   if( !( m_activeIndex >= 0 && m_activeIndex <= ( m_dimension  - 1 ) ) ) 
   {
      Verification( "Program Error << Array subscript beyond bound >>" ); 

      return error; 
   }

#endif // _DEBUG

   return m_memory[ m_activeIndex ]; 
}

/**
 * operator = overides the assign operator
 **/
template <class TYPE>
inline CTArray< TYPE >& CTArray< TYPE >::operator=( const CTArray< TYPE >& array ) 
{
   if( array.m_memory != NULL )
   {		
      if( m_isAllocated == true && m_dimension == array.m_dimension && m_memory != NULL ) 
      {
         m_activeIndex = array.m_activeIndex; 

         long sizeInBytes = m_dimension * sizeof( TYPE ); 
         memcpy( m_memory, array.m_memory, sizeInBytes ); 

         return * this; 
      }

      if( m_isAllocated == true )
      {
         delete [] m_memory;
      }
	
      m_memory      = NULL;  
      m_isAllocated = false; 
      m_dimension   = 0 ;    
      m_activeIndex = 0;     
	    
      m_memory = new TYPE[ array.m_dimension ]; 

      if( m_memory != NULL ) 
      {
         m_isAllocated = true;                
         m_dimension   = array.m_dimension;  
         m_activeIndex = array.m_activeIndex;

         long sizeInBytes = m_dimension * sizeof( TYPE ); 
         memcpy( m_memory, array.m_memory, sizeInBytes ); 		    
      }
      else
      {
         Verification( "System Error << Can not allocate memory correctly >>" ); 
      }		
   }
   else
   {
      if( m_isAllocated == true )
      {
         delete [] m_memory;
      }
	
	    
      m_memory      = NULL;  
      m_isAllocated = false; 
      m_dimension   = 0 ;    
      m_activeIndex = 0;    
   }

   return * this; 
}

/**
 * operator &= overides &=
 **/
template < class TYPE >
inline CTArray< TYPE >& CTArray< TYPE >::operator&= ( const CTArray< TYPE> & array )
{
   if( m_isAllocated == true )
   {
      delete [] m_memory;
   }
	
   m_isAllocated = false;               
   m_memory      = array.m_memory;      
   m_dimension   = array.m_dimension;   
   m_activeIndex = array.m_activeIndex; 
	
   return * this; 
}

//template < class TYPE >
//inline bool CTArray< TYPE >::operator==( const CTArray< TYPE >& array ) const
//{
//#ifdef _DEBUG
//
//	if(    m_memory      != array.m_memory        
//	    || m_dimension   != array.m_dimension )   
//	{
//		Verification( "Program Error << Unmatched comparison operand >>" ); 
//	}
//
//#endif // _DEBUG
//	
//	return ( m_activeIndex == array.m_activeIndex ); 
//}
//
//// [ ************ ] ......................................................
//// [ ʽ ] ......................................................
//// [ ************ ] ......................................................
//template < class TYPE >
//inline bool CTArray< TYPE >::operator!=( const CTArray< TYPE >& array ) const
//{
//#ifdef _DEBUG
//	
//	if(    m_memory      != array.m_memory        
//	    || m_dimension   != array.m_dimension )   
//	{
//		Verification( "Program Error << Unmatched comparison operand >>" ); 
//	}
//
//#endif // _DEBUG
//
//	
//	return ( m_activeIndex != array.m_activeIndex ); 
//}
//
//
//template < class TYPE >
//inline bool CTArray< TYPE >::operator>=( const CTArray< TYPE >& array ) const
//{
//#ifdef _DEBUG
//
//	
//	if(    m_memory      != array.m_memory       
//	    || m_dimension   != array.m_dimension )  
//	{
//		Verification( "Program Error << Unmatched comparison operand >>" );
//	}
//
//#endif // _DEBUG
//	
//	return ( m_activeIndex >= array.m_activeIndex );
//}
//
//template < class TYPE >
//inline bool CTArray< TYPE >::operator> ( const CTArray< TYPE >& array ) const
//{
//#ifdef _DEBUG
//
//	if(    m_memory      != array.m_memory        
//	    || m_dimension   != array.m_dimension )   
//	{
//		Verification( "Program Error << Unmatched comparison operand >>" ); 
//	}
//
//#endif // _DEBUG
//	
//	return ( m_activeIndex > array.m_activeIndex );
//}
//
//template < class TYPE >
//inline bool CTArray< TYPE >::operator<=( const CTArray< TYPE >& array ) const
//{
//#ifdef _DEBUG
//
//	if(    m_memory      != array.m_memory        
//	    || m_dimension   != array.m_dimension )   
//	{
//		Verification( "Program Error << Unmatched comparison operand >>" ); 
//	}
//
//#endif // _DEBUG
//
//	return ( m_activeIndex <= array.m_activeIndex );
//}
//
//template < class TYPE >
//inline bool CTArray< TYPE >::operator< ( const CTArray< TYPE >& array ) const
//{
//#ifdef _DEBUG
//	
//	if(    m_memory      != array.m_memory        
//	    || m_dimension   != array.m_dimension )   
//	{
//		Verification( "Program Error << Unmatched comparison operand >>" ); 
//	}
//
//#endif // _DEBUG
//
//	return ( m_activeIndex < array.m_activeIndex );
//}
//
//template < class TYPE >
//inline long CTArray< TYPE >::operator-( const CTArray< TYPE >& array ) const
//{
//#ifdef _DEBUG
//
//	if(    m_memory      != array.m_memory        
//	    || m_dimension   != array.m_dimension )  
//	{
//		Verification( "Program Error << Unmatched subtraction operand >>" ); 
//	}
//
//#endif // _DEBUG
//	
//	return ( m_activeIndex - array.m_activeIndex ); 
//}
//
//template < class TYPE >
//inline CTArray< TYPE >& CTArray< TYPE >::operator+( long step ) const
//{
//	static CTArray< TYPE > addition; 
//
//	addition.m_isAllocated = false;         
//	addition.m_memory      = m_memory;      
//	addition.m_dimension   = m_dimension;   
//	addition.m_activeIndex = m_activeIndex; 
//
//	addition.m_activeIndex += step ; 
//
//	return addition; 
//}
//
//template < class TYPE >
//inline CTArray< TYPE >& CTArray< TYPE >::operator+=( long step )
//{
//	m_activeIndex += step ; 
//
//	return * this; 
//}
//
//template < class TYPE >
//inline CTArray< TYPE >& CTArray< TYPE >::operator-( long step ) const
//{
//	static CTArray< TYPE > subtraction;
//
//	subtraction.m_isAllocated = false;         
//	subtraction.m_memory      = m_memory;      
//	subtraction.m_dimension   = m_dimension;   
//	subtraction.m_activeIndex = m_activeIndex; 
//
//	subtraction.m_activeIndex -= step ; 
//
//	return subtraction; 
//}
//
//template < class TYPE >
//inline CTArray< TYPE >& CTArray< TYPE >::operator-=( long step )
//{
//	m_activeIndex -= step ; 
//
//	return * this; 
//}
//
//template < class TYPE >
//inline CTArray< TYPE >& CTArray< TYPE >::operator++()
//{
//	m_activeIndex ++ ; 
//
//	return * this; 
//}
//
//template < class TYPE >
//inline CTArray< TYPE >& CTArray< TYPE >::operator++( int )
//{
//	static CTArray< TYPE > increment; // [ ֵ ]
//
//	increment.m_isAllocated = false;         // [ ûзڴ ]
//	increment.m_memory      = m_memory;      // [ ڴָ ]
//	increment.m_dimension   = m_dimension;   // [ ά ]
//	increment.m_activeIndex = m_activeIndex; // [ õǰԪصֵ ]
//
//	m_activeIndex ++ ; // [ ӵǰԪصֵ ]
//
//	return increment; // [ صֵ ]
//}
//
//// [ ************** ] ....................................................
//// [ ǰ׺ݼ ] ....................................................
//// [ ************** ] ....................................................
//template < class TYPE >
//inline CTArray< TYPE >& CTArray< TYPE >::operator--()
//{
//	m_activeIndex -- ; // [ СǰԪصֵ ]
//
//	return * this; // [ ؼŲֵ ]
//}
//
//// [ ************** ] ....................................................
//// [ ׺ݼ ] ....................................................
//// [ ************** ] ....................................................
//template < class TYPE >
//inline CTArray< TYPE >& CTArray< TYPE >::operator--( int )
//{
//	static CTArray< TYPE > decrement; // [ ݼֵ ]
//
//	decrement.m_isAllocated = false;         // [ ûзڴ ]
//	decrement.m_memory      = m_memory;      // [ ڴָ ]
//	decrement.m_dimension   = m_dimension;   // [ ά ]
//	decrement.m_activeIndex = m_activeIndex; // [ õǰԪصֵ ]
//
//	m_activeIndex -- ; // [ СǰԪصֵ ]
//
//	return decrement; // [ صݼֵ ]
//}


/**
 * IsNull returns true if the array is not allocated with any memory
 **/
template <class TYPE>
inline bool CTArray< TYPE >::IsNull() const
{
   return ( m_memory == NULL ); 
}

/**
 * IsNotNull returns true if the array memory is not null
 **/
template <class TYPE>
inline bool CTArray< TYPE >::IsNotNull() const
{
   return ( m_memory != NULL ); 
}

/**
 * GetDimension returns the number of cells
 **/
template <class TYPE>
inline long CTArray< TYPE >::GetDimension() const
{
   return m_dimension; 
}

/**
 * GetLowerBound returns the lower bound of the array
 **/
template <class TYPE>
inline long CTArray< TYPE >::GetLowerBound() const
{
   return ( - m_activeIndex ); 
}

/**
 * GetUpperBound returns the upper bound of the array
 **/
template <class TYPE>
inline long CTArray< TYPE >::GetUpperBound() const
{
   return ( m_dimension - m_activeIndex - 1 ); 
}

/**
 * Is_index_valid checks the validity of an array index
 **/
template <class TYPE>
inline bool CTArray< TYPE >::Is_index_valid( long index ) const
{
   return index >= - m_activeIndex && index < m_dimension - m_activeIndex ; 
}

/**
 * GetSizeInBytes returns the size of the array in bytes
 **/
template <class TYPE>
inline long CTArray< TYPE >::GetSizeInBytes() const
{
   return ( m_dimension * sizeof( TYPE ) ); 
}

/**
 * GetMemoryPointer returns the pointer of the starting memory address of the array
 **/
template <class TYPE>
inline TYPE * CTArray< TYPE >::GetMemoryPointer() const 
{
   return m_memory;
}

/**
 * Verification prints the error message
 **/
template <class TYPE>
inline void CTArray< TYPE >::Verification( char* message ) const
{
#ifdef _DEBUG

   printf( message ); 

#endif // _DEBUG
}

