// TMatrix.cpp: implementation of the CTMatrix class.
//
//////////////////////////////////////////////////////////////////////

#include "TMatrix.h"
#include <memory.h>
#include <stdio.h>

#ifndef NULL
#define NULL	0
#endif


template class CTMatrix< short >;
template class CTMatrix< int >;
template class CTMatrix< long >;
template class CTMatrix< float >;
template class CTMatrix< double >;
//template class CTMatrix< CPoint >;

/**
 * Constructor without input parameters
 **/
template <class TYPE>
inline CTMatrix< TYPE >::CTMatrix()
{
   Initialization(); 
}

/**
 * Constructor
 * \param rows the number of rows of the matrix
 * \param columns the number of columns of the matrix
 **/
template <class TYPE>
inline CTMatrix< TYPE >::CTMatrix( long rows, long columns )
{
   Initialization(); 

   Construction( rows, columns ); 
}

/**
 * Copy constructor
 * \param matrix the one to be copied
 **/
template <class TYPE>
inline CTMatrix< TYPE >::CTMatrix( const CTMatrix< TYPE >& matrix )
{
   Initialization(); 

   Construction( matrix ); 
}

/**
 * Destuctor
 **/
template <class TYPE>
inline CTMatrix< TYPE >::~CTMatrix()
{
   Destruction( ); 
}

/**
 * Routine used to construct a matrix instance
 * \param rows the number of rows
 * \param columns the number of columns
 **/
template <class TYPE>
inline void CTMatrix< TYPE >::Construction( long rows, long columns )
{

   Destruction( ); 

   if( rows > 0 && columns > 0 ) 
   {
      long elements = rows * columns ; 

      m_memory = new TYPE[ elements ]; 

      if( m_memory != NULL ) 
      {
         m_rows = rows; 
         m_columns = columns; 
         memset(m_memory,0,rows*columns*sizeof(TYPE)); 
      }
      else
      {
         Verification( "System Error << Can not allocate memory correctly >>" ); 
      }
   }
   else
   {
      Verification( "Program Error << Matrix rows and columns must be bigger than 0 >>" ); 
   }
}

/**
 * routins used to construct an instance by copying another instance
 **/
template <class TYPE>
inline void CTMatrix< TYPE >::Construction( const CTMatrix< TYPE >& matrix )
{
   if( matrix.m_memory != 0 ) 
   {
      Construction( matrix.m_rows, matrix.m_columns ); 

      long sizeInBytes = m_rows * m_columns * sizeof( TYPE ); 
      memcpy( m_memory, matrix.m_memory, sizeInBytes ); 
   }
   else
   {
      Destruction( ); 
   }
}

/**
 * Routine used to destruct an instance
 **/
template <class TYPE>
inline void CTMatrix< TYPE >::Destruction()
{
   if( m_rows>0 && m_memory != NULL ) 
   {
      delete [] m_memory; 

      Initialization( ); 
   }
}                                  

/**
 * filling an instance's content by memory copying
 * \param source points to the memory block to be copied
 **/
template <class TYPE>
inline void CTMatrix< TYPE >::ImportFrom( const void* source )
{
   if( m_memory != NULL ) 
   {
      long sizeInBytes = m_rows * m_columns * sizeof( TYPE ); 
      memcpy( m_memory, source, sizeInBytes ); 
   }
}

/**
 * ExportTo exports the memory content to a memory block
 * \param destination the targeted memory block
 **/

template <class TYPE>
inline void CTMatrix< TYPE >::ExportTo( void* destination ) const
{
   if( m_memory != NULL ) 
   {
      long sizeInBytes = m_rows * m_columns * sizeof( TYPE ); 
      memcpy( destination, m_memory, sizeInBytes );  
   }
}

/**
 * operator = overides the assign operator
 **/
template <class TYPE>
inline CTMatrix< TYPE >& CTMatrix< TYPE >::operator=( const CTMatrix< TYPE >& matrix ) 
{
   Construction( matrix ); 

   return * this; 
}


#ifdef _DEBUG

/**
 * operator [] accesses a row by index 
 **/
template <class TYPE>
inline CTArray< TYPE >& CTMatrix< TYPE >::operator[]( long index ) const
{
   static CTArray< TYPE > error; 
   if( !( index >= 0 && index < m_rows ) ) 
   {
      Verification( "Program Error << Matrix row index beyond bound >>" );

      return error; 
   }

   static CTArray< TYPE > rowArray; 
//	CTArray< TYPE > rowArray;
   rowArray.Construction( m_memory + index * m_columns, m_columns ); 
   return rowArray; 
} 

#else // _DEBUG

template <class TYPE>
inline TYPE* CTMatrix< TYPE >::operator[]( long index ) const
{
   return m_memory + index * m_columns; 
}

#endif // _DEBUG

//template <class TYPE>
//inline TYPE& CTMatrix< TYPE >::operator[]( const CImagePoint& point ) const
//{
//#ifdef _DEBUG
//
//	return ( *this )[ point.m_row ][ point.m_column ] ;
//
//#else // _DEBUG
//
//	return *( m_memory + point.m_row * m_columns + point.m_column );
//
//#endif // _DEBUG
//}

//template <class TYPE>
//inline bool CTMatrix< TYPE >::Is_point_valid( const CImagePoint& point ) const
//{
//	return point.m_row    >= 0 && point.m_row    < m_rows     
//		&& point.m_column >= 0 && point.m_column < m_columns; 
//}

/**
 * Is_row_valid returns true if the row index is within range
 **/
template <class TYPE>
inline bool CTMatrix< TYPE >::Is_row_valid( long row ) const
{
   return row >= 0 && row < m_rows; 
}

/**
 * Is_column_valid returns true if the column index is within range
 **/	
template <class TYPE>
inline bool CTMatrix< TYPE >::Is_column_valid( long column ) const
{
   return column >= 0 && column < m_columns; 
}

/**
 * IsNull returns true if the memory is null
 **/
template <class TYPE>
inline bool CTMatrix< TYPE >::IsNull() const
{
   return ( m_memory == NULL );
} 

/**
 * IsNotNull returns true if the memory is not null
 **/    
template <class TYPE>
inline bool CTMatrix< TYPE >::IsNotNull() const
{
   return ( m_memory != NULL );
}

/**
 * GetRows return the number of rows
 **/
template <class TYPE>
inline long CTMatrix< TYPE >::GetRows() const
{
   return m_rows;
}

/**
 * GetColumns return the number of columns
 **/
template <class TYPE>
inline long CTMatrix< TYPE >::GetColumns() const
{
   return m_columns;
}

/**
 * Get_height return the number of rows
 **/
template <class TYPE>
inline long CTMatrix< TYPE >::Get_height() const
{
   return m_rows;
}

/**
 * Get_width returns the number of columns
 **/
template <class TYPE>
inline long CTMatrix< TYPE >::Get_width() const
{
   return m_columns;
}

//// [ ************************** ] ........................................
//// [ ************************** ] ........................................
//template <class TYPE>
//inline CImagePoint CTMatrix< TYPE >::Get_size() const
//{
//	return CImagePoint( m_rows, m_columns );
//}

//template <class TYPE>
//inline long CTMatrix< TYPE >::GetRowUpperBound() const
//{
//	return ( m_rows - 1 );
//}
//
//// [ ********** ] ........................................................
//// [ ********** ] ........................................................
//template <class TYPE>
//inline long CTMatrix< TYPE >::GetColumnUpperBound() const
//{
//	return ( m_columns - 1 );
//}

/**
 * GetElements returns the number of cells of the matrix
 **/
template <class TYPE>
inline long CTMatrix< TYPE >::GetElements() const
{
   return ( m_rows * m_columns );
}

/**
 * GetSizeInBytes returns the memory size of the matrix in bytes
 **/
template <class TYPE>
inline long CTMatrix< TYPE >::GetSizeInBytes() const
{
   return ( m_rows * m_columns * sizeof( TYPE ) );
}

/**
 * GetMemoryPointer returns the memory pointer of the matrix
 **/
template <class TYPE>
inline TYPE * CTMatrix< TYPE >::GetMemoryPointer() const 
{
   return m_memory;
}


/**
 * Initialization releases memory and set members to zeros
 **/
template <class TYPE>
inline void CTMatrix< TYPE >::Initialization()
{
   m_memory = NULL; 
   m_rows = 0;      
   m_columns = 0;  

} 

/**
 * Verification prints error messages
 **/
template <class TYPE>
inline void CTMatrix< TYPE >::Verification( char* message ) const
{
#ifdef _DEBUG

   printf( message ); 

#endif // _DEBUG
}

