/***************************************************************************
 HEADER FILES
***************************************************************************/

#include <math.h>
#include <string.h>
#include <iostream>
#include "DLImage.h"
#include "DLLogger.h"
#include "DLComponent.h"
#include "getLines.h"


using namespace std;

/***************************************************************************
 MACROS
***************************************************************************/

/***************************************************************************
 STRUCTURES
***************************************************************************/

const static float       MAXDISTBETWEENCOMP  = 100.0;
const static short int   MINLINKS            = 15;
const static short int   MINCOMP             = 20;

const static short int   DL_NOT_ENOUGH_LINKS = -1;
const static short int   DL_NO_DECISION      = -2;

/***************************************************************************
 CLASS
***************************************************************************/

/**
 *  \class DLLineOrientationCalculation
 *
 *
 *  \brief This class is a port of the TwoBit method \c tbCalcLineOrientation.
 *   It uses  dress-right-dress algorithm to form the
 *   components into lines of text. in the horizontal and
 *   in the vertical direction then gets the mean of the
 *   distance between components in hor and vet direction
 *
 *	 NOTES:This version uses a triangle region to the right of CC.
 *                                                                  
 \code                                                                  
 -     
 -        
 -           
 -    |          
 -  _____| SLOPE     
 -                    
 -                       
 -                          
 -         TO THE RIGHT        
 x_________________________> DD (radius)  
 -                             
 |  \   -                          
 -                       
 |      \       -                    
 -                 
 |          \           -              
 -           
 |  BELOW       \               -        
 -     
 |                  \                       
                                                                       
 |                      \                       
                                                                      
 |                          \                       
 
 \endcode
*/
class DLLineOrientationCalculation {

   /*************************************************************************
   METHODS
   *************************************************************************/

   public:
   /** Constructor.  Because the public method is static, there is no need
    * to construct a DLLineOrientationCalculation object.
    */
   DLLineOrientationCalculation();

   /** Destructor. */
   virtual ~DLLineOrientationCalculation();

   /***********************************************************************
     GET METHODS
   ***********************************************************************/

   /**
    * Calculates the Line Orientation (Horizontal or Vertical) of the given image.
    * \param in the DLImage to process
    * \param fPercentDiff the % difference necessary to make the call, ie 5.0, 1.8, 20.4, etc.
    Default is <b>5.0</b>
    * \return 
    \arg ::DL_VERTICAL
    \arg ::DL_HORIZONTAL
    \arg ::DL_NO_DECISION (if % difference below fPerfectDiff)
    \arg ::DL_NOT_ENOUGH_LINKS (not enough links on page)
   */
   static int dlCalculateLineOrientation(const DLImage &in, float fPercentDiff = 5.0);

   /***********************************************************************
     SET METHODS
   ***********************************************************************/

   private:
   static void dlPutOnList(const vector<DLComponent> &CC, vector<DLnodes> &N, int i, int j, float SLOPE = .5);

    
   /***********************************************************************
   VARIABLES
   ***********************************************************************/

};


/***************************************************************************
 SUBROUTINES
***************************************************************************/

/***************************************************************************/
